/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
/* Eclipse Program Launcher
 *
 * This program performs the launching of the eclipse program along
 * with the splash window. If the splash window is displayed, the
 * java application will receive two extra arguments:
 *     -endsplash  <endSplashCommand>
 * The java program should be able to take the <endSplashCommand>
 * and simply execute it via Runtime.getRuntime().exec( command ).
 *
 * The Java application does not need to know the format of this
 * command but for the sake of this launcher, the command format is:
 *   <fullPathOfLauncherProgram> -endsplash <magicNumber>
 * In other words, another eclipse launcher program is run with
 * the arguments "-endsplash" and some magic number. On Windows,
 * the magic number corresponds to the window handle of the splash
 * window  and on Linux/UNIX, the magic number is the process ID.
 *
 * The launcher program starts a new process and executes a Java VM
 * with a set of arguments. The arguments to this VM are either
 * predefined (<wsSpecificArgs>), generated (e.g., -endsplash),
 * or are specified by the user <userArgs>. 
 * The predefined arguments include:
 *   -verify -classpath startup.jar org.eclipse.core.launcher.Main
 *   -ws win32|motif
 *
 * The actual VM used is either specified by the user, is shipped 
 * with eclipse (e.g., jre/bin/java*) or is in the user's PATH.
 *
 * The switches that can be specified by the user to the launcher:
 *  -vm <javaVM>               the Java VM to be used
 *  -application <class>       the starting workbench application class
 *  -nosplash                  do not display the splash screen
 *  -vmargs <userVMarg> ...    a list of arguments for the VM itself
 *
 * The -vmargs switch and all user specified VM arguments must appear
 * at the end of the command line, after all arguments that are
 * being passed to Java application. 
 *
 * The argument order for the new Java VM process is as follows:
 *
 * <javaVM> <userVMarg(s)> <wsSpecificArgs> -application <class>
 *     [-endsplash <endSplashCommand>] <userArgs>
 * 
 * If the -nosplash switch is given to the launcher, the java
 * application will not receive the "-endsplash" switch or its
 * associated command.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef _WIN32
#include <windows.h>
#include <direct.h>
#define getcwd _getcwd
#else /* Linux and UNIX */
#include <unistd.h>
#endif

#include "eclipseOS.h"

#define MAX_PATH_LENGTH   2000

/* Global Data */
static char*  program     = NULL;       /* full pathname of the program */
static char*  homeDir     = NULL;       /* directory where program resides */
static int    doSplash    = 1;              /* true: show the splash window */
static char*  javaVM      = NULL;       /* full pathname of the Java VM to run */
static char** userVMarg   = NULL;       /* user specific args for the Java VM  */
static char*  application  = "org.eclipse.ui.workbench";   /* default eclipse application */
static char*  endsplash   = NULL;       /* endsplash argument (eclipse parent info) */
static int    needConsole = 0;          /* True: eclipse application wants a console */


static void   parseArgs( int* argc, char* argv[] );
static char** getVMCommand( int argc, char* argv[], char* splashCmd );
static char*  findCommand( char* command );


int main( int argc, char* argv[] ){	
    char*   ch;
    char*   vmName;
    char*   shippedVM = NULL;
    char*   splashCmd = NULL; 
    char**  vmCommand;

	/* Strip off any extroneous <CR> from the last argument. If a shell script
	    on Linux is created in DOS format (lines end with <CR><LF>), the C-shell
	    does not strip off the <CR> and hence the argument is bogus and may 
	    not be recognized by the launcher or eclipse itself. */
	 ch = strchr( argv[ argc - 1 ], (int) '\r' );
	 if (ch != NULL) {
	     *ch = '\0';
	 }
	 
	/* Determine the full pathname of this program. */  
    program = findCommand( argv[0] );
#ifdef _WIN32
    if (strlen( program ) == 0) {
    	free( program );
    	program = malloc( MAX_PATH_LENGTH + 1 );
    	GetModuleFileName( NULL, program, MAX_PATH_LENGTH );
    }
#endif

    /* Find the home directory where the command resides. */
    homeDir = (char*) malloc( strlen( program ) + 1 );
    strcpy( homeDir, program );
    ch = strrchr( homeDir, (int)dirSeparator );
    if (ch != NULL) *(ch+1) = '\0';

    /* Parse command line arguments (looking for the VM to use). */
    parseArgs( &argc, argv );
    
    /* If the endsplash argument was given, destroy the splash
       window and quit. */
    if (endsplash != NULL) {
        destroySplash( endsplash );
        return 0;
    }

    /* If a VM was not specified as a command argument */
    if (javaVM == NULL){
    	/* Determine which type of VM should be used. */
    	vmName = needConsole ? consoleVM : defaultVM;
    	
        /* Try to find the VM shipped with eclipse. */
        shippedVM = malloc( strlen( homeDir ) + strlen( vmName ) + 10 );
 #ifdef J9VM
        sprintf( shippedVM, "%sive%cbin%c%s", homeDir, dirSeparator, dirSeparator, vmName );
 #else
        sprintf( shippedVM, "%sjre%cbin%c%s", homeDir, dirSeparator, dirSeparator, vmName );
 #endif
        javaVM = findCommand( shippedVM );

        /* If the VM was not shipped (or it was not found) */
        if (strlen( javaVM ) == 0){
            /* Try again looking for VM in the user's PATH. */
            free( javaVM );
            javaVM = findCommand( vmName );
        }
        
        /* If the default VM could not be found */
        if (strlen( javaVM ) == 0){
            /* Use the name of the shipped VM. An error should occur. */
            javaVM = shippedVM;
         }
        else {
        	free( shippedVM );
        	shippedVM = NULL;
        }
        
    }

    /* Create the splash screen and display it, if it is wanted. */
    if (doSplash){
    	splashCmd = createSplash( &argc, argv, homeDir );
    }

    /* Get the command to start the Java VM. */
    vmCommand = getVMCommand( argc, argv, splashCmd );
    startJavaVM( vmCommand );
    
    /* Do some preliminary cleanup. */
    free( homeDir );
    free( program );
    if (splashCmd  != NULL) free( splashCmd );
    if (shippedVM  != NULL) free( shippedVM );

    /* Process events while waiting for a stop indicator and/or the child to terminate. */
	processEvents();
    
    return 0;
}


/*
 * Parse arguments of the command.
 */
static void parseArgs( int* pArgc, char* argv[] ){
    int remArgs;
    int index;
    int i;
    
    /* Ensure the list of user argument is NULL terminated. */
    argv[ *pArgc ] = NULL;

	/* For each user defined argument (excluding the program) */
    for (index = 1; index < *pArgc; index++){
        remArgs = 0;
        
        if (strcmp( argv[ index ], "-vm" ) == 0 && (index+1) < *pArgc){
            javaVM = argv[ index+1 ];
            remArgs = 2;
        }
 
        /* All of the remaining arguments are user VM args. */    
        else if (strcmp( argv[ index ], "-vmargs" ) == 0){
            userVMarg = &argv[ index+1 ];
            argv[ index ] = NULL;
            *pArgc = index;
        }
       
        else if (strcmp( argv[ index ], "-application" ) == 0 && (index+1) < *pArgc){
            application = argv[ index+1 ];
            remArgs = 2;
        }

        else if (strcmp( argv[ index ], "-nosplash" ) == 0){
        	doSplash = 0;
            remArgs = 1;
        }

        else if (strcmp( argv[ index ], "-endsplash" ) == 0 && (index+1) < *pArgc){
        	endsplash = argv[ index+1 ];
            remArgs = 2;
        }

        else if (strcmp( argv[ index ], "-debug" ) == 0 || strcmp( argv[ index ], "-consoleLog" ) == 0){
        	needConsole = 1;
        }

		/* Remove any matched arguments from the list. */        
        if (remArgs > 0){
            for (i = (index + remArgs); i <= *pArgc; i++){
                argv[ i - remArgs ] = argv[ i ];
            }
            index--;
            *pArgc -= remArgs;
        }
    }
}
         
/*
 * Find the absolute pathname to where a command resides.
 * 
 * The string returned by the function must be freed.
 */
static char* findCommand( char* command ){ 
    char*  cmdPath;
    int    length;
    char*  ch;
    char*  dir;
    char*  path;
    struct stat stats;

    /* If the command was an abolute pathname, use it as is. */
    if (command[0] == dirSeparator || 
       (strlen( command ) > 2 && command[1] == ':')){
        length = strlen( command );
        cmdPath = malloc( length + 1 );
        strcpy( cmdPath, command );
    }

    else{
        /* If the command string contains a path separator */
        if (strchr( command, (int) dirSeparator ) != NULL){
            /* It must be relative to the current directory. */
            length = MAX_PATH_LENGTH + 1 + strlen( command );
            cmdPath = malloc( length + 1 );
            getcwd( cmdPath, length );
            if (cmdPath[ strlen( cmdPath ) - 1 ] != dirSeparator){
                length = strlen( cmdPath );
                cmdPath[ length ] = dirSeparator;
                cmdPath[ length+1 ] = '\0';
            }
            strcat( cmdPath, command );
        }

        /* else the command must be in the PATH somewhere */
        else{
            /* Get the directory PATH where executables reside. */
            path = getenv( "PATH" );
            length = strlen( path ) + 1 + strlen( command );
            cmdPath = malloc( length + 1 );

            /* Foreach directory in the PATH */
            dir = path;
            while (dir != NULL && *dir != '\0'){
                ch = strchr( dir, (int)pathSeparator );
                if (ch == NULL){
                    strcpy( cmdPath, dir );
                }
                else{
                    length = ch - dir;
                    strncpy( cmdPath, dir, length );
                    cmdPath[ length ] = '\0';
                    ch++;
                }
                dir = ch; /* advance for the next iteration */

                /* Determine if the executable resides in this directory. */
                if (cmdPath[ strlen( cmdPath ) - 1 ] != dirSeparator){
                    length = strlen( cmdPath );
                    cmdPath[ length ] = dirSeparator;
                    cmdPath[ length+1 ] = '\0';
                }
                strcat( cmdPath, command );
                
                /* If the file is not a directory and can be executed */
                if (stat( cmdPath, &stats ) == 0 && (stats.st_mode & S_IFREG) != 0){
                    /* Stop searching */
                    dir = NULL;
                }
            }
        }
    }

    /* Verify the resulting command actually exists. */
    if (stat( cmdPath, &stats ) != 0 || (stats.st_mode & S_IFREG) == 0){
        cmdPath[0] = '\0';
    }

    /* Return the absolute command pathname. */
    return cmdPath;
}



/*
 * Get the command and arguments to start the Java VM.
 *
 * Memory allocated by this function is assumed to be
 * deallocated when the program terminates.
 *
 * Some of the arguments returned by this function were
 * passed directly from the main( argv ) array so they 
 * should not be deallocated.
 */
static char** getVMCommand( int argc, char* argv[],char* splashCmd ){ 
    int    nArgVM = 0;
    int    nUserVMarg = 0;
    int    totalArgs;
    char** execArg;
    char*  jarFile = NULL;
    char*  splashExec;
    int    src;
    int    dst;
 
 	/* Calculate the number of user VM arguments and predefined VM arguments. */
 	if (userVMarg != NULL) {
	 	while (userVMarg[ nUserVMarg ] != NULL) {
 			nUserVMarg++;
 		}
 	}
 	while (argVM[ nArgVM ] != NULL) {
 		nArgVM++;
 	}
 		
    /* Allocate the arg list for the exec call.
       (VM + user VM args + VM args + -application X + -endsplash X + argv[] + NULL) */
    totalArgs  = 1 + nUserVMarg + nArgVM + 2 + 2 + argc + 1;
    execArg = (char**) malloc( totalArgs * sizeof( char* ) );
    dst = 0;
    execArg[ dst++ ] = javaVM;

    /* For each user specified argument to VM, add it to the exec arg list. */
    for (src = 0; src < nUserVMarg; src++){
	    execArg[ dst++ ] = userVMarg[ src ];
	}
    
    /* For each predefined argument to VM */
    for (src = 0; src < nArgVM; src++){
    	/* If the argument is not the startup jar, use it as is. */
     	if (strcmp( argVM[ src ], STARTUP_JAR_NAME ) != 0) {
	        execArg[ dst++ ] = argVM[ src ];
	    }
	    
	    /* else use the absolute path of the jar file. */
	    else {
			jarFile = malloc( strlen( homeDir ) + strlen( STARTUP_JAR_NAME ) + 1 );
    		strcpy( jarFile, homeDir );
    		strcat( jarFile, STARTUP_JAR_NAME );
	        execArg[ dst++ ] = jarFile;
	    }
    }

	/* Append the application to start. */
    execArg[ dst++ ] = "-application";
    execArg[ dst++ ] = application;
	
	/* Append the splash terminate command, if defined. */
    if (splashCmd != NULL){
        execArg[ dst++ ] = "-endsplash";
		splashExec = malloc( strlen( program ) + strlen( splashCmd ) + 20 );
		sprintf( splashExec, "%s -endsplash %s", program, splashCmd );
	    execArg[ dst++ ] = splashExec;
    }

	/* Append the remaining user defined argments. */
    for (src = 1; src < argc; src++){
        execArg[ dst++ ] = argv[ src ];
    }
    execArg[ dst++ ] = NULL;

	return execArg;
 }
         
