/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
/* UNIX/Motif specific logic for displaying the splash screen. */

#include "eclipseOS.h"
#include <Xm/Xm.h>
#include <Xm/Label.h>
#include <Xm/MwmUtil.h>
#include <Xm/MessageB.h>
#include <X11/Shell.h>

#include <sys/types.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <locale.h>

/* Global Variables */
char   dirSeparator = '/';
char   pathSeparator = ':';
#ifndef J9VM
char*  consoleVM = "java";
char*  defaultVM = "java";
char*  argVM[] =
{ "-classpath", STARTUP_JAR_NAME, "org.eclipse.core.launcher.Main",
  "-ws", "motif", NULL };
#else
char*  consoleVM = "j9";
char*  defaultVM = "j9";
char*  argVM[] =
{ "-jit", "-mca:1024", "-mco:1024", "-mn:256", "-mo:4096", "-moi:16384", "-mx:262144", "-ms:16", "-mr:16",
  "-cp", STARTUP_JAR_NAME, "org.eclipse.core.launcher.Main",
  "-ws", "motif", NULL };
#endif


/* Local variables */
static XtAppContext app;
static Widget topShell = 0;
static Widget shell    = 0;
static Pixmap splashPixmap = 0;
static char*  loResSplash = "splash_basic.xpm";
static char*  hiResSplash = "splash_full.xpm";
#define LO_RES_DEPTH 8

/* Local functions */
static void   centreShell( Widget widget, Widget expose );
static void   programIsReady( int );

/* Display an Error */

static void displayError( char* errorMsg ){
    char*         displayName = NULL;
    XtAppContext  context;
    Display*      display;
    Widget        msgParent;
    Widget        msgBox = 0;
    char*         argv[] = {};
    int           argc = 0;
    XmString      msg;
    Arg           arg[20];
    int           nArgs;
    XEvent        event;
   
    /* Determine the display name from the shell opened in the parent process. */
    if (topShell != 0) {
        displayName = DisplayString( XtDisplay( topShell ) );
    }

    /* Create a separate context, display and top shell for the message box.
     * The parent process's app-context, X display and shell cannot be used
     * in this child process because the event processing loop in the parent is
     * obtaining and dispatching events and an X error will occur if another 
     * process (the child) attempts to use the same display connection.
     */
    context   = XtCreateApplicationContext();
    display   = XtOpenDisplay( context, displayName, getOfficialName(), getOfficialName(), NULL, 0, &argc, argv );
    msgParent = XtAppCreateShell( getOfficialName(), getOfficialName(), applicationShellWidgetClass, display, NULL, 0 );

    /* Output a simple message box. */
    msg = XmStringCreateLocalized( errorMsg );
    nArgs = 0;
    XtSetArg( arg[ nArgs ], XmNdialogType, XmDIALOG_MESSAGE ); nArgs++;
    XtSetArg( arg[ nArgs ], XmNtitle, getOfficialName() ); nArgs++;
    XtSetArg( arg[ nArgs ], XmNmessageString, msg ); nArgs++;
    msgBox = XmCreateMessageDialog( msgParent, getOfficialName(), arg, nArgs );
    XtUnmanageChild( XmMessageBoxGetChild( msgBox, XmDIALOG_CANCEL_BUTTON ) );
    XtUnmanageChild( XmMessageBoxGetChild( msgBox, XmDIALOG_HELP_BUTTON ) );
    XtManageChild( msgBox );
    centreShell( msgBox, msgBox );

    /* Wait for the OK button to be pressed. */
    while (XtIsRealized( msgBox ) && XtIsManaged( msgBox )){
        XtAppNextEvent( context, &event );
        XtDispatchEvent( &event );
    }

    XtDestroyWidget( msgBox );
    XtDestroyWidget( msgParent );
    XtCloseDisplay( display );
}

/* Load the splash image depending on the current locale.
 *
 * The search sequence for a japanese locale (ja_JP.ujis) is:
 *
 *    1 - <homeDir>splash/ja_JP/<fileName>
 *    2 - <homeDir>splash/ja/<fileName>
 *    3 - <homeDir>splash/<fileName>
 *
 * Returns a Pixmap or XmUNSPECIFIED_PIXMAP if the file can not be found.
 */
Pixmap loadSplashImage( Screen* screen, char* homeDir, char* fileName ){
	Pixmap splashPixmap = XmUNSPECIFIED_PIXMAP;
    char* splashFile;
    char* locale;
    char* c;
    
    locale = strdup( setlocale ( LC_ALL, NULL ) );
    
    /* Remove the encoding portion if present. */
    c = strchr( locale, '.');
    if (c != NULL) c[0] = '\0';
	
    splashFile = (char*) malloc( strlen( homeDir ) + 256 );
    
    sprintf( splashFile, "%ssplash/%s/%s", homeDir, locale, fileName );
    splashPixmap = XmGetPixmap( screen, splashFile, 0, 0 );
    if (splashPixmap != XmUNSPECIFIED_PIXMAP) {
	    free( splashFile );
	    free( locale );
  		return splashPixmap;
    }
    
    /* Remove the country portion if present. */
    c = strchr( locale, '_');
    if (c != NULL) c[0] = '\0';
    sprintf( splashFile, "%ssplash/%s/%s", homeDir, locale, fileName );
    splashPixmap = XmGetPixmap( screen, splashFile, 0, 0 );
    if (splashPixmap != XmUNSPECIFIED_PIXMAP) {
	    free( splashFile );
	    free( locale );
  		return splashPixmap;
    }

    sprintf( splashFile, "%ssplash/%s", homeDir, fileName );
    splashPixmap = XmGetPixmap( screen, splashFile, 0, 0 );
    if (splashPixmap != XmUNSPECIFIED_PIXMAP) {
	    free( splashFile );
	    free( locale );
  		return splashPixmap;
    }

    free( splashFile );
    free( locale );
    return XmUNSPECIFIED_PIXMAP;
}

/* Create the Splash Window
 *
 * Initialize Xt, install a sigal handler and create an
 * overrideShell widget for the splash window. The splash termination
 * info returned defines the process ID of this launcher
 * program. To terminate the program, send (kill) a SIGUSR1
 * signal to the launcher program.
 */
char* createSplash( int* pArgc, char* argv[], char* homeDir ){
    Widget     label;
    Screen*    screen;
    Arg        arg[20];
    int        nArgs;
    int        depth;
    char*      result;
    char       buf[50];
    
    setlocale (LC_ALL, "");

    /* Create the top level shell that will not be used other than
       to initialize the application. */
    topShell = XtAppInitialize( &app, getOfficialName(), NULL, 0,
                                pArgc, argv, NULL, NULL, 0 ); 

    /* Install a signal handler to catch SIGUSR1. */
    signal( SIGUSR1, programIsReady );
    
   
    /* Create an application shell with no decorations.
     * Do not use an overrideShell because it is not known to the
     * window manager. As a result, when the error message box is
     * displayed on top of the splash window (overrideShell) button
     * events in the message box are sent to the splash window instead.
     * On Linux, this brings the splash window on top of the message
     * box which can no longer be accessed or terminated.
     */
    nArgs = 0;
    XtSetArg( arg[ nArgs ], XmNborderWidth, 0 ); nArgs++;
    XtSetArg( arg[ nArgs ], XmNmappedWhenManaged, False ); nArgs++;
    XtSetArg( arg[ nArgs ], XmNmwmDecorations, 0 ); nArgs++;
    XtSetArg( arg[ nArgs ], XmNmwmFunctions, MWM_FUNC_MOVE ); nArgs++;
    shell = XtCreateWidget( getOfficialName(), transientShellWidgetClass, topShell, arg, nArgs );

    /* Load the splash image. */
    screen = XDefaultScreenOfDisplay( XtDisplay( topShell ) );
    depth = XDefaultDepthOfScreen ( screen );
    splashPixmap = loadSplashImage (screen, homeDir, (depth <= LO_RES_DEPTH ? loResSplash : hiResSplash) );

    /* Create a label widget (only child of shell) with the pixmap. */
    nArgs = 0;
    if (splashPixmap != XmUNSPECIFIED_PIXMAP){
        XtSetArg( arg[ nArgs ], XmNlabelType, XmPIXMAP ); nArgs++;
        XtSetArg( arg[ nArgs ], XmNlabelPixmap, splashPixmap ); nArgs++;
    }
    XtSetArg( arg[ nArgs ], XmNmarginHeight, 0 ); nArgs++;
    XtSetArg( arg[ nArgs ], XmNmarginWidth,  0 ); nArgs++;
    label = XmCreateLabel( shell, getOfficialName(), arg, nArgs );
    XtManageChild( label );

    /* Centre the splash screen and display it. */
    centreShell( shell, label );

    /* Use the process Id as the splash termination information. */
    sprintf( buf, "%u", getpid() );
    result = (char*) malloc( strlen(buf) + 1 );
    strcpy( result, buf );
    return result;
}

/* Destroy the Splash Window 
 * 
 * This function will be called in a separate process from the process that
 * is displaying the splash screen. By sending a SIGUSR1 signal to that process,
 * it will exit its event loop and will destroy the splash window. 
 */
void destroySplash( char* splashId ) {
	pid_t  pid;
	
	if (splashId != NULL && strlen( splashId ) > 0 && 
	    sscanf( splashId, "%u", &pid ) == 1) {
		kill( pid, SIGUSR1 );
	}
}


/* Start Java command */
void startJavaVM( char* args[] ) {
	char* errnoMsg;
	char* errMsg;
	pid_t child;

#ifdef DEBUG_SPAWN
	int index;
	printf( "Start VM: %s\n", args[0] );
	for (index = 0; args[index] != NULL; index++) {
		printf( "args[%d] = '%s'\n", index, args[index] );
	}
#endif
	
	child = fork();
	if (child == 0) {
		execv( args[0], args );
		errnoMsg = strerror( errno );
		errMsg = (char*)malloc( strlen( errnoMsg ) + strlen( args[0] ) + 40 );
		sprintf( errMsg, "%s\n%s", errnoMsg, args[0] );

                /* If the splash screen was not displayed, print the message. */
                if (shell == 0) {
                    fprintf( stderr, "%s\n", errMsg );
                }
                /* else display a message box with the error, then kill the parent process. */
                else {
                    displayError( errMsg );
                    kill( getppid(), SIGUSR1 );
                }
                exit( 1 );
	}
}

/* Process Events */
void processEvents(){
    XEvent    event;

	/* If the splash window was created */
	if (shell != 0) {
		/* Process events while waiting for the real eclipse program to be ready. */
 		while (XtIsRealized( shell )){
  	    	XtAppNextEvent( app, &event );
        	XtDispatchEvent( &event );
    	}

    	/* Destroy the splash window. */
    	XtDestroyWidget( shell );
    	shell = 0;
    	
    	/* Destroy the pixmap. */
    	if (splashPixmap != XmUNSPECIFIED_PIXMAP) {
    		XFreePixmap( XtDisplay( topShell), splashPixmap );
    	}
    }
    
    /* Wait for the children to be terminated. */
    wait( NULL );      
}

/*------ local functions -----*/

/* Catch a signal that indicates the Java program is ready. */
static void   programIsReady( int sig ){
    if (shell != 0) {
        XtUnrealizeWidget( shell );
        XFlush( XtDisplay( shell ) );
    }
}


/* Centre the shell on the screen. */

static void centreShell( Widget widget, Widget expose ){
    XtAppContext context;
    XEvent       event;
    Arg   arg[20];
    int   nArgs;
    Position  x, y;
    Dimension width, height;
    Screen*   screen;
    int       waiting;

    /* Realize the shell to calculate its width/height. */
    XtRealizeWidget( widget );

    /* Get the desired dimensions of the shell. */
    nArgs = 0;
    XtSetArg( arg[ nArgs ], XmNwidth,  &width  ); nArgs++;
    XtSetArg( arg[ nArgs ], XmNheight, &height ); nArgs++;
    XtSetArg( arg[ nArgs ], XmNscreen, &screen ); nArgs++;
    XtGetValues( widget, arg, nArgs );

    /* Calculate the X and Y position for the shell. */
    x = (screen->width - width) / 2;
    y = (screen->height - height) / 2;

    /* Set the new shell position and display it. */
    nArgs = 0;
    XtSetArg( arg[ nArgs ], XmNx, x ); nArgs++;
    XtSetArg( arg[ nArgs ], XmNy, y ); nArgs++;
    XtSetValues( widget, arg, nArgs );
    XtMapWidget( widget );

    /* Wait for an expose event on the desired widget. This wait loop is required when
     * the startVM command fails and the message box is created before the splash
     * window is displayed. Without this wait, the message box sometimes appears
     * under the splash window and the user cannot see it.
     */
    context = XtWidgetToApplicationContext( widget );
    waiting = True;
    while (waiting) {
        XtAppNextEvent( context, &event );
        if (event.xany.type == Expose && event.xany.window == XtWindow( expose )) {
            waiting = False;
        }
        XtDispatchEvent( &event );
    }
    XFlush( XtDisplay( widget ) );
}

