/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
/* Windows specific logic for displaying the splash screen. */

#include "eclipseOS.h"

#include <windows.h>
#include <process.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

/* Global Variables */
char   dirSeparator = '\\';
char   pathSeparator = ';';
char*  consoleVM = "java.exe";
char*  defaultVM = "javaw.exe";
char*  argVM[] =
{ "-classpath", STARTUP_JAR_NAME, "org.eclipse.core.launcher.Main",
  "-ws", "win32", NULL };


/* Local variables */
static HWND    topWindow = 0;
static WNDPROC oldProc;
static char*  loResSplash = "splash_basic.bmp";
static char*  hiResSplash = "splash_full.bmp";
#define LO_RES_DEPTH 8


/* Local functions */
static LRESULT WINAPI WndProc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch (uMsg) {
		case WM_NCHITTEST: return HTCLIENT;
		case WM_CLOSE: 
	    	PostQuitMessage(  0 );
	    	break;
	}
	return CallWindowProc (oldProc, hwnd, uMsg, wParam, lParam);
}

/* Load the splash image depending on the current locale.
 *
 * This is the search sequence for an USA english locale:
 * 
 *     1 - <homeDir>splash\en_US\<fileName>
 *     2 - <homeDir>splash\en\<fileName>
 *     3 - <homeDir>splash\<fileName>
 *
 * Returns an HBITMAP or NULL if the file can not be found.
 */
HBITMAP loadSplashImage(char *homeDir, char *fileName) {
	HBITMAP hBitmap = NULL;
	LCID locale;
	char lang[4] = {0};
	char country[4] = {0};
	char *splashFile;

	locale = GetThreadLocale ();
	GetLocaleInfo(locale, LOCALE_SISO639LANGNAME, (LPTSTR) lang, 4);
	GetLocaleInfo(locale, LOCALE_SISO3166CTRYNAME, (LPTSTR) country, 4);
	
	splashFile = malloc ( strlen( homeDir ) + 256 );

    sprintf( splashFile, "%ssplash\\%s_%s\\%s", homeDir, lang, country, fileName );
	hBitmap = LoadImage (NULL, splashFile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	if (hBitmap != NULL) {
		free (splashFile);
		return hBitmap;
	}
	
	sprintf( splashFile, "%ssplash\\%s\\%s", homeDir, lang, fileName );
	hBitmap = LoadImage (NULL, splashFile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	if (hBitmap != NULL) {
		free (splashFile);
		return hBitmap;
	}
    
	sprintf( splashFile, "%ssplash\\%s", homeDir, fileName );
	hBitmap = LoadImage (NULL, splashFile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	if (hBitmap != NULL) {
		free (splashFile);
		return hBitmap;
	}

	free (splashFile);
	return NULL;
}

/* Create the Splash Window
 *
 * Create a popup window to display the bitmap image.
 *
 * Return the window's handle so that the termination
 * code in the Java program can send a WM_QUIT message
 * to the window to close it.
 *
 * The string returned from this function is eventually passed
 * as an argument to the destroySplash function.
 */
char* createSplash( int* pArgc, char* argv[], char* homeDir ){
    RECT    rect;
    HBITMAP hBitmap;
    HDC         hDC;
    int      depth;
    int     x, y;
    int     width, height;
    char*   result;
    char    buf[50];    
    
    /* Create a window that has no decorations. */
    topWindow = CreateWindowEx (0,
		"STATIC",
		(char *) getOfficialName(),
		SS_BITMAP | WS_POPUP,
		0,
		0,
		0,
		0,
		NULL,
		NULL,
		GetModuleHandle (NULL),
		NULL);
    oldProc = (WNDPROC) GetWindowLong (topWindow, GWL_WNDPROC);
    SetWindowLong (topWindow, GWL_WNDPROC, (LONG) WndProc);

    /* Load and set the bitmap in the window. */
    hDC = GetDC( NULL);
    depth = GetDeviceCaps( hDC, BITSPIXEL ) * GetDeviceCaps( hDC, PLANES);
    ReleaseDC(NULL, hDC); 
    hBitmap = loadSplashImage (homeDir, depth <= LO_RES_DEPTH ? loResSplash : hiResSplash );
    SendMessage (topWindow, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) hBitmap);

    /* Centre the splash screen and display it. */
    GetWindowRect (topWindow, &rect);
    width = GetSystemMetrics (SM_CXSCREEN);
    height = GetSystemMetrics (SM_CYSCREEN);
    x = (width - (rect.right - rect.left)) / 2;
    y = (height - (rect.bottom - rect.top)) / 2;
    SetWindowPos (topWindow, 0, x, y, 0, 0, SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE);
    ShowWindow (topWindow, SW_SHOW);

    /* Send the window handle to the Java program. */
    sprintf( buf, "%u", topWindow );
    result = (char*) malloc( strlen(buf) + 1 );
    strcpy( result, buf );
    return result;
}


/* Destroy the Splash Window */
void destroySplash( char* splashId ) {
	HWND  hwnd;
	
	if (splashId != NULL && strlen( splashId ) > 0 && 
	    sscanf( splashId, "%u", &hwnd ) == 1) {
	    PostMessage( hwnd, WM_QUIT, 0, 0 );
	}
}


/* Start the Java VM */
void startJavaVM( char* args[] ) {
	int   index;
	int   pid;
	char* javaVM = args[0];
	char* newArg;
	char* errnoMsg;
	char* errMsg;
	
	/* Any argument with spaces must be in double quotes. */
	for (index = 0; args[index] != NULL; index++) {
		if (strchr( args[ index ], (int) ' ' ) != NULL) {
			newArg = malloc( strlen( args[ index ] ) + 3 );
			sprintf( newArg, "\"%s\"", args[ index ] );
			args[ index ] = newArg;
		}
	}

#ifdef DEBUG_SPAWN
	printf( "Start VM: %s\n", javaVM );
	for (index = 0; args[index] != NULL; index++) {
		printf( "args[%d] = '%s'\n", index, args[index] );
	}
#endif

	pid = _spawnv( _P_NOWAIT, javaVM, args );
	
	if (pid == -1) {
		errnoMsg = strerror( errno );
		errMsg = malloc( strlen( errnoMsg ) + strlen( javaVM ) + 40 );
		sprintf( errMsg, "%s\n%s", errnoMsg, javaVM );
		MessageBox( topWindow, errMsg, getOfficialName(), MB_OK );
		PostQuitMessage( 1 );
	}
}
 

/* Process Events */
void processEvents(){
    MSG     msg;

	/* If the splash window was created */
	if (topWindow != 0) {
    	/* Process messages until the window is closed. */
   		while (GetMessage (&msg, NULL, 0, 0)) {
			TranslateMessage (&msg);
			DispatchMessage (&msg);
		}
    }
}


