package org.eclipse.core.internal.resources;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.core.internal.localstore.CoreFileSystemLibrary;
import org.eclipse.core.internal.utils.Assert;
import org.eclipse.core.internal.utils.Policy;
import java.io.InputStream;
import java.io.ByteArrayInputStream;

public class File extends Resource implements IFile {

	// default is true so we try/initialize validator first time through
	private static boolean validateSave = true;
	private static IValidator validator = null;
	private static final String PT_INTERNAL_FILE_VALIDATOR = "internalFileValidator";

protected File(IPath path, Workspace container) {
	super(path, container);
}
/**
 * @see IFile
 */
public void appendContents(InputStream content, boolean force, boolean keepHistory, IProgressMonitor monitor) throws CoreException {
	monitor = Policy.monitorFor(monitor);
	try {
		String message = Policy.bind("resources.settingContents", getFullPath().toString());
		monitor.beginTask(message, Policy.totalWork);
		Assert.isNotNull(content, "Content cannot be null.");
		try {
			workspace.prepareOperation();
			ResourceInfo info = getResourceInfo(false, false);
			checkAccessible(getFlags(info));

			workspace.beginOperation(true);
			validateSave();
			internalSetContents(content, force, keepHistory, true, Policy.subMonitorFor(monitor, Policy.opWork));
		} catch (OperationCanceledException e) {
			workspace.getWorkManager().operationCanceled();
			throw e;
		} finally {
			workspace.endOperation(true, Policy.subMonitorFor(monitor, Policy.buildWork));
		}
	} finally {
		monitor.done();
	}
}
/**
 * Changes this file to be a folder in the resource tree and returns
 * the newly created folder.  All related
 * properties are deleted.  It is assumed that on disk the resource is
 * already a folder/directory so no action is taken to delete the disk
 * contents.
 * <p>
 * <b>This method is for the exclusive use of the local resource manager</b>
 *
 * @see FileSystemResourceManager#reportChanges
 */
public IFolder changeToFolder() throws CoreException {
	getPropertyManager().deleteProperties(this);
	workspace.deleteResource(this);
	IFolder result = workspace.getRoot().getFolder(path);
	workspace.createResource(result, false);
	return result;
}
/**
 * @see IFile
 */
public void create(InputStream content, boolean force, IProgressMonitor monitor) throws CoreException {
	monitor = Policy.monitorFor(monitor);
	try {
		String message = Policy.bind("resources.creating", getFullPath().toString());
		monitor.beginTask(message, Policy.totalWork);
		checkValidPath(path, FILE);
		try {
			workspace.prepareOperation();
			checkDoesNotExist();
			Container parent = (Container) getParent();
			ResourceInfo info = parent.getResourceInfo(false, false);
			parent.checkAccessible(getFlags(info));

			workspace.beginOperation(true);
			IPath location = getLocalManager().locationFor(this);
			java.io.File localFile = location.toFile();
			if (force) {
				if (!CoreFileSystemLibrary.isCaseSensitive()) {
					if (localFile.exists()) {
						String name = getLocalManager().getLocalName(localFile);
						if (localFile.getName().equals(name)) {
							delete(true, null);
						} else {
							// The file system is not case sensitive and there is already a file
							// under this location.
							String msg = Policy.bind("resources.existsDifferentCase", location.removeLastSegments(1).append(name).toOSString());
							throw new ResourceException(IResourceStatus.FAILED_WRITE_LOCAL, getFullPath(), msg, null);
						}
					}
				}
			} else {
				if (localFile.exists()) {
					String msg = Policy.bind("resources.fileExists", localFile.getAbsolutePath());
					throw new ResourceException(IResourceStatus.FAILED_WRITE_LOCAL, getFullPath(), msg, null);
				}
			}
			monitor.worked(Policy.opWork * 40 / 100);

			workspace.createResource(this, false);
			boolean local = content != null;
			if (local) {
				try {
					internalSetContents(content, force, false, false, Policy.subMonitorFor(monitor, Policy.opWork * 40 / 100));
				} catch (CoreException e) {
					// a problem happened creating the file on disk, so delete from the workspace
					workspace.deleteResource(this);
					throw e; // rethrow
				}
			}
			setLocal(local, DEPTH_ZERO, Policy.subMonitorFor(monitor, Policy.opWork * 20 / 100));
			if (!local)
				getResourceInfo(true, true).setModificationStamp(IResource.NULL_STAMP);
		} catch (OperationCanceledException e) {
			workspace.getWorkManager().operationCanceled();
			throw e;
		} finally {
			workspace.endOperation(true, Policy.subMonitorFor(monitor, Policy.buildWork));
		}
	} finally {
		monitor.done();
	}
}
/**
 * @see IFile#getContents
 */
public InputStream getContents() throws CoreException {
	return getContents(false);
}
/**
 * @see IFile
 */
public InputStream getContents(boolean force) throws CoreException {
	ResourceInfo info = getResourceInfo(false, false);
	int flags = getFlags(info);
	checkAccessible(flags);
	checkLocal(flags, DEPTH_ZERO);
	return getLocalManager().read(this, force, null);
}
/**
 * @see IFile#getHistory
 */
public IFileState[] getHistory(IProgressMonitor monitor) throws CoreException {
	// FIXME: monitor is not used
	return getLocalManager().getHistoryStore().getStates(getFullPath());
}
public int getType() {
	return FILE;
}
/**
 * A validator hasn't been initialized. Check the extension point and try to create
 * a new validator if a user has one defined as an extension.
 */
private void initializeValidator() throws CoreException {
	IConfigurationElement[] configs = Platform.getPluginRegistry().getConfigurationElementsFor(ResourcesPlugin.PI_RESOURCES, PT_INTERNAL_FILE_VALIDATOR);
	// no-one is plugged into the extension point so disable validation
	if (configs == null || configs.length == 0) {
		validateSave = false;
		return;
	}
	// can only have one defined at a time. log a warning, disable validation, but continue with
	// the #setContents (e.g. don't throw an exception)
	if (configs.length > 1) {
		validateSave = false;
		IStatus status = new ResourceStatus(IResourceStatus.WARNING, Policy.bind("resources.oneValidator"));
		ResourcesPlugin.getPlugin().getLog().log(status);
		return;
	}
	// otherwise we have exactly one validator extension. Try to create a new instance 
	// from the user-specified class.
	try {
		IConfigurationElement config = configs[0];
		validator = (IValidator) config.createExecutableExtension("class");
	} catch (CoreException e) {
		throw new ResourceException(IResourceStatus.ERROR, null, Policy.bind("resources.initValidator"), e);
	}
}
protected void internalSetContents(InputStream content, boolean force, boolean keepHistory, boolean append, IProgressMonitor monitor) throws CoreException {
	if (content == null)
		content = new ByteArrayInputStream(new byte[0]);
	getLocalManager().write(this, content, force, keepHistory, append, monitor);
	ResourceInfo info = getResourceInfo(false, true);
	info.incrementContentId();
	workspace.updateModificationStamp(info);
}
/**
 * @see IFile
 */
public void setContents(InputStream content, boolean force, boolean keepHistory, IProgressMonitor monitor) throws CoreException {
	monitor = Policy.monitorFor(monitor);
	try {
		String message = Policy.bind("resources.settingContents", getFullPath().toString());
		monitor.beginTask(message, Policy.totalWork);
		try {
			workspace.prepareOperation();
			ResourceInfo info = getResourceInfo(false, false);
			checkAccessible(getFlags(info));

			workspace.beginOperation(true);
			validateSave();
			internalSetContents(content, force, keepHistory, false, Policy.subMonitorFor(monitor, Policy.opWork));
		} catch (OperationCanceledException e) {
			workspace.getWorkManager().operationCanceled();
			throw e;
		} finally {
			workspace.endOperation(true, Policy.subMonitorFor(monitor, Policy.buildWork));
		}
	} finally {
		monitor.done();
	}
}
/**
 * @see IFile#setContents
 */
public void setContents(IFileState source, boolean force, boolean keepHistory, IProgressMonitor monitor) throws CoreException {
	setContents(source.getContents(), force, keepHistory, monitor);
}
/**
 * Internal method. To be called only from the following methods:
 * <ul>
 * <li><code>IFile#appendContents</code></li>
 * <li><code>IFile#setContents(InputStream, boolean, boolean, IProgressMonitor)</code></li>
 * <li><code>IFile#setContents(IFileState, boolean, boolean, IProgressMonitor)</code></li>
 * </ul>
 * 
 * @see IValidator#validateSave
 */
private void validateSave() throws CoreException {
	// if validation is turned off then just return
	if (!validateSave)
		return;
	// first time through the validator hasn't been initialized so try and create it
	if (validator == null) 
		initializeValidator();
	// we were unable to initialize the validator. Validation has been turned off and 
	// a warning has already been logged so just return.
	if (validator == null)
		return;
	// otherwise call the API and throw an exception if appropriate
	final IStatus[] status = new IStatus[1];
	ISafeRunnable body = new ISafeRunnable() {
		public void run() throws Exception {
			status[0] = validator.validateSave(File.this);
		}
		public void handleException(Throwable exception) {
			status[0]  = new ResourceStatus(IResourceStatus.ERROR, null, Policy.bind("resources.errorValidator"), exception);
		}
	};
	Platform.run(body);
	if (!status[0].isOK())
		throw new ResourceException(status[0]);
}
}
