package org.eclipse.jdt.internal.core.refactoring;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IPath;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.core.refactoring.base.Refactoring;
import org.eclipse.jdt.internal.core.refactoring.base.RefactoringStatus;

import org.eclipse.jdt.internal.compiler.AbstractSyntaxTreeVisitorAdapter;
import org.eclipse.jdt.internal.compiler.ast.AstNode;
import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
import org.eclipse.jdt.internal.compiler.ast.QualifiedNameReference;
import org.eclipse.jdt.internal.compiler.ast.QualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.CompilationUnitScope;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.problem.ProblemHandler;
import org.eclipse.jdt.internal.compiler.util.CharOperation;
import org.eclipse.jdt.internal.core.CompilationUnit;

public abstract class RefactoringASTAnalyzer extends AbstractRefactoringASTAnalyzer{

	private List fSearchResults;
	
	public final RefactoringStatus analyze(List searchResults, ICompilationUnit cu) throws JavaModelException{
		fSearchResults= searchResults;
		return super.analyze(cu);
	}
	//------- type member analysis
	
	/* non java-doc
	 * override to limit analysis
	 */ 
	protected boolean checkFields(){
		return true;
	}
	/* non java-doc
	 * override to limit analysis
	 */ 
	protected boolean checkMemberTypes(){
		return true;
	}
	/* non java-doc
	 * override to limit analysis
	 */ 
	protected boolean checkSuperClasses(){
		return true;
	}
	/* non java-doc
	 * override to limit analysis
	 */ 
	protected boolean checkSuperInterfaces(){
		return true;
	}
	// --- other helper methods ---------
	/*
	 * override at will
	 */
	protected boolean nameDefinedInScope(char[] newName, BlockScope scope){
		if (scope == null)
			return false;
		if (scope.locals != null){
			for (int i= 0; i < scope.locals.length; i++){
				if (scope.locals[i] != null){
					char[] name= (scope.locals[i].name != null) ? scope.locals[i].name : new char[0];
					if (CharOperation.equals(newName, name))
						return true;
				}	
			}
		}
		if (scope.parent instanceof BlockScope)
			return nameDefinedInScope(newName, (BlockScope)scope.parent);
		else if (scope.parent instanceof ClassScope)
			return nameDefinedInScope(newName, (ClassScope)scope.parent);
		else 
			return false;
	}
	/*
	 * override at will
	 */
	protected boolean nameDefinedInScope(char[] newName, ClassScope scope){
		if (scope == null)
			return false;
		if (nameDefinedInType(newName, scope.referenceContext.binding))
			return true;
		if (scope.parent instanceof ClassScope)	
			return nameDefinedInScope(newName, (ClassScope)scope.parent);
		else if (scope.parent instanceof BlockScope)	
			return nameDefinedInScope(newName, (BlockScope)scope.parent);
		else if (scope.parent instanceof CompilationUnitScope)
			return nameDefinedInScope(newName, (CompilationUnitScope)scope.parent);
		else	
			return false;
	}
	protected boolean nameDefinedInScope(char[] newName, CompilationUnitScope scope){
		if (scope.topLevelTypes == null)
			return false;
		for (int i= 0; i < scope.topLevelTypes.length; i++){
			if (CharOperation.equals(newName, scope.topLevelTypes[i].sourceName))
				return true;
		}		
		return false;
	}
	protected boolean nameDefinedInType(char[] newName, SourceTypeBinding sourceTypeBinding){
		if (checkFields() && sourceTypeBinding.fields != null){
			for (int i= 0; i < sourceTypeBinding.fields.length; i++){
				if (CharOperation.equals(newName, sourceTypeBinding.fields[i].name))
					return true;
			}
		}
		if (checkMemberTypes() && sourceTypeBinding.memberTypes != null){
			for (int i= 0; i < sourceTypeBinding.memberTypes.length; i++){
				if (CharOperation.equals(newName, sourceTypeBinding.memberTypes[i].sourceName))
					return true;
			}
		}
		
		if (	checkSuperClasses()
			&& (sourceTypeBinding.superclass != null)
			&& (sourceTypeBinding.superclass instanceof SourceTypeBinding)
			&& nameDefinedInType(newName, (SourceTypeBinding)sourceTypeBinding.superclass))
				return true;
				
		if (checkSuperInterfaces() && sourceTypeBinding.superInterfaces != null){
			for (int i= 0; i < sourceTypeBinding.superInterfaces.length; i++){
				if ((sourceTypeBinding.superInterfaces[i] instanceof SourceTypeBinding)
					&& nameDefinedInType(newName, (SourceTypeBinding)sourceTypeBinding.superInterfaces[i]))
						return true;
			}
		}
		
		return false;
	}
	//-------
		
	protected boolean sourceRangeOnList(int start, int end){
		//DebugUtils.dump("start:" + start + " end:" + end);
		Iterator iter= fSearchResults.iterator();
		while (iter.hasNext()){
			SearchResult searchResult= (SearchResult)iter.next();
			//DebugUtils.dump("[" + searchResult.getStart() + ", " + searchResult.getEnd() + "]");
			if (start == searchResult.getStart() && end == searchResult.getEnd())
				return true;
		}
		return false;
	}
	protected boolean sourceRangeOnList(AstNode astNode){		
		if (astNode instanceof QualifiedNameReference)
			return sourceRangeOnList((QualifiedNameReference)astNode);
		else if (astNode instanceof QualifiedTypeReference)
			return sourceRangeOnList((QualifiedTypeReference)astNode);
		
		return sourceRangeOnList(astNode.sourceStart, astNode.sourceEnd + 1);
	}
	protected boolean sourceRangeOnList(QualifiedNameReference qualifiedNameReference){
		int start= qualifiedNameReference.sourceStart;
		return sourceRangeOnList(start, start + qualifiedNameReference.tokens[0].length);
	}
	protected boolean sourceRangeOnList(QualifiedTypeReference qualifiedTypeReference){
		int start= qualifiedTypeReference.sourceStart;
		return sourceRangeOnList(start, start + qualifiedTypeReference.tokens[0].length);
	}
}
