package org.eclipse.jdt.internal.ui.typehierarchy;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.IBasicPropertyConstants;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;

import org.eclipse.jdt.core.ElementChangedEvent;
import org.eclipse.jdt.core.IElementChangedListener;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaElementDelta;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.ITypeHierarchy;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;

import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.codemanipulation.StubUtility;
import org.eclipse.jdt.internal.ui.util.ArrayUtility;
import org.eclipse.jdt.internal.ui.util.JavaModelUtility;

/**
 * Base class for content providers for type hierarchy viewers.
 * Implementors must override 'getTypesInHierarchy'.
 * Java delta processing is also performed by the content provider
 */
public abstract class TypeHierarchyContentProvider implements ITreeContentProvider, IElementChangedListener {

	protected static final String[] UNSTRUCTURED= new String[] { IBasicPropertyConstants.P_TEXT, IBasicPropertyConstants.P_IMAGE };

	protected TypeHierarchyLifeCycle fTypeHierarchy;
	protected IMember[] fMemberFilter;
	protected boolean fShowAllTypes;
	
	protected TreeViewer fViewer;
	
	public TypeHierarchyContentProvider(TypeHierarchyLifeCycle lifecycle) {
		fTypeHierarchy= lifecycle;
		fMemberFilter= null;
		fShowAllTypes= false;
	}
	protected void addFilteredMembers(IType parent, List children) {
		try {
			IMethod[] methods= parent.getMethods();
			for (int i= 0; i < fMemberFilter.length; i++) {
				IMember member= fMemberFilter[i];
				if (parent.equals(member.getDeclaringType())) {
					children.add(member);
				} else if (member instanceof IMethod) {
					IMethod meth= StubUtility.findMethod((IMethod)member, methods);
					if (meth != null) {
						children.add(meth);
					}
				}
			}		
		} catch (JavaModelException e) {
			// ignore
		}
	}
	protected void addFilteredTypes(IType[] types, List children) {
		try {
			for (int i= 0; i < types.length; i++) {
				if (hasFilteredChildren(types[i])) {
					children.add(types[i]);
				}
			}
		} catch (JavaModelException e) {
			// ignore
		}
	}
	/**
	 * @see IContentProvider#dispose
	 */	
	public void dispose() {
		// just to be sure
		JavaCore.removeElementChangedListener(this);
	}
	/**
	 * @see IElementChangedListener#elementChanged
	 */
	public void elementChanged(ElementChangedEvent event) {
		if (getHierarchy() != null) {
			try {
				processDelta(event.getDelta());
			} catch(JavaModelException e) {
				JavaPlugin.getDefault().getLog().log(e.getStatus());
			}
		}
	}
	/**
	 * Called for the tree children
	 * @see ITreeContentProvider#getChildren
	 */	
	public Object[] getChildren(Object element) {
		if (element instanceof IType) {
			IType type= (IType)element;
			IType[] childrenTypes= getTypesInHierarchy(type);
				
			if (fMemberFilter != null) {
				List children= new ArrayList();
				addFilteredMembers(type, children);
				addFilteredTypes(childrenTypes, children);
				return children.toArray();
			} else {
				return childrenTypes;
			}			
		}
		return ArrayUtility.getEmptyArray();
	}
	/**
	 * Called for the root element
	 * @see IStructuredContentProvider#getElements	 
	 */
	public Object[] getElements(Object parent) {
		IType input= getInputType();
		if (input == null) {
			return ArrayUtility.getEmptyArray();
		} else {
			if (fMemberFilter != null) {
				try {
					if (!hasFilteredChildren(input)) {
						return ArrayUtility.getEmptyArray();
					}
				} catch (JavaModelException e) {
					// ignore
				}
			}		 	
			return new IType[] { input };
		}
	}
	protected final ITypeHierarchy getHierarchy() {
		return fTypeHierarchy.getHierarchy();
	}
	protected final IType getInputType() {
		return fTypeHierarchy.getInput();
	}
	public IMember[] getMemberFilter() {
		return fMemberFilter;
	}
	/**
	 * @see ITreeContentProvider#getParent
	 */
	public Object getParent(Object element) {
		return null;
	}
	/**
	 * Hook to overwrite. Filter will be applied on the returned types
	 */	
	protected IType[] getTypesInHierarchy(IType type) {
		return new IType[0];
	}
	/**
	 * @see ITreeContentProvider#hasChildren
	 */
	public boolean hasChildren(Object element) {
		if (element instanceof IType) {
			IType type= (IType)element;
			IType[] childrenTypes= getTypesInHierarchy(type);
				
			if (fMemberFilter != null) {
				List children= new ArrayList();
				addFilteredMembers(type, children);
				addFilteredTypes(childrenTypes, children);
				return children.size() > 0;
			} else {
				return childrenTypes.length > 0;
			}				
		}
		return false;
	}
	protected boolean hasFilteredChildren(IType type) throws JavaModelException {
		if (fShowAllTypes) {
			return true;
		}
		
		IMethod[] methods= type.getMethods();
		for (int i= 0; i < fMemberFilter.length; i++) {
			IMember member= fMemberFilter[i];
			if (type.equals(member.getDeclaringType())) {
				return true;
			} else if (member instanceof IMethod) {
				IMethod meth= StubUtility.findMethod((IMethod)member, methods);
				if (meth != null) {
					return true;
				}
			}
		}
		IType[] childrenTypes= getTypesInHierarchy(type);
		for (int i= 0; i < childrenTypes.length; i++) {
			if (hasFilteredChildren(childrenTypes[i])) {
				return true;
			}
		}
		return false;
	}
	/**
	 * @see IContentProvider#inputChanged
	 */
	public void inputChanged(Viewer part, Object oldInput, Object newInput) {
		if (part instanceof TreeViewer) {
			fViewer= (TreeViewer)part;
		} else {
			fViewer= null;
		}
		if (oldInput == null && newInput != null) {
			JavaCore.addElementChangedListener(this); 
		} else if (oldInput != null && newInput == null) {
			JavaCore.removeElementChangedListener(this); 
		}
	}
	/**
	 * @see IContentProvider#isDeleted
	 */
	public boolean isDeleted(Object obj) {
		try {
			if (obj instanceof IJavaElement) {
				IJavaElement elem= (IJavaElement)obj;
				return !(elem.exists() && JavaModelUtility.isOnBuildPath(elem));
			}
		} catch (JavaModelException e) {
			// dont handle here
		}
		return false;
	}
	/*
	 * Returns true if the problem has been completly handled
	 */	
	private boolean processChangeOnType(IJavaElementDelta delta) throws JavaModelException {
		IType type= (IType)delta.getElement();
		switch (delta.getKind()) {
			case IJavaElementDelta.REMOVED:
			case IJavaElementDelta.ADDED:
				fViewer.refresh();
				return true;
			case IJavaElementDelta.CHANGED:
				int flags= delta.getFlags();
				if ((flags & IJavaElementDelta.F_CHILDREN) != 0) {
					if (fMemberFilter != null) {
						int nAffected= delta.getAffectedChildren().length;
						int nChanged= delta.getChangedChildren().length;
						if (nAffected - nChanged > 0) {
							if (fMemberFilter == null) {
								fViewer.refresh(type);
							} else {
								fViewer.refresh();
							}
							return false;
						} else {
							fViewer.update(type, UNSTRUCTURED);
						}
					} 
				} else if ((flags & IJavaElementDelta.F_SUPER_TYPES) != 0) {
					//fireStructuralChange?
				} else {
					fViewer.update(type, UNSTRUCTURED);
				}
		}
		return false;
	}
	/*
	 * Returns true if the problem has been completly handled
	 */					
	private boolean processDelta(IJavaElementDelta delta) throws JavaModelException {
		IJavaElement element= delta.getElement();
		switch (element.getElementType()) {
			case IJavaElement.JAVA_PROJECT:
				if (!getInputType().getJavaProject().equals(element)) {
					return false;
				}
				break;	
			case IJavaElement.TYPE:
				if (getHierarchy().contains((IType)element)) {
					return processChangeOnType(delta);
				}
				return false;
		}
		IJavaElementDelta[] children= delta.getAffectedChildren();
		for (int i= 0; i < children.length; i++) {
			if (processDelta(children[i])) {
				return true;
			}
		}
		return false;
	}
	/**
	 * Sets members to filter the hierarchy for. Set to null to disable member filtering.
	 * When member filtering is enabled, the hierarchy contains only types that contain
	 * an implementation of one of the filter members and the members themself.
	 * The hierarchy can be empty as well.
	 */
	public void setMemberFilter(IMember[] memberFilter) {
		fMemberFilter= memberFilter;
	}
	/**
	 * In member filtering mode, show all types even if they do not contain one
	 * of the filtered members
	 */
	public void showAllTypes(boolean show) {
		fShowAllTypes= show;
	}
}
