package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This interface is implemented by scripting data elements
 * and allows users to access a data element's child elements
 * and attributes.
 * <p>
 * This interface is primarily intended for use 
 * by workbench users writing script code.
 * </p>
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see IDataModel
 */
public interface IDataElement {
/**
 * Adds an attribute with the specified name and value to this data element
 * as an immediate child attribute.
 *
 * @param attributeName name
 * @param attributeValue value
 * @return <code>true</code> if the attribute was added
 */
boolean addAttribute(String attributeName, String attributeValue);
/**
 * Adds an immediate child data element to this data element.
 *
 * @param elementName element name
 * @return added data element
 */
IDataElement addChildElement(String elementName);
/**
 * Returns a string array of the names of the attributes
 * of this data element.
 *
 * @return array of names, or an empty array if no attributes exist
 */
String[] getAttributeNames();
/**
 * Returns the value of the specified attribute of this data element.
 *
 * @param attributeName attribute name
 * @return attribute value
 */
String getAttributeValue(String attributeName);
/**
 * Returns an array of this data element's immediate child elements.
 *
 * @return array of child elements, or an empty array if no
 *		child elements exist.
 */
IDataElement[] getChildElements();
/**
 * Returns the name of this data element.
 *
 * @return element name
 */
String getName();
/**
 * Removes the specified attribute from this data element's list
 * of immediate child attributes. A value of true is returned if 
 * the operation was successful.
 *
 * @param attributeName name of attribute to remove
 * @return <code>true</code> if the attribute was removed
 */
boolean removeAttribute(String attributeName);
/**
 * Removes the specified data element from this data element's list
 * of immediate child data elements. A value of true is returned if
 * the operation was successful.
 *
 * @param dataElement element to remove
 * @return <code>true</code> if the element was removed
 */
boolean removeChildElement(IDataElement dataElement);
/**
 * Modifies the value of the specified child attribute of this data element 
 * to the specified string value.  A value of true is returned if the 
 * operation was successful.
 *
 * @param attributeName name
 * @param attributeValue new value
 * @return <code>true</code> if the attribute value was set
 */
boolean setAttributeValue(String attributeName, String attributeValue);
}
