package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Plugin;

/**
 * This is the primary interface used by script writers.
 * An object instance of this interface is exposed to
 * the script as a local script variable. The script writer
 * uses the methods of this interface to perform various
 * utility functions and to gain access to objects implementing
 * other interfaces available in the workbench environment.
 * <p>
 * This interface is primarily intended for use 
 * by workbench users writing script code.
 * </p>
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see IDataModel
 * @see IToolkitUI
 */
public interface IToolkit {
/**
 * Creates an empty "elements and attributes" data model.
 *
 * @return model element
 */
IDataModel createDataModel();
/**
 * Creates a workbench file in the specified workbench container.
 *
 * @param parent workbench container
 * @param name of the file
 * @return created workbench file
 * @exception Exception
 */
IFile createFile(IContainer container, String filename) throws Exception;
/**
 * Create a workbench folder in the specified workbench container.
 *
 * @param container parent workbench container
 * @param folderName folder name
 * @return created workbench folder
 * @exception Exception
 */
IFolder createFolder(IContainer container, String folderName) throws Exception;
/**
 * Creates a path object from a string representation
 *
 * @param path string representation of the path
 * @return path object
 * @exception Exception
 */
IPath createPath(String path) throws Exception;
/**
 * Creates a workbench project.
 *
 * @param projectName project name
 * @return created workbench project
 * @exception Exception
 */
IProject createProject(String projectName) throws Exception;
/**
 * Returns an array of workbench files that are contained by the specified workbench container.
 *
 * @param container parent workbench container
 * @return array of contained workbench files, or <code>null</code>
 * @exception Exception
 */
IFile[] getFiles(IContainer container) throws Exception;
/**
 * Returns an array of workbench folders that are contained within the specified
 * workbench container.
 *
 * @param container parent workbench container
 * @return array of contained workbench folders,  or <code>null</code>
 * @exception Exception
 */
IFolder[] getFolders(IContainer container) throws Exception;
/**
 * Returns the currently open workbench project.
 *
 * @return open project,  or <code>null</code>
 */
IProject getOpenProject();
/**
 * Returns the plugin that contains the extension that is executing.
 *
 * @return plugin object for the script extension. 
 */
Plugin getPlugin();
/**
 * Returns an array of all workbench projects.
 *
 * @return array of workbench projects, or an empty array if no projects exist
 */
IProject[] getProjects();
/**
 * Returns <code>true</code> if the specified object is of the specified type.
 * This method can be used from a scripting language
 * to determine whether an object is of a specified
 * class type, or interface type.
 *
 * @param object to test
 * @param classType fully qualified type name
 * @return <code>true</code> if the object is an instance of the
 *		specified type
 */
boolean isInstanceOf(Object object, String classType);
/**
 * Creates a data model from the data model definition within the 
 * specified workbench file.
 * 
 * @param file model file
 * @return loaded model
 * @exception Exception
 */
IDataModel loadDataModelFromFile(IFile file) throws Exception;
/**
 * Creates a data model from the data model definition within the 
 * specified file system file.
 *
 * @param fileSystemFileName name of model file
 * @return loaded model
 * @exception Exception
 */
IDataModel loadDataModelFromFilename(String fileSystemFilename)
	throws Exception;
/**
 * Displays an object's toString() value in the Script Console View.
 *
 * @param object to print
 */
void print(Object object);
/**
 * Displays a message box with an OK button.  Valid message types are
 * "information", "warning", and "error".
 *
 * @param title
 * @param message text
 * @param messageType "information", "warning", or "error"
 */
void promptForOk(String title, String message, String messageType);
/**
 * Displays a message box with OK and Cancel buttons.  A value of "true"
 * is returned if the OK button was pressed.
 *
 * @param title 
 * @param message text
 * @return <code>true</code> if OK was pressed
 */
boolean promptForOkCancel(String title, String message);
/**
 * Displays either an "open" or a "save" dialog that allows a user to
 * select a file system file.
 *
 * @param saveNotOpen <code>true</code> to display "Save" dialog,
 *		<code>false</code> to display "Open" dialog
 * @return file path
 * @exception Exception
 */
IPath promptForSystemFile(boolean saveNotOpen) throws Exception;
/**
 * Displays a dialog that allows a user to choose a file system folder.
 *
 * @return folder path
 * @exception Exception
 */
IPath promptForSystemFolder() throws Exception;
/**
 * Displays a dialog that allows a user to select a workbench container.
 * The containerRoot parameter may be specified as a starting point. 
 *
 * @param containerRoot top container
 * @param promptMessage message to display
 * @return selected workbench container, or <code>null</code>.
 * @exception Exception
 */
IContainer promptForWorkspaceContainer(
	IContainer containerRoot,
	String promptMessage)
	throws Exception;
/**
 * Displays a dialog which allows a user to choose one or more workbench
 * files.  The containerRoot parameter may be specified as a starting point.
 *
 * @param containerRoot top container
 * @param promptMessage message to display
 * @return array of selected files, or an empty array
 * @exception Exception
 */
IFile[] promptForWorkspaceFiles(IContainer containerRoot, String promptMessage)
	throws Exception;
}
