package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.resources.IResource;

/**
 * This is the primary interface used by script writers
 * using the UI rendering support in this package.
 * An object instance of this interface is exposed to
 * the script as a local script variable. The script writer
 * uses the methods of this interface to perform various
 * utility functions and to gain access to objects implementing
 * other interfaces available in the workbench environment.
 * <p>
 * This interface is primarily intended for use 
 * by workbench users writing script code.
 * </p>
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see IDataModel
 * @see IToolkit
 * @see StringDataMultiple
 * @see StringDataSingle
 */
public interface IToolkitUI extends IToolkit {
/**
 * Returns a specified user interface element's data source identifier.
 *
 * @param userInterfaceElementID user interface element identifier
 * @return data source identifier
 */
String getDataSource(String userInterfaceElementID);
/**
 * Returns the widget that generated the current event.
 *
 * @return widget
 */
Object getEventWidget();
/**
 * Returns a specified user interface element's link source identifier.
 * A link source is a user interface element that sends selection
 * notifications to this user interface element.
 *
 * @param userInterfaceElementID user interface element
 * @return link source for the element
 */
String getLinkSource(String userInterfaceElementID);
/**
 * Returns an array of resources currently selected in the workbench.
 *
 * @return selected resources, or an empty array if no resources were selected,
 * or null.
 */
IResource[] getResourcesSelected();
/**
 * Returns the data model currently associated with the user interface.
 *
 * @return data model
 */
IDataModel getUIDataModel();
/**
 * Returns the widget that has the given user interface element identifier.
 *
 * @param userInterfaceElementID interface element identifier
 * @return widget
 */
Object getWidget(String userInterfaceElementID);
/**
 * Links the specified user interface element to the specified data element
 * node(s) of the data model currently associated with the user
 * interface.  Whenever a data element node's value changes,
 * the user interface of the user interface element is automatically updated.
 * Also, if the user interface element's value changes, the data model element
 * is automatically updated.  The data source element name is specified 
 * in a string such as "company.department.name".  The success of this method
 * is determined by whether the specified user interface element is found.
 *
 * @param userInterfaceElementID user interface element identifier
 * @param dataSource data source identifier
 * @return <code>true</code> if successful
 */
boolean setDataSource(String userInterfaceElementID, String dataSource);
/**
 * Links two user interface elements together.  A user interface element,
 * given by the first identifier, becomes the target for selection
 * notifications from another user interface element given by the second
 * identifier.  The user interface of the target element is automatically
 * updated to reflect changes to the link source user interface element.
 * The success of this method
 * is determined by whether the specified user interface element is found.
 *
 * @param userInterfaceElementID target user interface element identifier
 * @param linkSourceUserInterfaceElementID source user interface element identifier
 * @return <code>true</code> if successful
 */
boolean setLinkSource(String userInterfaceElementID, String linkSourceUserInterfaceElementID);
/**
 * Links the specified user interface element to the specified string data object.
 * Whenever the user interface element's value changes, the string data object's value
 * is automatically updated.  This method is typically used by programmers who
 * have their own data model, and want to associate an element of their data model
 * to a user interface element, such that the user interface element displays the
 * value of the element.  An intermediate StringData instance is used to hold the
 * initial value to be presented, and contains an updated value when the user interface
 * completes.  The StringData instance can optionally hold a user object such as a data
 * model object for reference. The usage steps are&colon.
 * <ul>
 * <li>When the user interface is created, create amd store an instance of StringData</li>
 * <li>Obtain the initial string to be displayed in the user interface element from your data model object</li>
 * <li>Set the initial string to be displayed in the user interface using StringData.setValue</li>
 * <li>Optionally associate your data model object with the StringData instance by using StringData.setUserObject</li>
 * <li>Call this method giving the identifier of the user interface element, and the StringData instance</li>
 * <li>When the user interface is ending, retrieve the new current value using StringData.getValue</li>
 * <li>Retrieve your data model object using StringData.getUserObject</li>
 * <li>Set the new string value into the data model object</li>
 * </ul>
 *
 * @param userInterfaceElementID user interface element identifier
 * @param stringData data element
 */
public boolean setStringDataSource(String strID, IStringData stringData);
/**
 * Associates a data model with the user interface.  User interface elements
 * can then be associated with data model elements.
 *
 * @param dataModel data model
 */
void setUIDataModel(IDataModel dataModel);
}
