package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.IOException;
import java.util.Vector;

import org.eclipse.core.runtime.model.ConfigurationElementModel;
import org.eclipse.core.runtime.model.ConfigurationPropertyModel;
import org.eclipse.core.runtime.model.ExtensionModel;
import org.eclipse.core.runtime.model.PluginDescriptorModel;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.scripting.internal.ScriptPlugin;
import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.scripting.internal.views.ScriptDevelopmentWizard;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * This class provides a default implementation of a
 * script preference page. The preference pages are used to add
 * or modify scripts registered into specific extension points.
 * <p>
 * This class is primarily intended for use 
 * by developers who are script-enabling their workbench extensions.
 * </p>
 * <p>
 * Clients may subclass this class.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see IScriptPreferencePage
 * @see ScriptRunner
 */
public abstract class ScriptAbstractPreferencePage extends PreferencePage implements IScriptPreferencePage, IWorkbenchPreferencePage, Listener {
	private ExtensionModel extensionModel = null;

	private String contiguousName = null;
	private String scriptName = null;

	boolean restartMessageRequired = false;
	Button buttonScriptAdd = null;
	Button buttonScriptEdit = null;
	Button buttonScriptRemove = null;
	List listScripts = null;

	Vector vectorDescriptiveNames = new Vector(); // Displayed script names
	Vector vectorExtensionModels = new Vector(); // Displayed models
	Vector vectorAllExtensionModels = new Vector(); // All models

	static final String STRING_EMPTY = new String();
/**
 * Creates a configuration element.
 *
 * @param parent parent (configuration element (for new child element)
 *		or extension (for new top-level element))
 * @param name element name
 * @return created configuration element
 */
public ConfigurationElementModel createConfigurationElement(Object parent, String name) {

	ConfigurationElementModel element = new ConfigurationElementModel();

	element.setParent(parent);
	element.setName(name);
	return element;
}
/**
 * Creates a configuration property given a name and a value.
 *
 * @param name property name
 * @param value property value
 * @return created property
 */
public ConfigurationPropertyModel createConfigurationProperty(String name, String value) {

	ConfigurationPropertyModel property = new ConfigurationPropertyModel();

	property.setName(name);
	property.setValue(value);
	return property;
}
/**
 * Creates the user interface content of the parent composite.
 *
 * @param compositeParent parent container
 * @return page content
 */
public Control createContents(Composite compositeParent) {
	
	// Content
	//--------
	Composite compositeContent = new Composite(compositeParent, SWT.NULL);

	GridLayout gridLayout = new GridLayout();
	gridLayout.numColumns = 2;
	compositeContent.setLayout(gridLayout);

	GridData gridData = new GridData();
	gridData.verticalAlignment = GridData.FILL;
	gridData.horizontalAlignment = GridData.FILL;
	compositeContent.setLayoutData(gridData);

	// Label: Scripts
	//---------------
	Label label = new Label(compositeContent, SWT.LEFT);
	label.setText(ScriptStrings.getString("S_Scripts"));
	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.horizontalSpan = 2;
	label.setLayoutData(gridData);

	// List: Scripts
	//--------------	
	this.listScripts = new List(compositeContent, SWT.BORDER);
	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace = true;
	this.listScripts.setLayoutData(gridData);

	// Composite: for buttons
	//-----------------------
	Composite compositeGroup = new Composite(compositeContent, SWT.NULL);
	compositeGroup.setLayout(new GridLayout());
	gridData = new GridData();
	gridData.verticalAlignment = GridData.FILL;
	gridData.horizontalAlignment = GridData.FILL;
	compositeGroup.setLayoutData(gridData);

	// Button: Add
	//------------
	this.buttonScriptAdd = new Button(compositeGroup, SWT.PUSH);
	this.buttonScriptAdd.setText(ScriptStrings.getString("S_Add") + "...");
	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	this.buttonScriptAdd.setLayoutData(gridData);

	// Button: Edit
	//-------------	
	this.buttonScriptEdit = new Button(compositeGroup, SWT.PUSH);
	this.buttonScriptEdit.setText(ScriptStrings.getString("S_Edit") + "...");
	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	this.buttonScriptEdit.setLayoutData(gridData);

	// Button: Remove
	//---------------	
	this.buttonScriptRemove = new Button(compositeGroup, SWT.PUSH);
	this.buttonScriptRemove.setText(ScriptStrings.getString("S_Remove"));
	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	this.buttonScriptRemove.setLayoutData(gridData);

	// Populate the list of scripts associated with this extension point
	//------------------------------------------------------------------
	populateScriptList();

	// Select the first list item
	//---------------------------
	if (this.listScripts.getItemCount() > 0) {
		this.listScripts.select(0);
	}

	// Add listeners
	//--------------
	this.buttonScriptAdd.addListener(SWT.Selection, this);
	this.buttonScriptEdit.addListener(SWT.Selection, this);
	this.buttonScriptRemove.addListener(SWT.Selection, this);

	// Enable / disable the buttons
	//-----------------------------
	updateEnabledState();

	// Initialize flag for warning user that a workbench restart is required
	//----------------------------------------------------------------------
	setRestartMessageRequired(false);

	return compositeContent;
}
/**
 * Displays the add/modify script wizard dialog.  This function
 * is called when either the add, or modify button is pressed.
 * The boolean parameter, when true, signifies that the properties of a newly created
 * script is to be defined.  Otherwise, a false value signifies that
 * an existing script is being modified.
 *
 * @param newScript <code>true</code> for new script, <code>false</code>
 *		for existing script
 */
protected void displayScriptWizard(boolean newScript) {
	
	ScriptDevelopmentWizard wizard = new ScriptDevelopmentWizard(this, newScript);
	WizardDialog dialog = new WizardDialog(getControl().getShell(), wizard);

	// Display the dialog as modal / wait
	//-----------------------------------
	dialog.open();
}
/**
 * Displays a message indicating that the workbench needs restarting to activate
 * the changes to the scripted extensions.
 */
protected void finish() {
	if (this.restartMessageRequired == true) {
		MessageDialog.openInformation(getShell(), ScriptStrings.getString("S_Scripts"), ScriptStrings.getString("S_You_must_restart_the_workbench_to_activate_any_changes"));
		this.restartMessageRequired = false;
	}
}
/**
 * Returns the named child element of the specified parent.
 *
 * @param element parent
 * @param elementName name of child element
 * @return child element or <code>null</code> if not found
 */
public ConfigurationElementModel getConfigurationSubElement(ConfigurationElementModel element, String elementName) {

	ConfigurationElementModel[] elements = element.getSubElements();

	for (int i = 0; i < elements.length; ++i) {
		if (elements[i].getName().equals(elementName) == true)
			return elements[i];
	}

	return null;
}
/**
 * Returns the named child element of the specified parent.
 *
 * @param element parent
 * @param elementName name of the child element
 * @return element or <code>null</code> if not found
 */
public ConfigurationElementModel getConfigurationSubElement(ExtensionModel element, String elementName) {

	ConfigurationElementModel[] elements = element.getSubElements();

	for (int i = 0; i < elements.length; ++i) {
		if (elements[i].getName().equals(elementName) == true)
			return elements[i];
	}

	return null;
}
/**
 * Returns a version of the descriptive script name where all space characters
 * have been replaced by underscore characters.
 *
 * @return contiguous name
 */
public String getContiguousName() {
	return this.contiguousName;
}
/**
 * Returns the extension model associated with the descriptive name
 * of the extension selected by the user from the list of extensions.
 *
 * @return extension model
 */
public ExtensionModel getExtensionModel() {
	return this.extensionModel;
}
/**
 * Returns the value of a property given an element and a property name,
 *
 * @param element element name
 * @param propertyName property name
 * @return property value
 */
public String getPropertyValue(ConfigurationElementModel element, String propertyName) {

	if (element == null)
		return STRING_EMPTY;

	ConfigurationPropertyModel[] properties = element.getProperties();

	if (properties == null)
		return STRING_EMPTY;

	for (int i = 0; i < properties.length; ++i) {
		if (properties[i].getName() != null && properties[i].getName().equals(propertyName) == true) {
			if (properties[i].getValue() != null)
				return properties[i].getValue();
			else
				return STRING_EMPTY;
		}
	}

	return STRING_EMPTY;
}
/**
 * Returns the descriptive script name entered by the user on the script name wizard page.
 *
 * @return name of the script
 */
public String getScriptName() {
	return this.scriptName;
}
/**
 * Returns a list of descriptive names currently displayed for
 * a particular plugin extension type.  This list was previously created in method
 * getScriptNamesFromManifest.  This method is used by the script name wizard page to
 * ensure that an existing name is not entered on the script name wizard page.
 *
 * @return list of script names
 */
public java.util.List getScriptNames() {
	
	return this.vectorDescriptiveNames;
}
/**
 * Returns a list of descriptive names of all scripts associated with an
 * extension point by:
 * <ul>
 * <li>Obtaining the extension point identifier from the subclass</li>
 * <li>Obtaining a list of extensions that extend the extension point from the plugin manifest</li>
 * <li>Obtaining the descriptive name of each extension from the subclass</li>
 * </ul>
 *
 * This method is used by the preference page in order to list the scripts in the list
 * box.
 *
 * @return array of script names
 */
protected String[] getScriptNamesFromManifest() {

	String[] straScriptNames = null;

	// Obtain the adapter identifier that this script type supports
	//-------------------------------------------------------------
	String strAdapterID = getAdapterID();

	// Check for a valid extension point
	//----------------------------------
	//	if( false ) {

	if (strAdapterID != null && strAdapterID.length() > 0) {

		PluginDescriptorModel pluginDescriptor = ScriptPlugin.getPluginInstance().getUserScriptsPluginManifest();

		// Obtain the extensions
		//----------------------
		ExtensionModel[] extensionEntries = pluginDescriptor.getDeclaredExtensions();

		if (extensionEntries == null)
			extensionEntries = new ExtensionModel[0];

		// Find the extensions that extend the extension point
		//----------------------------------------------------
		String strDescriptiveName = null;

		this.vectorDescriptiveNames.removeAllElements();
		this.vectorExtensionModels.removeAllElements();
		this.vectorAllExtensionModels.removeAllElements();

		String strSimpleID = null;
		String strAdapter = null;
		int iIndexLeft = 0;
		int iIndexRight = 0;

		// For each extension that uses the adapter, obtain its descriptive name
		//----------------------------------------------------------------------
		for (int i = 0; i < extensionEntries.length; ++i) {

			// Add all extensions to the full list
			//------------------------------------
			this.vectorAllExtensionModels.addElement(extensionEntries[i]);
			
			strSimpleID = extensionEntries[i].getId();

			if (strSimpleID != null && strSimpleID.length() > 0) {

				// Obtain the adapter string
				//--------------------------
				iIndexLeft = strSimpleID.indexOf("[[");
				iIndexRight = strSimpleID.indexOf("]]");

				if (iIndexLeft >= 0 && iIndexRight > iIndexLeft + 2) {
					strAdapter = strSimpleID.substring(iIndexLeft + 2, iIndexRight);

					// Check for matching ID
					//----------------------
					if (strAdapter != null && strAdapter.equals(strAdapterID) == true) {

						strDescriptiveName = extensionEntries[i].getName();

						// Add the descriptive name to the list
						// Add the extension to the parallel list for the remove function
						//---------------------------------------------------------------
						if (strDescriptiveName != null && strDescriptiveName.length() > 0) {

							this.vectorDescriptiveNames.addElement(strDescriptiveName);
							this.vectorExtensionModels.addElement(extensionEntries[i]);
						}
					}					
				}
			}
		}

		// Convert the list to an array
		//-----------------------------
		straScriptNames = new String[vectorDescriptiveNames.size()];

		for (int i = 0; i < vectorDescriptiveNames.size(); ++i) {
			straScriptNames[i] = (String) vectorDescriptiveNames.elementAt(i);
		}
	}

	// The extension point identifier is invalid
	//------------------------------------------
	else {
		straScriptNames = new String[0];
	}

	return straScriptNames;
}
/**
 * Handles the add, modify, and remove button press.  The add and modify buttons
 * display the add/modify script wizard.  The remove button removes a script.
 * This method overrides the default event handler processing.
 *
 * @param event
 */
public void handleEvent(Event event) {

	// Add button pressed
	//-------------------
	if (event.widget == this.buttonScriptAdd) {

		// Display the script wizard with the
		// new script flag set to true
		//----------------------------
		displayScriptWizard(true);
	}

	// Edit button pressed
	//--------------------
	else if (event.widget == this.buttonScriptEdit) {

		// Store the selected script descriptive name
		//-------------------------------------------
		String[] straSelected = this.listScripts.getSelection();
		setScriptName(straSelected[0]);

		// Store the selected extension entry
		//-----------------------------------
		this.extensionModel = (ExtensionModel) this.vectorExtensionModels.elementAt(this.listScripts.getSelectionIndex());

		// Display the script wizard with the
		// new script flag set to false
		//-----------------------------------
		displayScriptWizard(false);
	}

	// Remove button pressed
	//----------------------
	else if (event.widget == this.buttonScriptRemove) {
		// Remove the selected script
		// The list is single selection
		//-----------------------------
		int iSelectedIndex = this.listScripts.getSelectionIndex();

		if (iSelectedIndex > -1) {
			boolean bRemoved = removeScript(iSelectedIndex);

			if (bRemoved == true) {
				// Select an item in the list
				//---------------------------
				if (iSelectedIndex < this.listScripts.getItemCount()) {
					this.listScripts.select(iSelectedIndex);
				}

				else if (iSelectedIndex - 1 >= 0) {
					this.listScripts.select(iSelectedIndex - 1);
				}
			}
		}
	}

	updateEnabledState();
}
/**
 * Implementation of IWorkbenchPreferencePage required method.  This implementation does nothing.
 *
 * @param workbench for this preference page
 */
public void init(IWorkbench workbench) {
}
/**	
 * Overrides the default implementation of processing performed
 * when the preference page Cancel button is pressed.  The workbench restart information
 * message is displayed as required.  The preference page is then closed.
 *
 * @return <code>true</code> to close the preference dialog page
 */
public boolean performCancel() {
	if (this.getControl().isVisible() == true)
		finish();
	return true;
}
/** 
 * Overrides the default implementation of processing performed
 * when the preference page OK button is pressed.  The workbench restart information
 * message is displayed as required.  The preference dialog page is then closed.
 *
 * @return true to close the preference dialog page
 */
public boolean performOk() {
	if (this.getControl().isVisible() == true)
		finish();
	return true;
}
/**
 * Populates the list box with descriptive names obtained from the plugin manifest.
 */
protected void populateScriptList() {

	getScriptNamesFromManifest();

	this.listScripts.removeAll();

	for (int i = 0; i < this.vectorDescriptiveNames.size(); ++i) {
		this.listScripts.add((String) this.vectorDescriptiveNames.elementAt(i));
	}
}
/**
 * A dialog is presented which allows the user to cancel the remove script operation.  
 * If not cancelled, the script extension is removed from the plugin manifest,
 * and the script's descriptive name is removed from the list box.
 *
 * @param index index of entry to remove
 */
public boolean removeScript(int index) {

	// Obtain the Scripts plugin manifest, and its descriptor
	//-------------------------------------------------------
	PluginDescriptorModel pluginDescriptor = ScriptPlugin.getPluginInstance().getUserScriptsPluginManifest();

	ExtensionModel extensionEntry = (ExtensionModel) this.vectorExtensionModels.elementAt(index);

	// Obtain the project name from the extension entry
	// Simple Identifier = "[[adapter name]]projectName"
	//--------------------------------------------------
	String strFolderName = extensionEntry.getId();

	int iNameIndex = strFolderName.indexOf("]]");

	if (iNameIndex >= 0 && strFolderName.length() >= iNameIndex + 2) {
		strFolderName = strFolderName.substring(iNameIndex + 2);
	}

	// Create warning message dialog
	//------------------------------
	StringBuffer strbMessage = new StringBuffer();

	// Warn of deleting script
	//------------------------
	strbMessage.append(ScriptStrings.getString("S_Are_you_sure_you_want_to_remove_this_script?"));
	strbMessage.append("\n\n    ");
	strbMessage.append(this.listScripts.getItem(index));

	boolean bRemove = MessageDialog.openConfirm(getControl().getShell(), ScriptStrings.getString("S_Remove_Script"), strbMessage.toString());

	if (bRemove == true) {

		// Remove the entry from the total list
		//-------------------------------------
		this.vectorAllExtensionModels.remove(this.vectorExtensionModels.elementAt(index));

		// Remove the entry from the displayed list
		//-----------------------------------------
		this.vectorExtensionModels.remove(index);

		// Alter the manifest
		//-------------------
		ExtensionModel[] extensionModels = new ExtensionModel[this.vectorAllExtensionModels.size()];
		this.vectorAllExtensionModels.copyInto(extensionModels);
		pluginDescriptor.setDeclaredExtensions(extensionModels);

		// Save the manifest
		//------------------
		try {
			ScriptPlugin.getPluginInstance().saveUserScriptsPluginManifest();
		}
		catch (IOException ex) {
		}

		populateScriptList();
	}

	// Return true if script container was removed
	//--------------------------------------------
	return bRemove;
}
/**
 * Sets the state of the restart message required flag to indicate whether an
 * informational message box requesting that the user restart the workbench is needed.
 * 
 * @param required <code>true</code> if the restart dialog is to be
 *		displayed when the preference page exits
 */
public void setRestartMessageRequired(boolean required) {

	this.restartMessageRequired = required;
}
/**
 * Sets the displayable label of the script.  The container name is generated from the
 * script name.  
 *
 * @param label display label
 */
public void setScriptName(String label) {
	
	this.scriptName = label.trim();

	// Store the selected script contiguous name as well
	//--------------------------------------------------
	String strContiguousName = this.scriptName;
	strContiguousName = strContiguousName.replace(' ', '_');
	this.contiguousName = strContiguousName;
}
/**
 * Makes the page visible, or invisible.  Triggers displaying of the restart information
 * dialog as required when the page becomes invisible.
 *
 * @param visible <code>true</code> to set page visible, <code>false</code> otherwise
 */
public void setVisible(boolean visible) {

	if (visible == false)
		finish();

	super.setVisible(visible);
}
/**
 * Enables or disables the modify and remove buttons.
 */
protected void updateEnabledState() {

	// Enable/disable the edit and remove buttons.  The
	// buttons are enabled if the user has selected
	// a script name
	//-------------------------------------------------
	boolean bScriptSelected = this.listScripts.getSelectionIndex() >= 0;

	this.buttonScriptEdit  .setEnabled( bScriptSelected );
	this.buttonScriptRemove.setEnabled( bScriptSelected );
}
/**
 * Called by the new/modify wizard when its Finish button has been pressed.
 * A plugin extension is given to the subclass to modify.
 *
 * @param newScript is <code>true</code> if a new script has been created, otherwise <code>false</code>
 * if an existing script has been modified.
 */
public void wizardComplete(boolean newScript) {

	// Create / modify the plugin extension
	//-------------------------------------
	PluginDescriptorModel pluginDescriptor = ScriptPlugin.getPluginInstance().getUserScriptsPluginManifest();

	ExtensionModel extensionModel = null;

	if (newScript == false) {
		// Retrieve the selected index from the list box
		// This is the index into the vector that holds the extensions
		//------------------------------------------------------------
		int iSelectedIndex = this.listScripts.getSelectionIndex();

		extensionModel = (ExtensionModel) this.vectorExtensionModels.elementAt(iSelectedIndex);
	}
	else {
		extensionModel = new ExtensionModel();
	}

	// Set extension element attributes
	// point="extensionPointID", id="containerName", name="descriptiveName"
	//---------------------------------------------------------------------	
	extensionModel.setParentPluginDescriptor(pluginDescriptor);
	extensionModel.setExtensionPoint(getExtensionPointID());
	extensionModel.setId("[[" + getAdapterID() + "]]" + getContiguousName());
	extensionModel.setName(getScriptName());

	// Ask the subclass to modify the extension
	// Indicate that this is a new script
	//-----------------------------------------
	modifyPluginExtension(extensionModel, true);

	// Add any new one to the list
	//----------------------------
	if (newScript == true) {
		this.vectorDescriptiveNames.addElement(getScriptName());
		this.vectorExtensionModels.addElement(extensionModel);
		this.vectorAllExtensionModels.addElement(extensionModel);

		this.listScripts.add(getScriptName());
	}

	// Set the extensions
	//-------------------
	ExtensionModel[] extensionModels = new ExtensionModel[this.vectorAllExtensionModels.size()];
	this.vectorAllExtensionModels.copyInto(extensionModels);
	pluginDescriptor.setDeclaredExtensions(extensionModels);

	// Save the manifest
	//------------------
	try {
		ScriptPlugin.getPluginInstance().saveUserScriptsPluginManifest();
		setRestartMessageRequired(true);
	}
	catch (IOException ex) {
	}

	// Repopulate the list
	//--------------------
	populateScriptList();
		
	// Select the new/modified item
	//-----------------------------
	if (this.listScripts.getItemCount() > 0) {

		// Find the modified extension entry in the vector
		//------------------------------------------------
		int iIndex = this.vectorExtensionModels.indexOf(extensionModel);

		// Select the index
		//-----------------
		this.listScripts.select(iIndex > -1 ? iIndex : 0);
	}

	return;
}
}
