package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.jface.preference.IPreferencePage;
import org.eclipse.jface.preference.IPreferencePageContainer;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.scripting.internal.uirenderer.XGRElementUISwtPreferencePage;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

/**
 * This class is a delegating wrapper for a script adaptor
 * preference page implementation. Actual script adaptors 
 * subclass this class.
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 */
public abstract class ScriptAdapterDelegatingPreferencePage implements IPreferencePage {
	/* package */ XGRElementUISwtPreferencePage page;	
/**
 * Default constructor.
 */
public ScriptAdapterDelegatingPreferencePage() {
	super();
	page = new XGRElementUISwtPreferencePage();
}
/**
 * Computes a size for this page's UI component. 
 *
 * @return the size of the preference page encoded as
 *   <code>new Point(width,height)</code>, or 
 *   <code>(0,0)</code> if the page doesn't currently have any UI component
 */
public final Point computeSize() {
	return page.computeSize();
}
/**
 * Creates the top level control for this dialog
 * page under the given parent composite.
 * <p>
 * Implementors are responsible for ensuring that
 * the created control can be accessed via <code>getControl</code>
 * </p>
 *
 * @param parent the parent composite
 */
public final void createControl(Composite parent) {
	page.createControl(parent);	
}
/**
 * Disposes the SWT resources allocated by this
 * dialog page.
 */
public final void dispose() {
	page.dispose();	
}
/**
 * Returns the top level control for this dialog page.
 *
 * @return the top level control
 */
public final Control getControl() {
	return page.getControl();
}
/**
 * Returns this dialog page's description text.
 *
 * @return the description text for this dialog page, 
 *  or <code>null</code> if none
 */
public final String getDescription() {
	return page.getDescription();
}
/**
 * Returns the current error message for this dialog page.
 * May be <code>null</null> to indicate no error message.
 * <p>
 * An error message should indicate some describe some error state 
 * as opposed to a message which may simply provide instruction
 * or information to the user.
 * </p>
 * 
 * @return the error message, or <code>null</code> if none
 */
public final String getErrorMessage() {
	return page.getErrorMessage();
}
/**
 * Returns this dialog page's image.
 *
 * @return the image for this dialog page, or <code>null</code>
 *  if none
 */
public final Image getImage() {
	return page.getImage();
}
/**
 * Returns the current message for this wizard page.
 * <p>
 * A message provides instruction or information to the 
 * user as opposed to an error message which should 
 * indicate some error state.
 * </p>
 * 
 * @return the message, or <code>null</code> if none
 */
public final String getMessage() {
	return page.getMessage();
}
/**
 * Returns this dialog page's title.
 *
 * @return the title of this dialog page, 
 *  or <code>null</code> if none
 */
public final String getTitle() {
	return page.getTitle();
}
/**
 * Returns whether this dialog page is in a valid state.
 * 
 * @return <code>true</code> if the page is in a valid state,
 *   and <code>false</code> if invalid
 */
public final boolean isValid() {
	return page.isValid() ;
}
/**
 * Checks whether it is alright to leave this page.
 * 
 * @return <code>false</code> to abort page flipping and the
 *  have the current page remain visible, and <code>true</code>
 *  to allow the page flip
 */
public final boolean okToLeave() {
	return page.okToLeave();
}
/**
 * Notifies that the container of this preference page has been canceled.
 * 
 * @return <code>false</code> to abort the container's cancel 
 *  procedure and <code>true</code> to allow the cancel to happen
 */
public final boolean performCancel() {
	return page.performCancel();
}
/**
 * Notifies that help has been requested for this dialog page.
 */
public final void performHelp() {
	page.performHelp();	
}
/**
 * Notifies that the OK button of this page's container has been pressed.
 * 
 * @return <code>false</code> to abort the container's OK
 *  processing and <code>true</code> to allow the OK to happen
 */
public final boolean performOk() {
	return page.performOk();
}
/**
 * Sets or clears the container of this page.
 *
 * @param preferencePageContainer the preference page container, or <code>null</code> 
 */
public final void setContainer(IPreferencePageContainer preferencePageContainer) {
	page.setContainer(preferencePageContainer);
}
/**
 * Sets this dialog page's description text.
 * 
 * @param description the description text for this dialog
 *  page, or <code>null</code> if none
 */
public final void setDescription(String description) {
	page.setDescription(description);	
}
/**
 * Sets this dialog page's image.
 *
 * @param image the image for this dialog page, 
 *  or <code>null</code> if none
 */
public final void setImageDescriptor(ImageDescriptor image) {
	page.setImageDescriptor(image);
}
/**
 * Sets the size of this page's UI component.
 *
 * @param size the size of the preference page encoded as
 *   <code>new Point(width,height)</code>
 */
public final void setSize(Point size) {
	page.setSize(size);	
}
/**
 * Set this dialog page's title.
 *
 * @param title the title of this dialog page, 
 *  or <code>null</code> if none
 */
public final void setTitle(String title) {
	page.setTitle(title);	
}
/**
 * Sets the visibility of this dialog page.
 *
 * @param visible <code>true</code> to make this page visible,
 *  and <code>false</code> to hide it
 */
public final void setVisible(boolean visible) {
	page.setVisible(visible);	
}
}
