package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardContainer;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.scripting.internal.uirenderer.XGRElementUISwtWizard;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Composite;

/**
 * This class is a delegating wrapper for a script adaptor
 * wizard implementation. Actual script adaptors
 * subclass this class.
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 */
public abstract class ScriptAdapterDelegatingWizard implements IWizard {
	/* package */ XGRElementUISwtWizard wizard;
/**
 * Default constructor.
 */
public ScriptAdapterDelegatingWizard() {
	super();
	wizard = new XGRElementUISwtWizard();
}
/**
 * Adds any last-minute pages to this wizard.
 * <p>
 * This method is called just before the wizard becomes visible, to give the 
 * wizard the opportunity to add any lazily created pages.
 * </p>
 */
public final void addPages() {
	wizard.addPages();
}
/**
 * Returns whether this wizard could be finished without further user
 * interaction.
 * <p>
 * The result of this method is typically used by the wizard container to enable
 * or disable the Finish button.
 * </p>
 *
 * @return <code>true</code> if the wizard could be finished,
 *   and <code>false</code> otherwise
 */
public final boolean canFinish() {
	return wizard.canFinish();
}
/**
 * Creates this wizard's controls in the given parent control.
 * <p>
 * The wizard container calls this method to create the controls
 * for the wizard's pages before the wizard is opened. This allows
 * the wizard to size correctly; otherwise a resize may occur when
 * moving to a new page.
 * </p>
 *
 * @param pageContainer the parent control
 */
public final void createPageControls(Composite pageContainer) {
	wizard.createPageControls(pageContainer);	
}
/**
 * Disposes of this wizard and frees all SWT resources.
 */
public final void dispose() {
	wizard.dispose();
}
/**
 * Returns the container of this wizard.
 *
 * @return the wizard container, or <code>null</code> if this
 *   wizard has yet to be added to a container
 */
public final IWizardContainer getContainer() {
	return wizard.getContainer();
}
/**
 * Returns the default page image for this wizard.
 * <p>
 * This image can be used for pages which do not
 * supply their own image.
 * </p>
 *
 * @return the default page image
 */
public final Image getDefaultPageImage() {
	return wizard.getDefaultPageImage();
}
/**
 * Returns the dialog settings for this wizard.
 * <p>
 * The dialog store is used to record state between
 * wizard invocations (for example, radio button selections,
 * last directory, etc.).
 * </p>
 *
 * @return the dialog settings, or <code>null</code> if none
 */
public final IDialogSettings getDialogSettings() {
	return wizard.getDialogSettings();
}
/**
 * Returns the successor of the given page.
 * <p>
 * This method is typically called by a wizard page
 * </p>
 *
 * @param page the page
 * @return the next page, or <code>null</code> if none
 */
public final IWizardPage getNextPage(IWizardPage page) {
	return wizard.getNextPage(page);
}
/**
 * Returns the wizard page with the given name belonging to this wizard.
 *
 * @param pageName the name of the wizard page
 * @return the wizard page with the given name, or <code>null</code> if none
 */
public final IWizardPage getPage(String pageName) {
	return wizard.getPage(pageName);
}
/**
 * Returns the number of pages in this wizard.
 *
 * @return the number of wizard pages
 */
public final int getPageCount() {
	return wizard.getPageCount();
}
/**
 * Returns all the pages in this wizard.
 *
 * @return a list of pages
 */
public final IWizardPage[] getPages() {
	return wizard.getPages();
}
/**
 * Returns the predecessor of the given page.
 * <p>
 * This method is typically called by a wizard page
 * </p>
 *
 * @param page the page
 * @return the previous page, or <code>null</code> if none
 */
public final IWizardPage getPreviousPage(IWizardPage page) {
	return wizard.getPreviousPage(page);
}
/**
 * Returns the first page to be shown in this wizard.
 *
 * @return the first wizard page
 */
public final IWizardPage getStartingPage() {
	return wizard.getStartingPage();
}
/**
 * Returns the title bar color for this wizard.
 *
 * @return the title bar color
 */
public final RGB getTitleBarColor() {
	return wizard.getTitleBarColor();
}
/**
 * Returns the window title string for this wizard.
 *
 * @return the window title string, or <code>null</code> for no title
 */
public final String getWindowTitle() {
	return wizard.getWindowTitle();
}
/**
 * Returns whether help is available for this wizard.
 * <p>
 * The result of this method is typically used by the container to
 * show or hide the Help button.
 * </p>
 *
 * @return <code>true</code> if help is available,
 *   and <code>false</code> if this wizard is helpless
 */
public final boolean isHelpAvailable() {
	return wizard.isHelpAvailable();
}
/**
 * Returns whether this wizard needs Previous and Next buttons.
 * <p>
 * The result of this method is typically used by the container.
 * </p>
 *
 * @return <code>true</code> if Previous and Next buttons are required,
 *   and <code>false</code> if none are needed
 */
public final boolean needsPreviousAndNextButtons() {
	return wizard.needsPreviousAndNextButtons();
}
/**
 * Returns whether this wizard needs a progress monitor.
 * <p>
 * The result of this method is typically used by the container.
 * </p>
 *
 * @return <code>true</code> if a progress monitor is required,
 *   and <code>false</code> if none is needed
 */
public final boolean needsProgressMonitor() {
	return wizard.needsProgressMonitor();
}
/**
 * Performs any actions appropriate in response to the user 
 * having pressed the Cancel button, or refuse if canceling
 * now is not permitted.
 *
 * @return <code>true</code> to indicate the cancel request
 *   was accepted, and <code>false</code> to indicate
 *   that the cancel request was refused
 */
public final boolean performCancel() {
	return wizard.performCancel();
}
/**
 * Performs any actions appropriate in response to the user 
 * having pressed the Finish button, or refuse if finishing
 * now is not permitted.
 *
 * Normally this method is only called on the container's
 * current wizard. However if the current wizard is a nested wizard
 * this method will also be called on all wizards in its parent chain.
 * Such parents may use this notification to save state etc. However,
 * the value the parents return from this method is ignored.
 *
 * @return <code>true</code> to indicate the finish request
 *   was accepted, and <code>false</code> to indicate
 *   that the finish request was refused
 */
public final boolean performFinish() {
	return wizard.performFinish();
}
/**
 * Sets or clears the container of this wizard.
 *
 * @param wizardContainer the wizard container, or <code>null</code> 
 */
public final void setContainer(IWizardContainer wizardContainer) {
	wizard.setContainer(wizardContainer);	
}
}
