package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.net.*;
import java.io.*;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.scripting.internal.datamodels.*;
import org.eclipse.scripting.internal.*;
import org.eclipse.scripting.internal.uirenderer.XGR;
import org.eclipse.swt.widgets.*;
import java.util.*;

/**
 * This class is a default implementation of a script runner.
 * It can be used by workbench extensions to add scripting
 * capability.  The supported presentation types are represented by static
 * strings which are:
 * <ul>
 * <li>WIZARD, for a wizard dialog</li>
 * <li>PREFERENCES, for a preferences dialog</li>
 * <li>PREFERENCE_PAGE, for a preference dialog page<li>
 * <li>PANEL, for a displayed area within an editor view
 * </ul>
 * <p>
 * This class is primarily intended for use 
 * by developers who are script-enabling their workbench extensions.
 * </p>
 * <p>
 * This class is not intended to be subclassed by clients.
 * </p>
 
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see ScriptAbstractPreferencePage
 */
public class ScriptRunner {

	public static final String WIZARD          = "wizard";
	public static final String PREFERENCE_PAGE = "preferencepage";
	public static final String PREFERENCES     = "preferences";
	public static final String PANEL           = "panel";
	
	private IPluginDescriptor pluginDescriptor = null;
	private ScriptManager scriptManager = null;
	private ScriptToolkit scriptToolkit = null;
	private String strDataFilename = null;
	private String strLanguage = null;
	private String strPresentationFilename = null;
	private int iPresentationType = XGR.UNKNOWN;
	private String strFilename = null;
	private String strTranslationsFilename = null;
	private String strJavaHandler = null;
	private ScriptAdapterEventHandler scriptAdapterEventHandler = null;

	private XGR xgr = null;
	private boolean bBuildTime = false;
/**
 * Creates a script runner.  The parameters are passed in from a plugin's
 * IExecutableExtension.setInitializationData implementation.
 *
 * @param config the configuration element used to trigger this execution. 
 * @param objData adapter data in the form of a <code>String</code>, 
 *		a <code>Hashtable</code>, or <code>null</code>.
 */
public ScriptRunner(IConfigurationElement configElement, Object objData) {
		
	// Save the plugin descriptor
	//---------------------------
	IExtension extension = configElement.getDeclaringExtension();

	this.pluginDescriptor = extension.getDeclaringPluginDescriptor();

	// Parse the initialization data
	//------------------------------

	// String Form
	//------------
	if (objData instanceof String) {
		parseInitializationString((String) objData);
	}

	// Hashtable Form
	//---------------
	else if (objData instanceof Hashtable) {
		parseHashtableStrings((Hashtable) objData);
	}
}
/**
 * Converts a file name to a full URL path.
 *
 * @param filename 
 * @param pluginPath 
 */
private String convertFilenameToFullPath(String filename, String pluginPath) {

	// See if the filename is ok by itself
	// This could be an absolute file specification
	//---------------------------------------------
	java.io.File file = new java.io.File(filename);

	if (file.exists() == true) {
		return filename;
	}

	// See if the filename is relative to the plugin path
	//---------------------------------------------------
	file = new java.io.File(pluginPath + filename);

	if (file.exists() == true) {
		return pluginPath + filename;
	}

	// See if the filename is a URL
	//-----------------------------
	try {
		URL url = new URL(filename);
		filename = url.toExternalForm();
	}
	catch (java.net.MalformedURLException ex) {
	}

	// This filename will fail to be found later
	//------------------------------------------
	return filename;
}
/**
 * Converts a string to the equivalent integer primitive.
 * @param strPresentationType 
 * @return int
 */
private int convertUIType( String strPresentationType )
{
	int iPresentationType = XGR.WIZARD;
	
	if( strPresentationType.equals( "wizard" ) == true )
	{
		iPresentationType = XGR.WIZARD;
	}
	
	else if( strPresentationType.equals( "preferences" ) == true )
	{
		iPresentationType = XGR.PREFERENCES;
	}
	
	else if( strPresentationType.equals( "panel" ) == true )
	{
		iPresentationType = XGR.PANEL;
	}
	
	return iPresentationType;
}
/**
 * Executes a script without a user interface.  The script
 * is read from a file, or URL input stream into a string buffer, and the string
 * is evaluated as a script.
 */
protected void executeScript() {

	StringBuffer strbufLines = new StringBuffer();

	// Read the contents from a URL
	//-----------------------------
	URL url = null;
	InputStream inputStream = null;
	byte[] byteArray = new byte[10000];

	try {
		url = new URL(this.strFilename);

		inputStream = url.openStream();

		if (inputStream != null) {
			int iRead = 0;
			String strRead = null;
			do {
				iRead = inputStream.read(byteArray);

				if( iRead > 0 )
				{
				strRead = new String(byteArray,0, iRead);
				strbufLines.append(strRead);}
			}
			while (iRead > 0);
		}
	}
	catch (MalformedURLException ex) {
	}
	catch (IOException ex) {
	}

	// Read the contents as a script file
	//-----------------------------------
	if (strbufLines.length() <= 0) {
		FileReader fileReader = null;
		BufferedReader bufferedReader = null;

		try {
			fileReader = new FileReader(this.strFilename);
			bufferedReader = new BufferedReader(fileReader);
		}

		catch (IOException ex) {
			ScriptReporter.reportException(ex);
			bufferedReader = null;
		}

		if (bufferedReader != null) {

			String strLine = null;

			while (true) {

				try {
					strLine = bufferedReader.readLine();

					if (strLine == null) {
						break;
					}

					// Trace.string( strLine );

					strbufLines.append(strLine);
					strbufLines.append('\n');
				}
				catch (IOException ex) {
					ScriptReporter.reportException(ex);
					break;
				}
			}

			try {
				bufferedReader.close();
				fileReader.close();
			}
			catch (IOException ex) {
				ScriptReporter.reportException(ex);
			}
		}
	}

	// Evaluate the script
	//--------------------
	if (strbufLines.length() > 0) {
		this.scriptManager.evaluateScript(new ScriptDescriptor(strbufLines.toString(), this.strLanguage, this.strFilename, 0, 0), null);
	}
}
/**
 * Executes a script with a user interface.
 *
 * @param parent parent for scripted user interface.
 */
protected void executeScriptUI(Object parent) {

	// Create a new work object
	//-------------------------
	this.xgr = new XGR();

	// Set runtime/buildtime mode
	//---------------------------
	this.xgr.setBuildTime(this.bBuildTime);

	if (this.bBuildTime == false) {
		// Set the script manager with the renderer
		//-----------------------------------------
		this.xgr.setScriptManager(this.scriptManager);

		// Give the renderer to the toolkit object
		//----------------------------------------
		if (this.scriptToolkit != null && this.scriptToolkit instanceof ScriptToolkitUI) {

			((ScriptToolkitUI) this.scriptToolkit).setGuiRenderer(this.xgr);

			// Set the java event handler if there is one
			//-------------------------------------------
			if (this.scriptAdapterEventHandler != null) {

				this.scriptAdapterEventHandler.setToolkit((ScriptToolkitUI) this.scriptToolkit);

				Vector vectorObjectDescriptors = new Vector();
				vectorObjectDescriptors.addElement(new ScriptObjectDescriptor("javahandler", this.scriptAdapterEventHandler));
				this.xgr.setJavaObjectDescriptors(vectorObjectDescriptors);
				this.scriptManager.setJavaMode(true);
			}
		}
	}

	// Set the UI presentation filename
	//---------------------------------
	try {
		this.xgr.setPresentationFilename(this.strPresentationFilename);
	}
	catch (Exception ex) {
		ScriptReporter.reportException(ex);
	}

	// Set the UI type
	//----------------
	if( this.iPresentationType == XGR.UNKNOWN )
		this.iPresentationType = XGR.WIZARD;
		
	this.xgr.setPresentationType(this.iPresentationType);

	// Set the NL translation file
	//----------------------------
	this.xgr.setTranslationsFilename(this.strTranslationsFilename);

	// Set the data model domain
	//--------------------------
	if (this.bBuildTime == false && this.strDataFilename != null) {

		try {
			XGRDataModelFactory factory = new XGRDataModelFactory();
			IXGRDataModel dataModel = factory.getDataModel(this.strDataFilename);
			this.xgr.setDataModel(dataModel);
		}

		catch (Exception ex) {
		}
	}

	// Show the UI model with the data model
	//--------------------------------------
	this.xgr.execute(parent, true);
}
/**
 * Returns the presentation file name which contains the user interface definition.
 *
 * @return file name of the UI definition
 */
public String getPresentationFilename() {
	return this.strPresentationFilename;
}
/**
 * Returns the filename that contains script code.
 *
 * @return file name of the script
 */
public String getScriptFilename() {
	return this.strFilename;
}
/**
 * Returns the top level widget that is rendered.
 *
 * @return widget
 */
public Object getWidget() {
	return this.xgr.getWidget();
}
/**
 * Creates a script manager, a script toolkit, registers
 * the script globals, and the selected resources with the script manager.  The
 * selected resources are given to the script.
 *
 * @param resourceSelectionArray selected resources
 */
public void initialize(IResource[] resourceSelectionArray) {
	
	// Create a script manager
	//------------------------
	if (this.bBuildTime == false) {
		
		try {
			this.scriptManager = new ScriptManager( this.pluginDescriptor.getPluginClassLoader() );
		}
		catch (Exception ex) {
			ScriptReporter.reportException(ex);
		}

		// Obtain the plugin
		//------------------
		Plugin plugin = null;

		try {
			if (this.pluginDescriptor != null) {
				plugin = this.pluginDescriptor.getPlugin();
			}
		}
		catch (CoreException ex) {
			ScriptReporter.reportException(ex);
		}

		// Create script toolkit
		//----------------------
		ScriptToolkit scriptToolkit = null;

		if (this.strPresentationFilename != null) {
			this.scriptToolkit = new ScriptToolkitUI(plugin, resourceSelectionArray);
		}
		else {
			this.scriptToolkit = new ScriptToolkit(plugin, resourceSelectionArray);
		}

		// Give the toolkit to the script manager
		//---------------------------------------
		if( this.scriptManager != null ) {
			this.scriptManager.setToolkit( this.scriptToolkit );
		}

		// Determine if there is a plugin instance
		//----------------------------------------
		ScriptPlugin scriptPlugin = ScriptPlugin.getPluginInstance();

		// Activate the script plugin if not activated
		//--------------------------------------------
		if( scriptPlugin == null )
		{
			scriptPlugin = ScriptPlugin.bootstrap();
		}

		// Register objects with the script manager
		// Plugin parameter is allowed to be null
		//-----------------------------------------
		if (scriptPlugin != null && this.scriptManager != null) {
			
			// Register the global variables and functions with the script manager
			// Including the AbstractPlugin
			// The function accepts a null plugin
			//-----------------------------------
			scriptPlugin.registerScriptGlobals(this.scriptManager, plugin, this.scriptToolkit);
		}

		// Register the selected elements
		//-------------------------------
		if (this.scriptManager != null && resourceSelectionArray != null) {
			this.scriptManager.declareElement("selection", resourceSelectionArray);
		}
	}

	return;
}
/**
 * Parses the parameters specified in a script adapter extension.
 * This includes file, language, ui, uitype, translations and data.
 *
 * @param hashProperties script properties
 */
protected void parseHashtableStrings(Hashtable hashProperties) {


	if (hashProperties != null) {
		// Get the installed location of the extension
		//--------------------------------------------
		URL url = this.pluginDescriptor.getInstallURL();

		String strPluginPath = url.getFile();

		// Filename
		//---------
		if (hashProperties.containsKey("file") == true) {
			this.strFilename = (String) hashProperties.get("file");
			this.strFilename = convertFilenameToFullPath(this.strFilename, strPluginPath);
		}

		// Language
		//---------
		if (hashProperties.containsKey("language") == true) {
			this.strLanguage = (String) hashProperties.get("language");
		}

		// User Interface
		//---------------
		if (hashProperties.containsKey("ui") == true) {
			this.strPresentationFilename = (String) hashProperties.get("ui");
			this.strPresentationFilename = convertFilenameToFullPath(this.strPresentationFilename, strPluginPath);
		}

		// User Interface type
		//--------------------
		if (hashProperties.containsKey("uitype") == true) {
			String strPresentationType = (String) hashProperties.get("uitype");
			this.iPresentationType = convertUIType(strPresentationType);
		}

		// National Language translation strings
		//--------------------------------------
		if (hashProperties.containsKey("translations") == true) {
			this.strTranslationsFilename = (String) hashProperties.get("translations");
			this.strTranslationsFilename = convertFilenameToFullPath(this.strTranslationsFilename, strPluginPath);
		}

		// Data
		//-----
		if (hashProperties.containsKey("data") == true) {
			this.strDataFilename = (String) hashProperties.get("data");
			this.strDataFilename = convertFilenameToFullPath(this.strDataFilename, strPluginPath);
		}

		// Java Handler
		//-------------
		if (hashProperties.containsKey("javahandler") == true) {
			this.strJavaHandler = (String) hashProperties.get("javahandler");
		}
	}
}
/**
 * This method parses the initialization string.
 * <br/>
 * Examples:
 * <ul>
 * <li>file=sample.js</li>
 * <li>ui=sampleui.xhtml uitype="wizard"</li>
 * <li>ui=sampleui.xhtml data=sampledata.xml</li>
 * <li>translations=abc.properties</li>
 * </ul>
 *
 * @param strInitialization initialization string
 */
protected void parseInitializationString(String strInitialization) {

	if (strInitialization != null) {

		// Initialize string buffers
		//--------------------------
		String strPresentationType = null;

		StringBuffer strbFilename = null;
		StringBuffer strbDataFilename = null;
		StringBuffer strbJavaHandler = null;
		StringBuffer strbLanguage = null;
		StringBuffer strbPresentationFilename = null;
		StringBuffer strbPresentationType = null;
		StringBuffer strbTranslationsFilename = null;
		StringBuffer strbUserInterface = null;

		StringBuffer strbCurrent = null;

		// Create a parser
		//----------------
		StringTokenizer st = new StringTokenizer(strInitialization);

		int iNumberOfTokens = st.countTokens();

		String strToken = null;

		// Initialize string buffers with token strings
		//---------------------------------------------
		for (int i = 0; i < iNumberOfTokens; ++i) {
			
			strToken = st.nextToken();

			// File
			//-----
			if (strToken.indexOf("file=") == 0 && strToken.length() > 5) {
				strbFilename = new StringBuffer(strToken.substring(5));
				strbCurrent = strbFilename;
			}

			// Language
			//---------
			else if (strToken.indexOf("language=") == 0 && strToken.length() > 9) {
				strbLanguage = new StringBuffer(strToken.substring(9));
				strbCurrent = strbLanguage;
			}

			// User Interface
			//---------------
			else if (strToken.indexOf("ui=") == 0 && strToken.length() > 3) {
				strbPresentationFilename = new StringBuffer(strToken.substring(3));
				strbCurrent = strbPresentationFilename;
			}

			// User Interface Type
			//--------------------
			else if (strToken.indexOf("uitype=") == 0 && strToken.length() > 7) {
				strbPresentationType = new StringBuffer(strToken.substring(7));
				strbCurrent = strbPresentationType;
			}

			// Data
			//-----
			else if (strToken.indexOf("data=") == 0 && strToken.length() > 5) {
				strbDataFilename = new StringBuffer(strToken.substring(5));
				strbCurrent = strbDataFilename;
			}

			// Translations
			//-------------
			else if (strToken.indexOf("translations=") == 0 && strToken.length() > 13) {
				strbTranslationsFilename = new StringBuffer(strToken.substring(13));
				strbCurrent = strbTranslationsFilename;
			}

			// javahandler
			//------------
			else if (strToken.indexOf("javahandler=") == 0 && strToken.length() > 12) {
				strbJavaHandler = new StringBuffer(strToken.substring(12));
				strbCurrent = strbJavaHandler;
			}

			// Handle space characters within an attribute
			// The token is a part of the current string buffer
			//-------------------------------------------------
			else if (strbCurrent != null) {
				strbCurrent.append(" ");
				strbCurrent.append(strToken);
			}
		}

		// Get the installed location of the extension
		//--------------------------------------------
		URL url = this.pluginDescriptor.getInstallURL();

		String strPluginPath = url.toExternalForm();

		// Convert string buffers and store the resulting properties
		//----------------------------------------------------------
		if (strbFilename != null) {
			this.strFilename = convertFilenameToFullPath(strbFilename.toString(), strPluginPath);
		}
		
		if (strbDataFilename != null) {
			this.strDataFilename = convertFilenameToFullPath(strbDataFilename.toString(), strPluginPath);
		}

		if (strbJavaHandler != null) {
			this.strJavaHandler = strbJavaHandler.toString();
		}
		
		if (strbLanguage != null) {
			this.strLanguage = strbLanguage.toString();
		}
		
		if (strbPresentationType != null) {
			this.iPresentationType = convertUIType(strbPresentationType.toString());
		}

		if (strbPresentationFilename != null) {
			this.strPresentationFilename = convertFilenameToFullPath(strbPresentationFilename.toString(), strPluginPath);
		}

		if (strbTranslationsFilename != null) {
			this.strTranslationsFilename = convertFilenameToFullPath(strbTranslationsFilename.toString(), strPluginPath);
		}		
	}
}
/**
 * If a script filename has been previously set, then the script is executed.
 * Otherwise, if a presentation filename has been previously been set, then the
 * user interface contained within the file is rendered.
 *
 * @param compositeParent parent user interface composite
 */
public void run(Composite compositeParent) {

	// Run a script
	//-------------
	if (this.strFilename != null) {
		executeScript();
	}

	// Run a script with user interface
	//---------------------------------
	if (this.strPresentationFilename != null) {
		executeScriptUI(compositeParent);
	}
}
/**
 * This sets the build time state in order to turn off execution of
 * scripts.
 *
 * @param bBuildTime <code>true</code> is scripts in UI definitial are to be ignored
 */
public void setBuildTime(boolean bBuildTime) {
	this.bBuildTime = bBuildTime;
}
/**
 * Sets the data model file name.
 *
 * @param filename 
 */
public void setDataFilename(String filename) {
	this.strDataFilename = filename;
}
/**
 * Sets the presentation file name which contains a user interface definition.
 *
 * @param filename 
 */
public void setPresentationFilename(String filename) {
	this.strPresentationFilename = filename;
}
/**
 * Sets the presentation user interface type to be renderered.  This may be one of&colon.
 * <ul>
 * <li>WIZARD</li>
 * <li>PREFERENCE_PAGE</li>
 * <li>PREFERENCES</li>
 * <li>PANEL</li>
 * </ul>
 *
 * @param strPresentationType presentation type
 */
public void setPresentationType(String strPresentationType) {

	// Convert string to type
	//-----------------------
	     if( strPresentationType.equals( WIZARD          ) == true ) this.iPresentationType = XGR.WIZARD;
	else if( strPresentationType.equals( PREFERENCE_PAGE ) == true ) this.iPresentationType = XGR.PREFERENCEPAGE;
	else if( strPresentationType.equals( PREFERENCES     ) == true ) this.iPresentationType = XGR.PREFERENCES;
	else if( strPresentationType.equals( PANEL           ) == true ) this.iPresentationType = XGR.PANEL;
	else this.iPresentationType = XGR.UNKNOWN;
}
/**
 * Associates an event handler with the adaptor
 *
 * @param handler script adaptor event handler
 */
public void setScriptAdapterEventHandler(ScriptAdapterEventHandler handler) {
	
	this.scriptAdapterEventHandler = handler;
}
/**
 * Sets the filename that contains script code.
 *
 * @param filename
 */
public void setScriptFilename(String filename) {
	this.strFilename = filename;
}
/**
 * Sets the script language to be used.  Ths string 'javascript' is
 * currently the only language supported, and is the default script language.
 *
 * @param language
 */
public void setScriptLanguage(String language) {
	this.strLanguage = language;
}
/**
 * Sets the root properties file name that contains translation strings.  The organization
 * of transation files is the same as property bundle files.  A string to
 * be translated is prefixed with a '%' character.
 *
 * @param translationFilename
 */
public void setTranslationsFilename(String filename) {
	this.strTranslationsFilename = filename;
}
}
