package org.eclipse.scripting.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This class handles calling all user defined script language functions,
 * and java functions.
 */
import org.eclipse.scripting.*;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;
import org.eclipse.core.runtime.*;

public class ScriptManager {
	private String _strDefaultScriptLanguage = new String("javascript");
	private Vector _vectorObjectDescriptors = null;
	private IToolkit _toolkit = null;
	private boolean _bJavaMode = false;
	private boolean _bJavascriptLoaded = false;
	private Method _methodEval = null;
	private Method _methodDeclareBean = null;
	private Method _methodUndeclareBean = null;
	private Object _objAdapter = null;
	private static HashMap _loaderTable = new HashMap();
	/**
	 * ScriptManager constructor
	 */
	public ScriptManager() {
	}
	/**
	 * ScriptManager constructor
	 */
	public ScriptManager(ClassLoader classLoaderParent) {

		// Prepare JavaScript engine adapter
		if (classLoaderParent != null) {
			try {
				// Obtain the JavaScript engine adapter plugin
				IPluginRegistry registry = Platform.getPluginRegistry();
				IPluginDescriptor descriptor = registry.getPluginDescriptor("org.eclipse.scripting.adapters.javascript");

				// Create a class loader for the jars using the parent class loader
				if (descriptor != null) {

					URLClassLoader classLoader;
					URLClassLoader cachedLoader = (URLClassLoader)_loaderTable.get(classLoaderParent); // get loader for parent context
					if (cachedLoader == null) {
						
						// Use the adapter plugin class loader to obtain fully resolved classpath
						// (runtime entries, VA projects, dev mode entries, fragment entries).
						// Create a new URL loader using these entries PLUS properly parented.
						// NOTE: the new loader will NOT cause plugin activation of the adapter plugin.
						URLClassLoader pluginLoader = (URLClassLoader) descriptor.getPluginClassLoader();
						URL[] cp = pluginLoader.getURLs();
						classLoader = new URLClassLoader(cp, classLoaderParent);
					}
					else classLoader = cachedLoader;						

					// Obtain the JavaScript engine adapter class				
					Class classAdapter = classLoader.loadClass("org.eclipse.scripting.adapters.javascript.ScriptEngineJavaScript");

					if (classAdapter != null) {

						// Create an instance of the JavaScript engine adapter class
						_objAdapter = classAdapter.newInstance();

						// Call its initialize method
						Class[] classArray = new Class[0];
						Method methodInitialize = classAdapter.getMethod("initialize", classArray);
						Object[] objArray = new Object[0];
						Object objReturn = methodInitialize.invoke(_objAdapter, objArray);

						// Store its eval method
						classArray = new Class[4];
						classArray[0] = String.class;
						classArray[1] = Integer.class;
						classArray[2] = Integer.class;
						classArray[3] = Object.class;
						_methodEval = classAdapter.getMethod("eval", classArray);

						// Store its declareBean method
						classArray = new Class[2];
						classArray[0] = String.class;
						classArray[1] = Object.class;
						_methodDeclareBean = classAdapter.getMethod("declareBean", classArray);

						// Store its unDeclareBean method
						classArray = new Class[1];
						classArray[0] = String.class;
						_methodUndeclareBean = classAdapter.getMethod("undeclareBean", classArray);
					}
					if (cachedLoader == null) _loaderTable.put(classLoaderParent,classLoader); // cache the loader if all went well
				}
			}
			catch (ClassNotFoundException ex) {
			}
			catch (IllegalAccessException ex) {
			}
			catch (InstantiationException ex) {
			}
			catch (InvocationTargetException ex) {
			}
			catch (NoSuchMethodException ex) {
			}
		}
	}
	/**
	 * 
	 * @param strElementName java.lang.String
	 * @param objElement java.lang.Object
	 */
	public void declareElement(String strElementName, Object objElement) {

		if (_objAdapter != null && _methodDeclareBean != null) {

			Object[] objArray = new Object[2];
			objArray[0] = strElementName;
			objArray[1] = objElement;

			try {
				Object objReturn = _methodDeclareBean.invoke(_objAdapter, objArray);
			}
			catch (IllegalAccessException ex) {
			}
			catch (InvocationTargetException ex) {
			}
		}
	}
	/**
	 * This method invokes a static class method, or a method on an existing java object.
	 * This method takes a script string and assumes that it is a class and
	 * method name in the form "a.b.c.methodname", or an object name and method
	 * name in the form "name.methodname.  The string is separated
	 * into a class name string, or object name string and a method name string.  A class instance is
	 * created if required, and the method invoked.
	 * 
	 * @param scriptDescriptor com.ibm.itp.scripting.core.ScriptDescriptor
	 */
	public void evaluateJava(ScriptDescriptor scriptDescriptor) {

		// Separate the package/class from the method name
		//------------------------------------------------
		String strClassName = null;
		String strMethodName = null;

		// Obtain the method text as determined by the script descriptor object
		// In java mode, this will be in _strScriptText
		// In non-java mode, this will be in _strJavaMethod
		//---------------------------------------------------------------------
		String strMethod = scriptDescriptor._strJavaMethod != null ? scriptDescriptor._strJavaMethod : scriptDescriptor._strScriptText;

		// Separate the method name and class name
		//----------------------------------------
		if (strMethod != null) {
			// Eliminate beginning / trailing white space
			//-------------------------------------------
			String strSpecification = strMethod.trim();

			// Look for the last period separating the
			// class name from the method name		
			//----------------------------------------
			int iIndex = strSpecification.lastIndexOf(".");

			if (iIndex > -1) {
				strClassName = strSpecification.substring(0, iIndex);

				if (iIndex < strSpecification.length()) {
					strMethodName = strSpecification.substring(iIndex + 1);
				}
			}

			// If there is no class name, the assume a java handler
			//-----------------------------------------------------
			else {
				strClassName = "javahandler";
				strMethodName = strSpecification;
			}
		}

		// If a method name exists, eliminate the parameters (...)
		//--------------------------------------------------------
		int iIndex = strMethodName.indexOf("(");
		if (iIndex > -1) {
			strMethodName = strMethodName.substring(0, iIndex);
		}

		// Try to find the java object
		// If a java object is found, call its member function
		// Otherwise assume that it is a java static member function and call it
		//----------------------------------------------------------------------
		if (strClassName != null && strMethodName != null) {
			// See if this is an existing object
			//----------------------------------
			Object objectJava = null;

			if (_vectorObjectDescriptors != null) {
				ScriptObjectDescriptor objectDescriptor = null;

				for (int i = 0; i < _vectorObjectDescriptors.size(); ++i) {
					objectDescriptor = (ScriptObjectDescriptor) _vectorObjectDescriptors.elementAt(i);

					if (objectDescriptor.name != null && objectDescriptor.name.equals(strClassName) == true) {
						objectJava = objectDescriptor.object;
					}
				}
			}

			// Java object method
			//-------------------
			if (objectJava != null) {
				evaluateJavaObjectMethod(objectJava, strMethodName, scriptDescriptor);
			}

			// Java static method
			//-------------------
			else {
				evaluateJavaClassMethod(strClassName, strMethodName, scriptDescriptor);
			}
		}
	}
	/**
	 * 
	 */
	public void evaluateJavaClassMethod(String strClassName, String strMethodName, ScriptDescriptor scriptDescriptor) {

		// Create a class instance
		//------------------------
		Class classInstance = null;

		try {
			classInstance = Class.forName(strClassName);
		}
		catch (Exception ex) {
			ScriptReporter.reportJavaEvaluationException(ex, scriptDescriptor);
			classInstance = null;
		}

		// Look for the class method
		//--------------------------
		Method methodJava = null;

		if (classInstance != null) {
			try {
				methodJava = classInstance.getMethod(strMethodName, null);
			}
			catch (Exception ex) {
				ScriptReporter.reportJavaEvaluationException(ex, scriptDescriptor);
				methodJava = null;
			}
		}

		// Invoke the method without parameters
		//-------------------------------------
		if (methodJava != null) {
			try {
				methodJava.invoke(null, null);
			}
			catch (Exception ex) {
				ScriptReporter.reportJavaEvaluationException(ex, scriptDescriptor);
			}
		}
	}
	/**
	 * 
	 */
	public void evaluateJavaObjectMethod(Object objectJava, String strMethodName, ScriptDescriptor scriptDescriptor) {

		// Obtain the class instance
		//--------------------------
		Class classInstance = objectJava.getClass();

		// Look for the class method
		//--------------------------
		Method methodJava = null;

		if (classInstance != null) {
			try {
				methodJava = classInstance.getMethod(strMethodName, null);
			}
			catch (Exception ex) {
				ScriptReporter.reportJavaEvaluationException(ex, scriptDescriptor);
				methodJava = null;
			}
		}

		// Invoke the method without parameters
		//-------------------------------------
		if (methodJava != null) {
			try {
				methodJava.invoke(objectJava, null);
			}
			catch (Exception ex) {
				ScriptReporter.reportJavaEvaluationException(ex, scriptDescriptor);
			}
		}
	}
	/**
	 * 
	 */
	public void evaluateLanguageScript(String strLanguage, ScriptDescriptor scriptDescriptor) {

		if (strLanguage.equals("javascript") == false) {
			return;
		}

		// Evaluate the script
		//--------------------
		if (_objAdapter != null && _methodEval != null) {

			Object[] objArray = new Object[4];
			objArray[0] = scriptDescriptor._strFilename;
			objArray[1] = new Integer( scriptDescriptor._iLineNumber );
			objArray[2] = new Integer( scriptDescriptor._iColumnNumber );
			objArray[3] = scriptDescriptor._strScriptText;
			
			try {
				Object objReturn = _methodEval.invoke(_objAdapter, objArray);
			}
			catch (IllegalAccessException ex) {
			    ScriptReporter.reportScriptEvaluationException( ex, scriptDescriptor );
			}
			catch (InvocationTargetException ex) {
			    ScriptReporter.reportScriptEvaluationException( ex, scriptDescriptor );
			}
		}

		/*
			try {
				_engineJavaScript.eval(scriptDescriptor._strFilename, scriptDescriptor._iLineNumber, scriptDescriptor._iColumnNumber, scriptDescriptor._strScriptText);
			}
			catch (ScriptEngineException ex) {
			    ScriptReporter.reportScriptEvaluationException( ex, scriptDescriptor );
			}
		*/
		return;
	}
	/**
	 * Evaluates a script.
	 */
	public void evaluateScript(ScriptDescriptor scriptDescriptor, Object objEventWidget) {

		if (scriptDescriptor != null) {

			// Give the event widget to the toolkit
			//-------------------------------------
			if (_toolkit instanceof ScriptToolkitUI) {
				((ScriptToolkitUI) _toolkit).setEventWidget(objEventWidget);
			}

			// Java only
			//----------
			if (_bJavaMode == true) {
				evaluateJava(scriptDescriptor);
			}

			// A script language other than java
			//----------------------------------
			else {
				// If no language is specified, use the default language
				//------------------------------------------------------
				String strLanguage = null;

				if (scriptDescriptor._strScriptLanguage != null) {
					strLanguage = scriptDescriptor._strScriptLanguage;

					if (strLanguage.length() == 0) {
						strLanguage = null;
					}
				}

				if (strLanguage == null) {
					strLanguage = _strDefaultScriptLanguage;
				}

				// Convert "text/xxx" to "xxx" from xhtml files
				//---------------------------------------------
				if (strLanguage.length() > 5) {
					int iIndex = strLanguage.toLowerCase().indexOf("text/");

					if (iIndex >= 0) {
						strLanguage = strLanguage.substring(iIndex + 5);
					}
				}

				// If we have a language, evaluate the script
				//-------------------------------------------
				if (strLanguage != null) {
					
					// Java
					//-----
					if (strLanguage.toLowerCase().equals("java") == true) {
						if (scriptDescriptor._strJavaMethod != null) {
							evaluateJava(scriptDescriptor);
						}
					}

					// Script Language
					//----------------
					else {
						if (scriptDescriptor._strScriptText != null) {
							evaluateLanguageScript(strLanguage, scriptDescriptor);
						}
					}
				}
			}
		}
	}
	/**
	 *
	 */
	public void setDefaultScriptLanguage(String strDefaultScriptLanguage) {
		_strDefaultScriptLanguage = strDefaultScriptLanguage;
	}
	/**
	 * 
	 * @param bJavaMode boolean
	 */
	public void setJavaMode(boolean bJavaMode) {
		_bJavaMode = bJavaMode;
	}
	/**
	 * 
	 */
	public void setJavaObjectDescriptors(Vector vectorObjectDescriptors) {
		_vectorObjectDescriptors = vectorObjectDescriptors;
	}
	/**
	 * Sets the toolkit
	 */
	public void setToolkit(IToolkit toolkit) {
		_toolkit = toolkit;
	}
	/**
	 * 
	 */
	public void undeclareElement(String strElementName) {

		if (_objAdapter != null && _methodUndeclareBean != null) {

			Object[] objArray = new Object[1];
			objArray[0] = strElementName;

			try {
				Object objReturn = _methodUndeclareBean.invoke(_objAdapter, objArray);
			}
			catch (IllegalAccessException ex) {
			}
			catch (InvocationTargetException ex) {
			}
		}
	}
}
