package org.eclipse.scripting.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.net.*;
/**
 * Handles obtaining translated national language strings from a properties
 * file.
 */
import java.io.*;
import java.util.*;
import java.util.ResourceBundle;
import java.util.MissingResourceException;

public class ScriptNLTranslationManager {
	private ResourceBundle _resourceBundle = null;
/**
 * Constructor saves the base name of the properties file without the locale
 * suffix.
 */
public ScriptNLTranslationManager( String strBaseFilename )
{
	setPropertiesFile( strBaseFilename );
}
/**
 * Attempts to create an input stream from a file name.  If successful
 * an attempt is made to create a resource bundle from the input stream.
 */
private ResourceBundle getResourceBundleFromFilename(String strFilename) {

	ResourceBundle resourceBundle = null;

	InputStream inputStream = null;

	// Attempt to open as a URL
	//-------------------------
	URL url = null;

	try {
		url = new URL(strFilename);

		inputStream = url.openStream();
	}
	catch (MalformedURLException ex) {
		inputStream = null;
	}
	catch (IOException ex) {
		inputStream = null;
	}

	// Attempt to open as a file
	//--------------------------
	if (inputStream == null) {
		try {
			inputStream = new FileInputStream(strFilename);
		}
		catch (FileNotFoundException ex) {
			inputStream = null;
		}
	}
	if (inputStream != null) {
		try {
			resourceBundle = new PropertyResourceBundle(inputStream);
		}
		catch (Exception e) {
			resourceBundle = null;
		}
	}

	return resourceBundle;
}
/**
 * Given a key, this function returns the associated translated string.
 * The string is trimmed of leading and trailing white space first.
 */
public String getTranslatedString( String strKey )
{
	String strKeyTrimmed = strKey.trim();
	
	if( _resourceBundle != null )
	{
		try
		{
			return _resourceBundle.getString( strKeyTrimmed );
		}
		catch( MissingResourceException ex )
		{
		}
	}

	return strKeyTrimmed;
}
/**
 * Attempts to create a resource bundle from a base filename.  The
 * search order is:
 * <ul>
 * <li>baseFilename_locale.properties</li>
 * <li>baseFilename.properties</li>
 * </ul>
 * where locale is the default locale for the system.
 */
public void setPropertiesFile( String strBaseFilename )
{
	// Check input parameter
	//----------------------
	if( strBaseFilename == null || strBaseFilename.length() == 0 )
	{
		return;
	}
	
	// Strip off any trailing ".properties"
	//-------------------------------------
	int iIndex = strBaseFilename.lastIndexOf( ".properties" );
	if( iIndex > 0 )
	{
		strBaseFilename = strBaseFilename.substring( 0, iIndex );
	}

	// Attempt to find baseFilename_locale.properties
	//-----------------------------------------------	
	StringBuffer strbFilename = new StringBuffer();

	Locale locale = Locale.getDefault();

	// Full language and country
	//--------------------------
	if( locale.toString() != null && locale.toString().length() > 0 )
	{
		strbFilename.setLength( 0 );
		strbFilename.append( strBaseFilename );
		strbFilename.append( "_" );
		strbFilename.append( locale.toString() );
		strbFilename.append( ".properties" );
		
		_resourceBundle = getResourceBundleFromFilename( strbFilename.toString() );		
	}

	// Language only
	//--------------	
	if( _resourceBundle == null && locale.getLanguage() != null && locale.getLanguage().length() > 0 )
	{
		strbFilename.setLength( 0 );
		strbFilename.append( strBaseFilename );
		strbFilename.append( "_" );
		strbFilename.append( locale.getLanguage() );
		strbFilename.append( ".properties" );

		_resourceBundle = getResourceBundleFromFilename( strbFilename.toString() );
	}

	// If not found, attempt to find baseFilename.properties
	//------------------------------------------------------
	if( _resourceBundle == null )
	{
		strbFilename.setLength( 0 );
		strbFilename.append( strBaseFilename );
		strbFilename.append( ".properties" );
		
		_resourceBundle = getResourceBundleFromFilename( strbFilename.toString() );
	}
}
}
