package org.eclipse.scripting.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.*;
import org.eclipse.core.runtime.model.*;
import java.util.*;
import java.net.*;
/**
 * This class reflects the static methods of the Platform class.  This allows scripters
 * to access methods of the Platform class through an object.
 */
import org.eclipse.core.runtime.*;

public class ScriptPlatform {
/**
 * ScriptPlatform constructor comment.
 */
public ScriptPlatform() {
}
/**
 * Adds the given authorization information to the keyring. The
 * information is relevant for the specified protection space and the
 * given authorization scheme. The protection space is defined by the
 * combination of the given server URL and realm. The authorization 
 * scheme determines what the authorization information contains and how 
 * it should be used. The authorization information is a <code>Map</code> 
 * of <code>String</code> to <code>String</code> and typically
 * contain information such as usernames and passwords.
 *
 * @param serverUrl the URL identifying the server for this authorization
 *		information. For example, "http://www.hostname.com/".
 * @param realm the subsection of the given server to which this
 *		authorization information applies.  For example,
 *		"realm1@hostname.com" or "" for no realm.
 * @param authScheme the scheme for which this authorization information
 *		applies. For example, "Basic" or "" for no authorization scheme
 * @param info a <code>Map</code> containing authorization information 
 *		such as usernames and passwords
 * @exception CoreException if there are problems setting the
 *		authorization information. Reasons include:
 * <ul>
 * <li>The keyring could not be saved.</li>
 * </ul>
 */
public void addAuthorizationInfo(URL serverUrl, String realm, String authScheme, Map info) throws CoreException {
	Platform.addAuthorizationInfo(serverUrl, realm, authScheme, info);
}
/** 
 * Adds the given log listener under the given name to the notification list
 * of the platform. There can be only one registered listener
 * for a given name. If there is a listener registered against that name
 * it is replaced.
 * <p>
 * Once registered, a listener starts receiving notification as entries
 * are added to the log via <code>log</code>. The listener continues to
 * receive notifications until it is replaced or removed.
 * </p>
 * <p>
 * Note: This facility was added to support the ISV toolkit, which
 * contains a special ISV debug tool that can display the log---live.
 * It should not be used for other purposes.
 * XXX is this still true?
 * </p>
 * <p>
 * The platform must be running.
 * </p>
 *
 * @param listener the listener to register
 * @see ILogListener
 * @see #removeLogListener
 */
public void addLogListener(ILogListener listener) {
	Platform.addLogListener( listener );
}
/**
 * Adds the specified resource to the protection space specified by the
 * given realm. All resources at or deeper than the depth of the last
 * symbolic element in the path of the given resource URL are assumed to
 * be in the same protection space.
 *
 * @param resourceUrl the URL identifying the resources to be added to
 *		the specified protection space. For example,
 *		"http://www.hostname.com/folder/".
 * @param realm the name of the protection space. For example,
 *		"realm1@hostname.com"
 * @exception CoreException if there are problems setting the
 *		authorization information. Reasons include:
 * <ul>
 * <li>The keyring could not be saved.</li>
 * </ul>
 */
public void addProtectionSpace(URL resourceUrl, String realm) throws CoreException {
	Platform.addProtectionSpace(resourceUrl, realm);
}
/**
 * Returns a URL which is the local equivalent of the
 * supplied URL. This method is expected to be used with 
 * plug-in-relative URLs returned by IPluginDescriptor.
 * If the specified URL is not a plug-in-relative URL, it 
 * is returned asis. If the specified URL is a plug-in-relative
 * URL of a file (incl. .jar archive), it is returned as 
 * a locally-accessible URL using "file:" or "jar:file:" protocol
 * (caching the file locally, if required). If the specified URL
 * is a plug-in-relative URL of a directory,
 * an exception is thrown.
 *
 * @param url original plug-in-relative URL.
 * @return the resolved URL
 * @exception IOException if unable to resolve URL
 * @see #resolve
 * @see IPluginDescriptor#getInstallURL
 */
public URL asLocalURL(URL url) throws IOException {
	return Platform.asLocalURL(url);
}
/**
 * Removes the authorization information for the specified protection
 * space and given authorization scheme. The protection space is defined
 * by the given server URL and realm.
 *
 * @param serverUrl the URL identifying the server to remove the
 *		authorization information for. For example,
 *		"http://www.hostname.com/".
 * @param realm the subsection of the given server to remove the
 *		authorization information for. For example,
 *		"realm1@hostname.com" or "" for no realm.
 * @param authScheme the scheme for which the authorization information
 *		to remove applies. For example, "Basic" or "" for no
 *		authorization scheme.
 * @exception CoreException if there are problems removing the
 *		authorization information. Reasons include:
 * <ul>
 * <li>The keyring could not be saved.</li>
 * </ul>
 */
public void flushAuthorizationInfo(URL serverUrl, String realm, String authScheme) throws CoreException {
	Platform.flushAuthorizationInfo(serverUrl, realm, authScheme);
}
/**
 * Returns the adapter manager used for extending
 * <code>IAdaptable</code> objects.
 * The platform must be running.
 *
 * @return the adapter manager for this platform
 * @see IAdapterManager
 */
public IAdapterManager getAdapterManager() {
	return Platform.getAdapterManager();
}
/**
 * Returns the authorization information for the specified protection
 * space and given authorization scheme. The protection space is defined
 * by the given server URL and realm. Returns <code>null</code> if no
 * such information exists.
 *
 * @param serverUrl the URL identifying the server for the authorization
 *		information. For example, "http://www.hostname.com/".
 * @param realm the subsection of the given server to which the
 *		authorization information applies.  For example,
 *		"realm1@hostname.com" or "" for no realm.
 * @param authScheme the scheme for which the authorization information
 *		applies. For example, "Basic" or "" for no authorization scheme
 * @return the authorization information for the specified protection
 *		space and given authorization scheme, or <code>null</code> if no
 *		such information exists
 */
public Map getAuthorizationInfo(URL serverUrl, String realm, String authScheme) {
	return Platform.getAuthorizationInfo(serverUrl, realm, authScheme);
}
/**
 * Returns the command line args provided to the platform when it was first run.
 * Note that individual platform runnables may be provided with different arguments
 * if they are being run individually rather than with <code>Platform.run()</code>.
 * 
 * @return the command line used to start the platform
 */
public String[] getCommandLineArgs() {
	return Platform.getCommandLineArgs();
}
/**
 * Returns the identified option.  <code>null</code>
 * is returned if no such option is found.   Options are specified
 * in the general form <code>&ltplug-in id&gt/&ltoption-path&gt</code>.  
 * For example, <code>org.eclipse.core.runtime/debug</code>
 *
 * @param option the name of the option to lookup  
 */
public String getDebugOption(String option) {
	return Platform.getDebugOption(option);
}
/**
 * Returns the location of the platform.
 * The platform must be running.
 */
public IPath getLocation() {
	return Platform.getLocation();
}
/**
 * Returns the plug-in runtime object for the identified plug-in
 * or <code>null</code> if no such plug-in can be found.  If
 * the plug-in is defined but not yet activated, the plug-in will
 * be activated before being returned.
 * The platform must be running.
 *
 * @param id the unique identifier of the desired plugin 
 *		(e.g., <code>"org.eclipse.core.runtime"</code>).
 * @return the plug-in runtime object, or <code>null</code>
 */
public Plugin getPlugin(String id) {
	return Platform.getPlugin(id);
}
/**
 * Returns the plug-in registry for this platform.
 * The platform must be running.
 *
 * @return the plug-in registry
 * @see IPluginRegistry
 */
public IPluginRegistry getPluginRegistry() {
	return Platform.getPluginRegistry();
}
/**
 * Returns the location in the local file system of the plug-in 
 * state area for the given plug-in.
 * The platform must be running.
 * <p>
 * The plug-in state area is a file directory within the
 * platform's metadata area where a plug-in is free to create files.
 * The content and structure of this area is defined by the plug-in,
 * and the particular plug-in is solely responsible for any files
 * it puts there. It is recommended for plug-in preference settings.
 * </p>
 *
 * @param plugin the plugin whose state location is returned
 * @return a local file system path
 */
public IPath getPluginStateLocation(Plugin plugin) {
	return Platform.getPluginStateLocation(plugin);
}
/**
 * Returns the protection space (realm) for the specified resource, or
 * <code>null</code> if the realm is unknown.
 *
 * @param resourceUrl the URL of the resource whose protection space is
 *		returned. For example, "http://www.hostname.com/folder/".
 * @return the protection space (realm) for the specified resource, or
 *		<code>null</code> if the realm is unknown
 */
public String getProtectionSpace(URL resourceUrl) {
	return Platform.getProtectionSpace(resourceUrl);
}
/**
 * Returns a plug-in registry containing all of the plug-ins discovered
 * on the given plug-in path.  Any problems encountered are noted in
 * the supplied status.
 * <p>
 * The given plug-in path is the list of locations in which to look for plug-ins.
 * If an entry identifies a directory (i.e., ends in a '/'), this methods 
 * attempts to scan all sub-directories for plug0-ins.  Alternatively, an
 * entry may identify a particular plug-in definition file (<code>plugin.xml</code>).
 * </p>
 * <p>
 * <b>Note:</b> this method does not affect the running platform.  It is intended
 * for introspecting installed plug-ins on this and other platforms.  The returned
 * registry is <b>not</b> the same as the platform's registry.
 * </p>
 *
 * @param pluginPath the list of locations in which to look for plug-ins.
 * @param status a multi-status which is updated to track any problems
 *		encountered while scanning for plug-ins.
 */
public PluginRegistryModel parsePlugins(URL[] pluginPath, Factory factory) {
	return Platform.parsePlugins(pluginPath, factory);
}
/** 
 * Removes the indicated log listener from the notification list
 * of the platform.  
 * If no such listener exists, no action is taken.
 * The platform must be running.
 *
 * @param name the name to deregister
 * @see ILogListener
 * @see #addLogListener
 */
public void removeLogListener(ILogListener logListener) {
	Platform.removeLogListener(logListener);
}
/**
 * Returns a URL which is the resolved equivalent of the
 * supplied URL. This method is expected to be used with 
 * plug-in-relative URLs returned by IPluginDescriptor.
 * If the specified URL is not a plug-in URL, it is returned
 * asis. If the specified URL is a plug-in URL, it is
 * resolved to a locally-accessisble URL using "file" protocol.
 *
 * @param url original plug-in-relative URL.
 * @return the resolved URL
 * @exception IOException unable to resolve URL
 * @see IPluginDescriptor#getInstallURL
 */
public URL resolve(URL url) throws java.io.IOException {

	return Platform.resolve(url);
}
/**
 * Runs the given runnable in a protected mode.   Exceptions
 * thrown in the runnable are logged and passed to the runnable's
 * exception handler.  Such exceptions are not rethrown by this method.
 *
 * @param code the runnable to run
 */
public void run(ISafeRunnable runnable) {
	Platform.run(runnable);
}
}
