package org.eclipse.scripting.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.internal.plugins.RegistryWriter;
import org.eclipse.core.internal.plugins.PluginDescriptor;
import java.io.*;
import org.eclipse.core.runtime.model.*;
/**
 * This is the plugin class for the script and scriptui extensions.
 * This class is a subclass of AbstractPlugin.  There is only one shared
 * instance of this class, which can be obtained via the getPlugin
 * member function.  This class loads all script language extensions, and
 * caches them.
 */
import org.eclipse.core.runtime.*;
import org.eclipse.core.resources.*;
import org.eclipse.jface.resource.ImageDescriptor;
import java.net.MalformedURLException;
import java.net.URL;
import java.io.IOException;
import java.io.File;
import java.io.FileOutputStream;

import java.util.*;

public class ScriptPlugin extends Plugin {
	
	private static ScriptPlugin   _scriptPlugin   = null;
	private static ScriptPlatform _scriptPlatform = new ScriptPlatform();

	private boolean               _bStartupComplete                  = false;
	private Hashtable             _hashLanguageConfigurationElements = new Hashtable();
	private IPluginDescriptor     _pluginDescriptor                  = null;
	private PluginDescriptorModel _descriptorUserScripts             = null;
	private File                  _fileUserScripts                   = null;
/**
 * This method creates and stores the single instance on the first invocation.
 */
public ScriptPlugin(IPluginDescriptor pluginDescriptor) {
	
	super(pluginDescriptor);

	_pluginDescriptor = pluginDescriptor;

	// Save this instance
	//-------------------
	if (_scriptPlugin == null) {
		_scriptPlugin = this;
	}
}
/**
 * Forces startup of the script plugin only if not started.
 */
public static ScriptPlugin bootstrap() {

	// Activate myself if not already activated
	//-----------------------------------------
	if (_scriptPlugin == null) {
	    
		IPluginRegistry registry = Platform.getPluginRegistry();
		IPluginDescriptor descriptor = registry.getPluginDescriptor(ScriptConstants.getPluginID());

		try {
			descriptor.getPlugin();
		}
		
		catch (CoreException ex) {
		}
	}

	return _scriptPlugin;
}
/**
 * This method looks for all extensions of the com.ibm.itp.scripting.language
 * extension point.  The configuration element of each extension is cached, but
 * the extensions are not created.  The extensions are created when the language
 * is requested.
 */
public void cacheScriptLanguageExtensions() {

	// Obtain the registry
	//--------------------
	IPluginRegistry registry = Platform.getPluginRegistry();	

	if (registry != null) {
		
		// Locate the scriptengine extension point
		//----------------------------------------
		IExtensionPoint extensionPoint = registry.getExtensionPoint(ScriptConstants.getPluginID(), "language");

		// Get the scriptlanguage extensions registered for the extension point
		//---------------------------------------------------------------------
		if (extensionPoint == null) {
			ScriptReporter.reportError(ScriptStrings.getString("S_Cannot_find_language_extension_point_in_plugin_registry"), null);
		}

		if (extensionPoint != null) {
			IExtension extensions[] = extensionPoint.getExtensions();

			// For each extension, get its scriptlanguage elements
			// Each extension may support more than one script language
			//---------------------------------------------------------
			IExtension extension = null;
			IConfigurationElement elementScriptLanguage = null;
			Object objLanguageString = null;

			for (int i = 0; i < extensions.length; ++i) {
				extension = extensions[i];
				IConfigurationElement elements[] = extension.getConfigurationElements();

				// Look for scriptlanguage elements
				// Store the language string, and the configuration element
				//---------------------------------------------------------
				if (elements != null) {
					
					for (int j = 0; j < elements.length; ++j) {
						elementScriptLanguage = elements[j];

						if (elementScriptLanguage.getName().equals("scriptlanguage") == true) {
							objLanguageString = elementScriptLanguage.getAttribute("language");

							if (objLanguageString != null && objLanguageString instanceof String) {
								_hashLanguageConfigurationElements.put(objLanguageString, elementScriptLanguage);
							}
						}
					}
				}
			}
		}
	}

	return;
}
/**
 *
 */
public void createUserScriptsPluginManifestFileContent(File file) {

	try {
		boolean bCreated = file.createNewFile();

		if (file.exists() == true && file.canWrite() == true) {

			FileOutputStream outputStream = null;

			outputStream = new FileOutputStream(file);

			StringBuffer strbFile = new StringBuffer();

			strbFile.append( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n" );
			strbFile.append( "<plugin id=\"org.eclipse.scripting.userscripts\" name=\"User Scripts\" vendor-name=\"IBM\" version=\"1.0\">\n" );
			strbFile.append( "  <requires>\n" );
			strbFile.append( "    <import plugin=\"org.eclipse.scripting\"/>\n" );
			strbFile.append( "  </requires>\n" );
			strbFile.append( "</plugin>\n" );
				
			outputStream.write(strbFile.toString().getBytes());
			outputStream.close();
		}
	}

	catch (IOException ex) {
	}

	return;
}
/**
 * Returns the image descriptor of an image file relative to this plugin
 */
public static ImageDescriptor getImageDescriptor(String strImage) {

	ScriptPlugin scriptPlugin = getPluginInstance();

	if (scriptPlugin != null) {

		// Obtain my plugin descriptor
		//----------------------------
		IPluginDescriptor pluginDescriptor = scriptPlugin.getDescriptor();

		// Determine where I am installed
		//-------------------------------
		URL path = pluginDescriptor.getInstallURL();

		// Add the relative file location to the install location
		// Create the image descriptor
		//-------------------------------------------------------
		URL urlFullPathString = null;

		try {
			urlFullPathString = new URL(path, strImage);
			return ImageDescriptor.createFromURL(urlFullPathString);
		}

		catch (MalformedURLException e) {
		}
	}
	return null;
}
/**
 * This method returns the single instance of this class.
 */
public static ScriptPlugin getPluginInstance() {
	return _scriptPlugin;
}
/**
 * Returns an array of script language descriptors.
 */
public ScriptLanguageDescriptor[] getScriptLanguageDescriptors() {
	
	ScriptLanguageDescriptor[] descriptors = null;

	if (_hashLanguageConfigurationElements != null) {
	    
		descriptors = new ScriptLanguageDescriptor[_hashLanguageConfigurationElements.size()];
		int i = 0;

		Enumeration enumElements = _hashLanguageConfigurationElements.elements();

		IConfigurationElement elementScriptLanguage = null;
		String                strLanguage           = null;
		String                strFileExtension      = null;

		while (enumElements.hasMoreElements()) {
	        
			// Obtain the "extension" xml element
			//-----------------------------------
			elementScriptLanguage = (IConfigurationElement) enumElements.nextElement();

			// Obtain the "scriptlanguage" xml child element
			//----------------------------------------------
			strLanguage = (String) elementScriptLanguage.getAttribute("language");
			strFileExtension = (String) elementScriptLanguage.getAttribute("fileextension");
			descriptors[i++] = new ScriptLanguageDescriptor(strLanguage, strFileExtension);
		}
	}

	// No languages, create empty array
	//---------------------------------
	else {
		descriptors = new ScriptLanguageDescriptor[0];
	}

	return descriptors;
}
/**
 * Returns a string array of script languages
 * 
 */
public String[] getScriptLanguageNames() {
	
	String[] straScriptLanguages = null;

	if (_hashLanguageConfigurationElements != null) {
		
		straScriptLanguages = new String[_hashLanguageConfigurationElements.size()];
		int i = 0;

		Enumeration enumKeys = _hashLanguageConfigurationElements.keys();

		while (enumKeys.hasMoreElements()) {
			straScriptLanguages[i++] = (String) enumKeys.nextElement();
		}
	}

	// No languages, create empty array
	//---------------------------------
	else {
		straScriptLanguages = new String[0];
	}

	return straScriptLanguages;
}
/**
 * Returns the manifest for user defined scripts.
 */
public PluginDescriptorModel getUserScriptsPluginManifest() {

	// Open the plugin.xml file that contains extensions for scripts
	//--------------------------------------------------------------
	if (_descriptorUserScripts == null) {

		// Construct the path to the user scripts manifest file
		//-----------------------------------------------------
		IPath path = getStateLocation();
		path = path.addTrailingSeparator();
		path = path.append("plugin.xml");

		StringBuffer strbPluginFilespec = new StringBuffer(path.toOSString());

		// Obtain the file
		//----------------	
		_fileUserScripts = new File(strbPluginFilespec.toString());

		// Create the file if it doesn't exist
		//------------------------------------
		if (_fileUserScripts.exists() == false || _fileUserScripts.length() == 0) {
			createUserScriptsPluginManifestFileContent(_fileUserScripts);
		}

		// Obtain the plugin
		//------------------
		URL[] urlArray = null;
		try {
			urlArray = new URL[] { new URL("file", null, -1, path.toString())};
		}
		catch (MalformedURLException ex){
		}
		
		Factory factory = new Factory(new MultiStatus(ScriptConstants.getPluginID(), 0, "", null));
		PluginRegistryModel registryModel = Platform.parsePlugins(urlArray, factory);

		_descriptorUserScripts = registryModel.getPlugin(ScriptConstants.getPluginIDUserScripts());
	}

	return _descriptorUserScripts;
}
/**
 * Given a script manager, this class registers with the script manager,
 * <ul>Platform object</li>
 * <li>Plugin object</li>
 * <li>Toolkit object</li>
 * <li>IWorkspace object</li>
 * </ul>
 */
public void registerScriptGlobals(ScriptManager scriptManager, Plugin plugin, ScriptToolkit toolkit) {

	if (scriptManager != null) {
		scriptManager.declareElement("platform", _scriptPlatform);
		scriptManager.declareElement("toolkit", toolkit);
		scriptManager.declareElement("workspace", ResourcesPlugin.getWorkspace());

		if (plugin != null) {
			scriptManager.declareElement("plugin", plugin);
		}
	}
}
/**
 * Save the Scripts plugin manifest file.
 */
public void saveUserScriptsPluginManifest() throws IOException {

	// If the file, or manifest is null, then do not save since
	// the manifest has not been modified
	//---------------------------------------------------------
	if (_descriptorUserScripts != null && _fileUserScripts != null) {

		// Create a plugin descriptor from the model
		//------------------------------------------	
		PluginDescriptor pluginDescriptor = new PluginDescriptor();

		pluginDescriptor.setDeclaredExtensionPoints(_descriptorUserScripts.getDeclaredExtensionPoints());
		pluginDescriptor.setDeclaredExtensions(_descriptorUserScripts.getDeclaredExtensions());
		pluginDescriptor.setEnabled(_descriptorUserScripts.getEnabled());
		pluginDescriptor.setId(_descriptorUserScripts.getId());
		pluginDescriptor.setLocation(_descriptorUserScripts.getLocation());
		pluginDescriptor.setName(_descriptorUserScripts.getName());
		pluginDescriptor.setPluginClass(_descriptorUserScripts.getPluginClass());
		pluginDescriptor.setProviderName(_descriptorUserScripts.getProviderName());
		pluginDescriptor.setRegistry(_descriptorUserScripts.getRegistry());
		pluginDescriptor.setRequires(_descriptorUserScripts.getRequires());
		pluginDescriptor.setRuntime(_descriptorUserScripts.getRuntime());
		pluginDescriptor.setVersion(_descriptorUserScripts.getVersion());

		// Create an output stream
		//------------------------
		FileOutputStream outputStream = new FileOutputStream(_fileUserScripts);
		PrintWriter printWriter = new PrintWriter(outputStream);

		// Write out the header
		//---------------------
		printWriter.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		printWriter.println("");
		printWriter.println("<!-- ==== User Scripts Plugin ==== -->");

		// Write out the descriptor
		//-------------------------
		RegistryWriter registryWriter = new RegistryWriter();
		registryWriter.writePluginDescriptor(pluginDescriptor, printWriter, 0);
		printWriter.close();
	}
}
/**
 * This function is called when the extension is loaded by the
 * plugin class loader.  If this is the first instance, the
 * script language extensions are loaded and cached.
 */
public void startup() {
	
	if (_bStartupComplete == false) {
	    
		cacheScriptLanguageExtensions();
		_bStartupComplete = true;
	}

//	getUserScriptsPluginManifest();

	// Returning true signifies success!
	//----------------------------------
	return;
}
}
