package org.eclipse.scripting.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This class has static functions which are called whenever any
 * error occurs that needs to be displayed on the script console.
 */
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.util.Date;

import org.eclipse.scripting.internal.uirenderer.XGRXmlParserException;
import org.eclipse.scripting.internal.views.ScriptConsoleViewer;
import org.xml.sax.SAXParseException;

public class ScriptReporter {
	/**
	 * Prints out the line of text corresponding the line number,
	 * the line number, and the column number.
	 */
	protected static void printFileContext(String strFilename, int iLineNumber, int iColumnNumber) {

		try {
			// Attempt to read the file
			// Obtain the zero based line
			//---------------------------
			ScriptReaderFileSystemFile reader = new ScriptReaderFileSystemFile(strFilename);
			String strLine = reader.getLine(iLineNumber);

			// Print the line and position indicator
			//--------------------------------------
			printLineContext(strLine, iLineNumber, iColumnNumber);
		}

		catch (Exception ex) {
		}
	}
	/**
	 * Prints out the line of text corresponding the line number,
	 * the line number, and the column number.
	 */
	protected static void printFileContext(ScriptDescriptor scriptDescriptor, int iLineNumber, int iColumnNumber) {

		try {
			// Attempt to read the file
			// Obtain the zero based line
			//---------------------------
			ScriptReaderFileSystemFile reader = new ScriptReaderFileSystemFile(scriptDescriptor._strFilename);
			String strLine = reader.getLine(iLineNumber);

			// Print the line and position indicator
			//--------------------------------------
			printLineContext(strLine, iLineNumber, iColumnNumber);
		}

		catch (Exception ex) {
		}
	}
	/**
	 * Prints a header line with the date and time.
	 */
	public static void printHeader() {
		println("--- " + new Date().toString() + " ---");
	}
	/**
	 * Print the line and a caret under the error
	 *
	 */
	protected static void printLineContext(String strLine, int iLineNumber, int iColumnNumber) {
		if (strLine != null) {
			ScriptConsoleViewer scriptConsole = ScriptConsoleViewer.getInstance();

			if (scriptConsole != null) {
				// Print the line
				//---------------
				scriptConsole.println(strLine);
				/*
							// Javascript currently does not return the correct column number
							// Place a caret under the line at the given column
							//---------------------------------------------------------------
							if( iColumnNumber < strLine.length() )
							{
								StringBuffer strbPositionIndicator = new StringBuffer( iColumnNumber + 1 );
				
								for( int i=0; i<iColumnNumber; ++i )
								{
									strbPositionIndicator.append( '.' );
								}
								
								strbPositionIndicator.append( '^' );
							
								scriptConsole.println( strbPositionIndicator.toString() );
							}
				*/
			}
		}
	}
	/**
	 * Prints a line of text
	 *
	 */
	public static void println(String strText) {
		ScriptConsoleViewer scriptConsole = ScriptConsoleViewer.getInstance();

		if (scriptConsole != null) {
			scriptConsole.println(strText);
		}
	}
	/**
	 * Prints the line number, column number and file name.
	 */
	protected static void printLocation(String strFilename, int iLineNumber, int iColumnNumber) {
		ScriptConsoleViewer scriptConsole = ScriptConsoleViewer.getInstance();

		if (scriptConsole != null) {
			scriptConsole.println(ScriptStrings.getString("S_line") + "=" + Integer.toString(iLineNumber + 1) + ", " + ScriptStrings.getString("S_column") + "=" + Integer.toString(iColumnNumber + 1) + ", " + ScriptStrings.getString("S_file") + "=" + strFilename);
		}
	}
	/**
	 * Prints out an object using its toString() function.
	 */
	public static void printObject(Object object) {
		if (object instanceof Exception) {
			printStackTrace((Exception) object);
		}

		else {
			println(object.toString());
		}
	}
	/**
	 * Prints out the stack trace given a Throwable object.
	 *
	 */
	protected static void printStackTrace(Throwable ex) {
		ScriptConsoleViewer scriptConsole = ScriptConsoleViewer.getInstance();

		if (scriptConsole != null) {
			// Create a byte array output stream
			//----------------------------------
			ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

			// Create a print writer
			//----------------------
			if (outputStream != null) {
				PrintWriter printWriter = new PrintWriter(outputStream);

				// Output the stack trace to the output stream
				//--------------------------------------------
				if (printWriter != null) {
					try {
						ex.printStackTrace(printWriter);

						// Empty the print writer to the output stream
						//--------------------------------------------
						printWriter.flush();

						// Write the output to the console
						//--------------------------------
						scriptConsole.println(outputStream.toString());
					}
					catch (Exception exPrint) {
					}
				}
			}
		}
	}
	/**
	 * Reports an error string and its detail string.
	 *
	 */
	public static void reportError(String strError, String strDetail) {
		ScriptConsoleViewer scriptConsole = ScriptConsoleViewer.getInstance();

		if (scriptConsole != null) {
			// Print the header line
			//----------------------
			printHeader();

			// Convert the error string to a locale specific string
			//-----------------------------------------------------

			// Output the locale specific string
			//----------------------------------
			scriptConsole.println(strError);

			if (strDetail != null) {
				scriptConsole.println(strDetail);
			}
		}
	}
	/**
	 * This method reports exception information.
	 *
	 */
	public static void reportException(Exception exception) {
		ScriptConsoleViewer scriptConsole = ScriptConsoleViewer.getInstance();

		if (scriptConsole != null) {
			// Print the header line
			//----------------------
			printHeader();

			// Print the exception
			//--------------------
			scriptConsole.println(exception.toString());

			printStackTrace(exception);
		}
	}
	/**
	 * Prints out the information pertaining to a Java evaluation error.
	 *
	 */
	public static void reportJavaEvaluationException(Exception exception, ScriptDescriptor scriptDescriptor) {
		ScriptConsoleViewer scriptConsole = ScriptConsoleViewer.getInstance();

		if (scriptConsole != null) {
			// Print the header line
			//----------------------
			printHeader();

			// Print the exception
			//--------------------
			scriptConsole.println(exception.toString());

			// Print the file, line, and column information
			//---------------------------------------------
			printLocation(scriptDescriptor._strFilename, scriptDescriptor._iLineNumber, scriptDescriptor._iColumnNumber);

			// Print the line in the file and the position indicator
			//------------------------------------------------------
			printFileContext(scriptDescriptor, scriptDescriptor._iLineNumber, scriptDescriptor._iColumnNumber);

			// Print the actual exception's stack trace
			// only if it isn't a no such method exception
			//--------------------------------------------
			if (exception instanceof NoSuchMethodException == false) {
				if (exception instanceof InvocationTargetException) {
					Throwable throwable = ((InvocationTargetException) exception).getTargetException();

					if (throwable != null) {
						scriptConsole.println(throwable.getMessage());
						printStackTrace(throwable);
					}
				}

				// Print the exception's stack trace
				//----------------------------------
				else {
					printStackTrace(exception);
				}
			}
		}
	}
	/**
	 * Prints out error information pertaining to a parser exception when parsing
	 * a user interface definition file.
	 */
	public static void reportPresentationParserException(XGRXmlParserException exception) {
		ScriptConsoleViewer scriptConsole = ScriptConsoleViewer.getInstance();

		if (scriptConsole != null) {
			// Print the header line
			//----------------------
			printHeader();

			// Print the exception
			//--------------------
			scriptConsole.println(exception._strMessage);

			// Print file, line, and column information
			//-----------------------------------------
			printLocation(exception._strFilename, exception._iLineNumber, exception._iColumnNumber);

			// Print the line, and the position indicator
			//-------------------------------------------
			printLineContext(exception._strLine, exception._iLineNumber, exception._iColumnNumber);
		}
	}
	/**
	 * Prints out a parser exception when parsing an XML file.
	 */
	public static void reportSAXParseException(SAXParseException exception) {
		ScriptConsoleViewer scriptConsole = ScriptConsoleViewer.getInstance();

		if (scriptConsole != null) {
			// Print the header line
			//----------------------
			printHeader();

			// Print the SAX exception
			//------------------------
			scriptConsole.println(exception.toString());

			String strFilename = exception.getSystemId();

			if (strFilename.indexOf("file:/") == 0) {
				strFilename = strFilename.substring(6);
			}

			strFilename.replace('/', '\\');

			// Print file, line, and column information
			//-----------------------------------------
			printLocation(strFilename, exception.getLineNumber() - 1, exception.getColumnNumber() - 1);

			// Print the line in the file and the position indicator
			//------------------------------------------------------
			printFileContext(strFilename, exception.getLineNumber() - 1, exception.getColumnNumber() - 1);
		}
	}
	/**
	 * Reports exception information currently to stderr.  All
	 * scripting framework exceptions are handled here.
	 */
	public static void reportScriptEvaluationException(Exception exception, ScriptDescriptor scriptDescriptor) {

		ScriptConsoleViewer scriptConsole = ScriptConsoleViewer.getInstance();

		if (scriptConsole != null) {

			// Print the header line
			//----------------------
			printHeader();

			// Print the exception
			//--------------------
			scriptConsole.println(exception.getLocalizedMessage());

			if (exception instanceof InvocationTargetException) {
				InvocationTargetException tException = (InvocationTargetException) exception;
				Throwable throwable = tException.getTargetException();

				if (throwable instanceof ScriptEngineException) {
					ScriptEngineException sException = (ScriptEngineException) throwable;

					scriptConsole.println(sException.getLocalizedMessage());
				}
			}

			printLocation(scriptDescriptor._strFilename, scriptDescriptor._iLineNumber, scriptDescriptor._iColumnNumber);
			printFileContext(scriptDescriptor._strFilename, scriptDescriptor._iLineNumber, scriptDescriptor._iColumnNumber);
		}
	}
}
