package org.eclipse.scripting.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This class provides the default functions for all scripts, such
 * as "print".
 */
import org.eclipse.scripting.*;
import org.eclipse.core.runtime.*;
import org.eclipse.core.resources.*;
import org.eclipse.scripting.internal.datamodels.*;
import java.util.*;
import org.eclipse.ui.dialogs.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.SWT;

public class ScriptToolkit implements IToolkit
{
	IResource[] _resourceSelectionArray = null;
	Plugin      _plugin                 = null;
	Shell       _shell                  = null;
/**
 *
 */
public ScriptToolkit( Plugin plugin, IResource[] resourceSelectionArray ) 
{
	_resourceSelectionArray = resourceSelectionArray;
	_plugin                 = plugin;
}
/**
 *
 */
public IDataModel createDataModel()
{
	XGRXmlElementsAndAttributesDataModel model = new XGRXmlElementsAndAttributesDataModel();

	return model;
}
/**
 * 
 */
public IFile createFile( IContainer container, String strRelativePath ) throws Exception
{
//	Trace.functionEntry( this, "createFile" );
	
	IFile file = null;

	try
	{
		// Remove any beginning separator from the relative path
		//------------------------------------------------------
		if( strRelativePath.startsWith( "" + IPath.SEPARATOR ) == true )
		{
			strRelativePath = strRelativePath.substring( 1 );
		}

		if( strRelativePath.length() > 0 ) 
		{		
			// Assume that the container exists.
			// Validate the relative path
			// Create the full path with the filename on the end
			//--------------------------------------------------
			IPath pathFull = container.getFullPath().append( strRelativePath ).removeTrailingSeparator();

			// Trace.string( pathFull.toString() );
			
			// Do nothing if there is a trailing separator
			//--------------------------------------------
			if( pathFull.hasTrailingSeparator() == false )
			{
				// Create the full path without the filename on the end
				//-----------------------------------------------------
				IPath pathContainer = (IPath)pathFull.clone();
				pathContainer = pathContainer.removeLastSegments( 1 );
	
				// Ensure that the container path exists
				//--------------------------------------
				IWorkspace workspace = ResourcesPlugin.getWorkspace();
				if( workspace != null && workspace.getRoot().exists( pathContainer ) ) 
				{
					// Create a file object
					//---------------------
					file = container.getFile( new Path( strRelativePath ) );

					// Do nothing if the file already exists
					//--------------------------------------
					if( file != null && file.exists() == false )
					{
						file.create( null, true, null );
					}
				}
			}
		}
	}

	// Catch and rethrow all exceptions
	//---------------------------------
	catch( Exception ex ) 
	{
		file = null;
		throw ex;
	}

	finally
	{
//		Trace.functionExit( this, "createFile" );
	}
		
	return file; 
}
/**
 * 
 */
public IFolder createFolder( IContainer containerBase, String strRelativePath ) throws Exception
{
//	Trace.functionEntry( this, "createFolder" );
	
	IFolder folder = null;

	try
	{	
		// Remove any beginning separator from the relative path
		//------------------------------------------------------
		if( strRelativePath.startsWith( "" + IPath.SEPARATOR) == true )
		{
			strRelativePath = strRelativePath.substring( 1 );
		}
	
		if( strRelativePath.length() > 0 ) 
		{		
			// Assume that the folder exists
			// We must validate the path to make sure that it is valid
			// Create the full path with the folder on the end
			//--------------------------------------------------------
			IPath pathFull  = containerBase.getFullPath().append( strRelativePath ).removeTrailingSeparator();

//			Trace.string( pathFull.toString() );
		
			// Do nothing if there is a trailing separator
			//--------------------------------------------
			if( pathFull.hasTrailingSeparator() == false )
			{
				// Create the full path without the new folder on the end
				//-------------------------------------------------------
				IPath pathContainerBase  = (IPath)pathFull.clone();
				pathContainerBase = pathContainerBase.removeLastSegments( 1 );

				// Ensure that the folder path exists
				//-----------------------------------
				IWorkspace workspace = ResourcesPlugin.getWorkspace();
				if( workspace != null && workspace.getRoot().exists( pathContainerBase ) == true ) 
				{
					// Create a folder object
					//-----------------------
					folder = containerBase.getFolder( new Path( strRelativePath ) );

					// Do nothing if the folder already exists
					//----------------------------------------
					if( folder != null && folder.exists() == false )
					{
						folder.create( true, true, null );
					}
				}
			}
		}
	}

	// Catch and rethrow all exceptions
	//---------------------------------
	catch( Exception ex ) 
	{
		folder = null;
		throw ex;
	}

	finally
	{
//		Trace.functionExit( this, "createFolder" );
	}

	return folder;
}
/**
 * Creates an IPath object from a path string.
 */
public IPath createPath( String strPath ) throws Exception
{
	return new Path( strPath );
}
/**
 * Creates a project with the given name.
 *
 */
public IProject createProject( String strProject ) throws Exception
{
	IProject project = null;
	
	IWorkspace workspace = ResourcesPlugin.getWorkspace();

	if( workspace != null  ) {

		project = workspace.getRoot().getProject( strProject );

		if( project != null && project.exists() == false ) {
			project.create( null );
		}
	}
	
	return project;	
}
/**
 * Returns an array of files given a container.
 *
 */
public IFile[] getFiles( IContainer container ) throws Exception
{
//	Trace.functionEntry( this, "getFiles" );

	IFile[] files = null;
	
	try
	{
		// Count the number of files
		//--------------------------
		int iCount = getResourceCount( container.members( false ), IResource.FILE );

		// Create the returning IFiles array
		//----------------------------------
		files = new IFile[iCount];

		// Set the array contents
		//-----------------------
		files = (IFile[])getResources( container.members( false ), files, IResource.FILE );
	}			

	// Catch and rethrow all exceptions
	//---------------------------------
	catch( Exception ex ) 
	{
		files = null;
		throw ex;
	}

	finally
	{
//		Trace.functionExit( this, "getFiles" );
	}
	
	return files;
}
/**
 * Finds the children of the given container
 * and places them in a new array.
 */
public IFolder[] getFolders( IContainer container ) throws Exception
{
//	Trace.functionEntry( this, "getFolders" );

	IFolder[] folders = null;
	
	try
	{	
		// Count the number of folders
		//----------------------------
		int iCount = getResourceCount( container.members( false ), IResource.FOLDER );

		// Create the returning IFolder array
		//-----------------------------------
		folders = new IFolder[iCount];

		// Set the array contents
		//-----------------------
		folders = (IFolder[])getResources( container.members( false ), folders, IResource.FOLDER );
	}
	
	// Catch and rethrow all exceptions
	//---------------------------------
	catch( Exception ex ) 
	{
		folders = null;
		throw ex;
	}

	finally
	{
//		Trace.functionExit( this, "getFolders" );
	}

	return folders;
}
/**
 * Returns the currently open workspace project.
 */
public IProject getOpenProject() {

	IWorkspace workspace = ResourcesPlugin.getWorkspace();

	if (workspace != null) {

		IProject projects[] = workspace.getRoot().getProjects();

		if (projects != null) {

			for (int i = 0; i < projects.length; ++i) {

				if (projects[i].isOpen() == true) {

					return projects[i];
				}
			}
		}
	}
	
	return null;
}
/**
 * Returns the plugin object.
 */
public Plugin getPlugin()
{
	return _plugin;
}
/**
 * Returns an array of all projects.
 */
public IProject[] getProjects() {

	IWorkspace workspace = ResourcesPlugin.getWorkspace();

	if( workspace != null ) {
		return workspace.getRoot().getProjects();
	}
	
	return null;
}
/**
 * Counts the number of enumerated elements of a given resource type
 * The enumeration is invalid after this call.
 */
private static final int getResourceCount( IResource[] resources, int iResourceConstant )
{
	// Count the number of elements
	//-----------------------------
	int iCount = 0;
	
	for( int i=0; i<resources.length; ++i )
	{
		if( resources[i].getType() == iResourceConstant )
		{
			++iCount;
		}
	}
	
	return iCount;
}
/**
 * Given a resource type, this function
 * copies the elements of the resource type from the enumeration to the array.
 * The enumeration is invalid after this call.
 */
private final static IResource[] getResources( IResource[] resourcesSource, IResource[] resourcesTarget, int iResourceConstant )
{
	// Set the array contents
	//-----------------------
	int iTargetIndex = 0;

	for( int i=0; i<resourcesSource.length; ++i )
	{
		if( resourcesSource[i].getType() == iResourceConstant )
		{
			resourcesTarget[iTargetIndex++] = resourcesSource[i];
		}
	}
	
	return resourcesTarget;
}
/**
 * Returns a boolean indicating whether an object is of a specific class type.
 */
public boolean isInstanceOf( Object object, String strClassType )
{
	Class classInstance = object.getClass();

	// Check interfaces
	//-----------------
	Class[] classArray = classInstance.getInterfaces();

	for( int i=0; i<classArray.length; ++i )
	{
		if( classArray[i].getName().equals( strClassType ) == true )
		{
			return true;
		}
	}

	// Check superclasses
	//-------------------
	do
	{
		classInstance = classInstance.getSuperclass();
		if( classInstance != null )
		{
			if( classInstance.getName().equals( strClassType ) == true )
			{
				return true;
			}
		}
	} while( classInstance != null );
	
	return false;
}
/**
 * Loads a data model from a workspace file.
 */
public IDataModel loadDataModelFromFile( IFile file ) throws Exception
{
//	Trace.functionEntry( this, "loadDataModel( IFile )" );

	IDataModel dataModel = null;
	
	try
	{
		XGRDataModelFactory factory = new XGRDataModelFactory();
			
		dataModel = factory.getDataModel( file );
	}

	// Catch and rethrow all exceptions
	//---------------------------------
	catch( Exception ex ) 
	{
		throw ex;
	}

	finally
	{
//		Trace.functionExit( this, "loadDataModel( IFile )" );
	}

	return dataModel;
}
/**
 * Loads a data model from a file system file.
 */
public IDataModel loadDataModelFromFilename( String strFilename ) throws Exception
{
//	Trace.functionEntry( this, "loadDataModel( String )" );

	IDataModel dataModel = null;
	
	try
	{
		XGRDataModelFactory factory = new XGRDataModelFactory();
			
		dataModel = factory.getDataModel( strFilename );
	}

	// Catch and rethrow all exceptions
	//---------------------------------
	catch( Exception ex ) 
	{
		throw ex;
	}

	finally
	{
//		Trace.functionExit( this, "loadDataModel( String )" );
	}

	return dataModel;
}
/**
 * Prints a string to the script console.
 */
public void print( Object object )
{
//	Trace.functionExit( this, "print" );

	try
	{
		if( object != null )
		{
			ScriptReporter.printObject( object );
		}
		else
		{
			ScriptReporter.println( "null" );
		}
	}

	// Catch all exceptions
	//---------------------
	catch( Exception ex ) 
	{
	}

	finally
	{
//		Trace.functionExit( this, "print" );
	}

}
/**
 * Displays a dialog with ok button.
 */
public void promptForOk( String strTitle, String strMessage, String strMessageType )
{
	int iIconType = SWT.ICON_INFORMATION;

	     if( strMessageType.equals( "error"   ) == true ) { iIconType = SWT.ICON_ERROR;   }
	else if( strMessageType.equals( "warning" ) == true ) { iIconType = SWT.ICON_WARNING; }
	else if( strMessageType.equals( "working" ) == true ) { iIconType = SWT.ICON_WORKING; }
	
	MessageBox messageBox = new MessageBox( _shell, SWT.OK | iIconType | SWT.APPLICATION_MODAL );

	messageBox.setText( strTitle );
	messageBox.setMessage( strMessage );

	messageBox.open();
}
/**
 * Displays a dialog with Ok and Cancel buttons.  False is returned if the
 * Cancel button was pressed.  True is returned if the Ok button was pressed.
 */
public boolean promptForOkCancel( String strTitle, String strMessage )
{
	MessageBox messageBox = new MessageBox( _shell, SWT.OK | SWT.CANCEL | SWT.ICON_QUESTION | SWT.APPLICATION_MODAL);

	messageBox.setText( strTitle );
	messageBox.setMessage( strMessage );

	return messageBox.open() == SWT.CANCEL ? false : true;
}
/**
 * Displays the system file selection dialog and uses
 * the FileDialog class.
 * The dialog will be displayed as either an open, or saveas dialog.
 * A single
 * IPath is returned, or null if no file was selected.
 */
public IPath promptForSystemFile( boolean bSaveNotOpen ) throws Exception
{
//	Trace.functionEntry( this, "promptForDesktopFile" );
	
	IPath pathSelected = null;

	try
	{
		// Create the system file selection dialog and wait for a result
		//--------------------------------------------------------------
		FileDialog dialog = new FileDialog( _shell, bSaveNotOpen == true ? SWT.SAVE : SWT.OPEN );

		// Obtain the selected system file specification
		//----------------------------------------------
		String strSelectedFile = dialog.open();

		// Create a path from the system file specification
		//-------------------------------------------------
		if( strSelectedFile != null )
		{
			pathSelected = new Path( strSelectedFile );
		}
	}
	
	// Catch and rethrow all exceptions
	//---------------------------------
	catch( Exception ex )
	{
		pathSelected = null;
		throw ex;
	}

	finally
	{
//		Trace.functionExit( this, "promptForDesktopFile" );
	}

	return pathSelected;
}
/**
 * This method displays the system directory selection dialog and uses
 * the org.eclipse.swt.widgets.DirectoryDialog.
 * The dialog will be displayed as either an open, or saveas dialog.
 * A single IPath is returned, or null if no directory was selected.
 */
public IPath promptForSystemFolder() throws Exception
{
//	Trace.functionEntry( this, "promptForSystemFolder" );
	
	IPath pathSelected = null;

	try
	{
		// Create the system folder selection dialog and wait for a result
		//----------------------------------------------------------------
		DirectoryDialog dialog = new DirectoryDialog( _shell, SWT.SAVE );
		
		String strSelectedDirectory = dialog.open();

		if( strSelectedDirectory != null)
		{
			// Create a path from the system folder specification
			//---------------------------------------------------
			pathSelected = new Path( strSelectedDirectory );
		}

	}
	
	// Catch and rethrow all exceptions
	//---------------------------------
	catch( Exception ex )
	{
		pathSelected = null;
		throw ex;
	}

	finally
	{
//		Trace.functionExit( this, "promptForSystemFolder" );
	}

	return pathSelected;
}
/**
 * Displays the container selection dialog.
 * The dialog will be displayed with the given IFolder as the root.
 * A single
 * IFolder is returned.  If the folderRoot parameter is null, the
 * open solution is used as the root folder.
 */
public IContainer promptForWorkspaceContainer(IContainer containerRoot, String strMessage) throws Exception {

	IContainer containerSelected = null;
	java.util.List listSelected = null;

	try {
		// Create the container selection dialog and wait for a result
		// If container is null then the dialog will show all projects
		//------------------------------------------------------------
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(_shell, containerRoot, false, strMessage);

		// Create a new shell, otherwise
		// the controls will not be active
		//--------------------------------
		dialog.setBlockOnOpen(true);
		dialog.open();

		// Obtain the list of selected containers
		//---------------------------------------
		Object[] objSelected = dialog.getResult();

		// Select the first container
		//---------------------------
		if (objSelected != null && objSelected.length > 0) {
			if (objSelected[0] instanceof IPath) {
				IPath path = (IPath) objSelected[0];
				if (path != null) {
					IWorkspace workspace = ResourcesPlugin.getWorkspace();
					containerSelected = (IContainer) workspace.getRoot().findMember(path.toString());
				}
			}
		}
	}

	// Catch and rethrow all exceptions
	//---------------------------------
	catch (Exception ex) {
		containerSelected = null;
		throw ex;
	}

	return containerSelected;
}
/**
 * Displays the resource selection dialog.  An array
 * of IResources is returned.
 */
public IFile[] promptForWorkspaceFiles(IContainer containerRoot, String strMessage) throws Exception {

	IFile[] fileArray = null;

	try {
		// If folderRoot is null assign it the open solution
		//--------------------------------------------------
		java.util.List listResources = null;

		if (strMessage == null) {
			strMessage = new String();
		}

		// Show the resource selection dialog with folderRoot as the root
		//---------------------------------------------------------------
		ResourceSelectionDialog dialog = new ResourceSelectionDialog(_shell, containerRoot, strMessage);

		// Create a new shell, otherwise
		// the controls will not be active
		//--------------------------------
		dialog.setBlockOnOpen(true);
		dialog.open();

		// Store the selected resource vector elements into an array
		//----------------------------------------------------------
		Object[] objResources = dialog.getResult();

		// Count the number of files
		// (They currently are all files,
		//  but it's a weird design returning a vector of IResource)
		//----------------------------------------------------------
		if (objResources != null) {
			int iNumberOfFiles = 0;

			for (int i = 0; i < objResources.length; ++i) {
				if (objResources[i] instanceof IFile) {
					++iNumberOfFiles;
				}
			}

			fileArray = new IFile[iNumberOfFiles];

			for (int i = 0; i < objResources.length; ++i) {
				if (objResources[i] instanceof IFile) {
					fileArray[i] = (IFile) objResources[i];
				}
			}
		}
	}

	// Catch and rethrow all exceptions
	//---------------------------------
	catch (Exception ex) {
		throw ex;
	}

	finally {
		if (fileArray == null) {
			fileArray = new IFile[0];
		}
	}

	return fileArray;
}
}
