package org.eclipse.scripting.internal.datamodels;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This class contains a list of data elements.
 * Node names are relative names.  For instance, a node
 * with a fully qualified name of "a.b.c" would have a name of
 * "c".  Each data element contained by this data node has the same
 * name as the data node itself.  This node can be thought of as
 * a container of indexed elements.
 */
import org.eclipse.scripting.internal.*;
import java.util.Vector;

public abstract class XGRBaseDataNode implements IXGRDataNode 
{
	protected static final String _strRootName = "root";
	
	protected IXGRDataNode _nodeParent          = null;
	protected Vector       _vectorChildNodes    = null;
/**
 *
 */
public XGRBaseDataNode( IXGRDataNode nodeParent )
{
//	Trace.functionEntry( this, "XGRBaseDataNode" );

//	Trace.object( "nodeParent", nodeParent );
		
	// Remember my parent
	//-------------------
	_nodeParent = nodeParent;

	// Add me to the parent's list of children
	//----------------------------------------
	if( nodeParent != null )
	{
		nodeParent.addChildNode( this );
	}

//	Trace.functionExit( this, "XGRBaseDataNode" );
}
/**
 *
 */
public void addChildNode( IXGRDataNode nodeChild )
{
//	Trace.functionEntry( this, "addChildNode" );
	
	if( nodeChild != null )
	{
		if( _vectorChildNodes == null )
		{
			_vectorChildNodes = new Vector();
		}
		
		_vectorChildNodes.addElement( nodeChild );
	}

//	Trace.functionExit( this, "addChildNode" );
}
/**
 *
 */
public Vector getChildNodes()
{
	return _vectorChildNodes;
}
/**
 *
 */
public String getFullyQualifiedName()
{
	StringBuffer strbufName = new StringBuffer( getName() );

	IXGRDataNode nodeParent = this;
	
	while( ( nodeParent = nodeParent.getParentNode() ) != null )
	{
		strbufName.insert( 0, nodeParent.getName() + "." );
	}

	// Strip off the beginning root element "ROOT."
	//---------------------------------------------
	return strbufName.toString().substring( 5 );
}
/**
 * This function returns an array of elements that match the path string
 * beginning with this node.
 */
public IXGRDataNode[] getNodes( String strDataPath ) 
{
//	Trace.functionEntry( this, "getNodes" );
	
	IXGRDataNode[] nodeaReturn = null;
	
	// Recursively look for this path
	//-------------------------------
	if( strDataPath != null )
	{
		Vector vectorReturnedNodes = new Vector();
	
		getNodesRecursive( strDataPath, vectorReturnedNodes );

		// Convert the vector to an array
		//-------------------------------
		int iNumberOfNodes = vectorReturnedNodes.size();
	
		if( vectorReturnedNodes.size() > 0 )
		{
			nodeaReturn = new IXGRDataNode[ iNumberOfNodes ];
		
			for( int i=0; i<vectorReturnedNodes.size(); ++i )
			{
				nodeaReturn[i] = (IXGRDataNode)vectorReturnedNodes.elementAt( i );
			}
		}
	}

//	Trace.functionExit( this, "getNodes" );
	return nodeaReturn;
}
/**
 * Search for a leaf nodes given a path a.b.c
 * Each leaf node is added to the vector
 */
protected void getNodesRecursive( String strPath, Vector vectorReturnedNodes )
{
//	Trace.functionEntry( this, "getNodesRecursive" );

//	Trace.string( getType() + " " + strPath );

	if( _vectorChildNodes   != null &&
		strPath             != null &&
		vectorReturnedNodes != null    )
	{
		// Check for period separator indicating a subpath
		//------------------------------------------------
		int iPeriod             = strPath.indexOf( '.' );

		// Check for an index specification of the form abc[12], or abc[12].
		//------------------------------------------------------------------
		int iOpenSquareBracket  = strPath.indexOf( '[' );
		int iCloseSquareBracket = strPath.indexOf( ']' );
		int iIndexSearch = -1;  // All occurances is the default

		if( iOpenSquareBracket > -1 && iCloseSquareBracket > iOpenSquareBracket    )
		{
			if( iPeriod <= -1 || iPeriod > iOpenSquareBracket )
			{
				try
				{
					iIndexSearch = new Integer( strPath.substring( iOpenSquareBracket + 1, iCloseSquareBracket ) ).intValue();
				}
				
				catch( Exception ex )
				{
					ScriptReporter.reportError( ScriptStrings.getString( "S_Invalid_index_syntax" ), strPath );
					return;
				}
			}
		}

		// Determine end of name string
		// 1. abc.
		// 2. abc[12].
		//-----------------------------
		int iNameEnd = iIndexSearch > -1 ? iOpenSquareBracket : iPeriod;

		// Obtain the first token
		//-----------------------
		String strName = iNameEnd < 0 ? strPath : strPath.substring( 0, iNameEnd );

		// Search through all the children for a name, and occurance match
		//----------------------------------------------------------------
		XGRBaseDataNode nodeChild    = null;
		int             iIndexFound  = -1;

//		Trace.stringValue( "strName", strName );
//		Trace.value( "iIndexSearch", iIndexSearch );
		
		for( int i=0; i<_vectorChildNodes.size(); ++i )
		{
			nodeChild = (XGRBaseDataNode)_vectorChildNodes.elementAt( i );

			// Check for name match
			//---------------------
			if( strName.equals( nodeChild.getName() ) == true )
			{
				++iIndexFound;
				
				// Check for occurance match
				//--------------------------
				if( iIndexSearch == -1 || iIndexSearch == iIndexFound )
				{
					// If no subpath, the child to the returned node list
					//---------------------------------------------------
					if( iPeriod < 0 )
					{
						vectorReturnedNodes.addElement( nodeChild );
					}

					// Otherwise, continue down the subpath
					//-------------------------------------
					else
					{
						String strSubpath = strPath.substring( iPeriod + 1 );
			
						nodeChild.getNodesRecursive( strSubpath, vectorReturnedNodes );
					}

					// Break out if required occurance has been found
					//-----------------------------------------------
					if( iIndexSearch != -1 && iIndexSearch == iIndexFound )
					{
						break;
					}
				}
			}
		}
	}

//	Trace.functionExit( this, "getNodesRecursive" );
	return;
}
/**
 *
 */
public IXGRDataNode getParentNode() 
{
	return _nodeParent;
}
/**
 *
 */
public boolean removeChildNode( IXGRDataNode nodeChild ) 
{
	if( nodeChild != null && _vectorChildNodes != null )
	{
		return _vectorChildNodes.removeElement( nodeChild );
	}
	
	return false;
}
/**
 */
public void setParentNode( IXGRDataNode nodeParent )
{
	_nodeParent = nodeParent;
}
/**
 */
public String toString()
{
	int iNumberOfChildren = _vectorChildNodes == null ? 0 : _vectorChildNodes.size();
	return new String( "name=" + getName() + " value=" + getValue() + " children=" + Integer.toString( iNumberOfChildren ) );
}
}
