package org.eclipse.scripting.internal.datamodels;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This handler handles the creation of element and attribute tree nodes for this
 * data model.  This handler also recognizes only one DTD.  The input source
 * for the DTD is done at construction time.
 */
import org.xml.sax.*;
import java.io.*;
import org.eclipse.scripting.internal.*;

public class XGRXmlElementsAndAttributesSaxHandler extends XGRXmlSaxHandler
{
	protected boolean _bFirstElement = true;
	
	protected static InputSource _inputSourceItpseaDtd = null;
	
	protected static final String _strItpseaDtd = "<?xml encoding=\"US-ASCII\"?><!ELEMENT itpsea (element*)><!ATTLIST itpsea type CDATA #REQUIRED><!ELEMENT element (attribute*,element*)><!ATTLIST element name CDATA #REQUIRED><!ELEMENT attribute EMPTY><!ATTLIST attribute name CDATA #REQUIRED value CDATA #REQUIRED>";
/**
 * This XGRXmlElementsAndAttributesSaxHandler constructor creates a static
 * InputSource which contains the DTD content.  The DTD string is a static
 * String.  This is done when the first instance is constructed.
 */
public XGRXmlElementsAndAttributesSaxHandler( IXGRDataNode nodeRoot, String strDirectory ) 
{
	super( nodeRoot, strDirectory );

	// Create the single instawnce of an
	// input source that holds iptsea.dtd
	//-----------------------------------
	if( _inputSourceItpseaDtd == null )
	{
		// Generate an input source for itpsea.dtd
		//----------------------------------------
		ByteArrayInputStream inputStream = new ByteArrayInputStream( _strItpseaDtd.getBytes() );

		if( inputStream != null )
		{
			_inputSourceItpseaDtd = new InputSource( inputStream );
		}
	}
}
/**
 */
public void endElement( String strName ) throws SAXException 
{
//	Trace.functionEntry( this, "endElement" );

//	Trace.string( strName );

	XGRXmlDataNode nodeTop = (XGRXmlDataNode)_stackNodes.pop();

//	Trace.string( "Popped " + nodeTop.getName() );
	
	if( nodeTop.getXmlType() == IXGRDataNodeXml.ELEMENT )
	{
		_nodeCurrent = nodeTop;
	}
	
//	Trace.functionExit( this, "endElement" );
}
/**
 * This is the only method of the IEntityResolver interface.  This
 * method is here so that DTDs can be found.  This data model only
 * recognizes one DTD.  This method recognizes this model's DTD filename
 * and returns the InputSource previously created at construction time.
 */
public InputSource resolveEntity( String strPublicId, String strSystemId ) throws IOException, SAXException 
{
//	Trace.functionEntry( this, "resolveEntity" );

	InputSource inputSource = null;
	
//	Trace.stringValue( "publicId", strPublicId );
//	Trace.stringValue( "systemId", strSystemId );

	if( strSystemId != null )
	{
		// Look for the DTD that this model supports
		//------------------------------------------
		if( strSystemId.lastIndexOf( "itpsea.dtd" ) == strSystemId.length() - 10 )
		{
			if( _inputSourceItpseaDtd != null )
			{
				inputSource = _inputSourceItpseaDtd;
			}
		}
		
		if( inputSource == null )
		{
			inputSource = super.resolveEntity( strPublicId, strSystemId );
		}
	}	

//	Trace.object( "inputSource", inputSource );
//	Trace.functionExit( this, "resolveEntity" );
	return inputSource;
}
/**
 */
public void startElement( String namespaceURI, String strElement, String qName, Attributes attributes ) throws SAXException
{
//	Trace.functionEntry( this, "startElement" );

//	Trace.string( strElement );
//	Trace.stringValue( "current node", _nodeCurrent.getName() );

	XGRXmlDataNode nodeChild = null;

	// Element tag
	//------------
	if( strElement.equals( "element" ) == true )
	{
		if( _bFirstElement == true )
		{
			throw new SAXException( "Incompatible model" );
		}
		
		// Obtain the element's name from the "name" attribute
		//----------------------------------------------------
		String strElementName = null;
	
		for( int i=0; i<attributes.getLength(); ++i )
		{
			if( attributes.getLocalName( i ).equals( "name" ) == true )
			{
				strElementName = attributes.getValue( i );
				break;
			}
		}

		// Create the element node
		//------------------------
		if( strElementName != null )
		{
			nodeChild = new XGRXmlElementsAndAttributesDataNode( _nodeCurrent, 
			                                strElementName,
			                                null,
			                                IXGRDataNodeXml.ELEMENT );
			
//			Trace.string( "Added element " + nodeChild.getName() + " to element " + _nodeCurrent.getName() );

			_stackNodes.push( _nodeCurrent );
			_nodeCurrent = nodeChild;
		}
	}

	// Attribute tag
	//--------------
	else if( strElement.equals( "attribute" ) == true )
	{
		if( _bFirstElement == true )
		{
			throw new SAXException( "Incompatible model" );
		}
		
		// Obtain the attribute's name from the "name" attribute
		// and value from the "value" attribute
		//------------------------------------------------------
		String strAttributeName  = null;
		String strAttributeValue = null;
	
		for( int i=0; i<attributes.getLength(); ++i )
		{
			if( attributes.getLocalName( i ).equals( "name" ) == true )
			{
				strAttributeName = attributes.getValue( i );
			}
			
			else if( attributes.getLocalName( i ).equals( "value" ) == true )
			{
				strAttributeValue = attributes.getValue( i );
			}	
		}
			
		if( strAttributeName != null && strAttributeValue != null )
		{
			// Create a new node and add it to its parent
			//-------------------------------------------
			nodeChild = new XGRXmlElementsAndAttributesDataNode( _nodeCurrent,
									        strAttributeName,
										    strAttributeValue,
										    IXGRDataNodeXml.ATTRIBUTE );

//			Trace.string( "Added attribute " + strAttributeName + " to element " + _nodeCurrent.getName() );

			_stackNodes.push( _nodeCurrent );
		}
	}
	
	// Add all other tags to the tree as is with their attributes
	// including the document name tag
	//-----------------------------------------------------------
	else
	{
		if( _bFirstElement == true )
		{
			if( strElement.equals( "itpsea" ) == false )
			{
				throw new SAXException( "Incompatible model" );
			}
			else
			{
				_bFirstElement = false;
			}
		}
		
		super.startElement( namespaceURI, strElement, qName, attributes );
	}
	
//	Trace.functionExit( this, "startElement" );
}
}
