package org.eclipse.scripting.internal.datamodels;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This handler creates a tree of XgrXmlDataNodes.  The root of the tree
 * is a dummy node.  This is so that a search of indexed elements can
 * be accomplished easier.  XML attributes become child elements.  Each
 * element is identified by its type of either ELEMENT, or ATTRIBUTE.
 */
import org.xml.sax.*;
import org.eclipse.scripting.internal.*;
import java.io.*;
import java.util.Stack;

public class XGRXmlSaxHandler implements ContentHandler, ErrorHandler, EntityResolver
{
	protected Stack          _stackNodes              = null;
	protected String         _strDirectory            = null;
	protected String         _strDocumentDeclarations = null;
	protected IXGRDataNode   _nodeCurrent             = null;
	protected IXGRDataNode   _nodeRoot                = null;
/**
 */
public XGRXmlSaxHandler( IXGRDataNode nodeRoot, String strDirectory )
{
//	Trace.functionEntry( this, "XGRXmlSaxHandler" );

	_nodeRoot = _nodeCurrent = nodeRoot;
	_strDirectory = strDirectory;

//	Trace.functionExit( this, "XGRXmlSaxHandler" );
}
/**
 * This method is called for characters that reside between
 * element tags.
 */
public void characters( char[] ch, int iStart, int iLength ) throws SAXException 
{
//	Trace.functionEntry( this, "characters" );

	String strCharacters = new String( ch );	

//	Trace.stringValue( "strCharacters", strCharacters );
	
	if( _strDocumentDeclarations == null )
	{
		_strDocumentDeclarations = strCharacters.substring( 0, iStart );
		
//		Trace.stringValue( "iStart strDocumentDeclarations", _strDocumentDeclarations );

		int iEnd = _strDocumentDeclarations.lastIndexOf("<");

		if( iEnd != -1 )
		{
			_strDocumentDeclarations = _strDocumentDeclarations.substring( 0, iEnd );
//			Trace.stringValue( "iEnd strDocumentDeclarations", _strDocumentDeclarations );
		}
	}
	
	strCharacters = strCharacters.substring( iStart, iStart + iLength ).trim();

	if( strCharacters.length() > 0 && _nodeCurrent != null )
	{
		_nodeCurrent.setValue( strCharacters );
		
//		Trace.string( strCharacters );
	}
	
//	Trace.functionExit( this, "characters" );
}
/**
 */
public void endDocument() throws SAXException 
{
//	Trace.functionEntry( this, "endDocument" );

	_stackNodes  = null;
	_nodeCurrent = null;
	
//	Trace.functionExit( this, "endDocument" );
}
	/**
	 * Receive notification of the end of an element.
	 */
public void endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName) throws org.xml.sax.SAXException {
	_nodeCurrent = (XGRXmlDataNode)_stackNodes.pop();
}
	/**
	 * End the scope of a prefix-URI mapping.
	 */
public void endPrefixMapping(java.lang.String prefix) throws org.xml.sax.SAXException {}
/**
 */
public void error( SAXParseException ex )
{
//	Trace.functionEntry( this, "error" );
	
	ScriptReporter.reportSAXParseException( ex );

//	Trace.functionExit( this, "error" );
}
/**
 */
public void fatalError( SAXParseException ex )
{
//	Trace.functionEntry( this, "fatalError" );
	
	ScriptReporter.reportSAXParseException( ex );

//	Trace.functionExit( this, "fatalError" );
}
/**
 */
public String getDocumentDeclarations() 
{
	return _strDocumentDeclarations;
}
/**
 */
public IXGRDataNode getRootNode()
{
	return _nodeRoot;
}
/**
 */
public void ignorableWhitespace( char[] ch, int start, int length ) throws SAXException 
{
//	Trace.functionEntry( this, "ignorableWhitespace" );
//	Trace.functionExit( this, "ignorableWhitespace" );
}
/**
 */
public void processingInstruction( String strTarget, String strData ) throws SAXException 
{
//	Trace.functionEntry( this, "processingInstruction" );

//	Trace.string( "target: " + strTarget );
//	Trace.string( "data:   " + strData   );
	
//	Trace.functionExit( this, "processingInstruction" );
}
/**
 * This is the only method of the IEntityResolver interface.  This
 * method is here so that DTDs can be found in the same directory as
 * the XML file.  Only systemIds are received.  The first one for the
 * xml file, and the second for the dtd file.  When the xml file is
 * received, its directory string is stored.  When the dtd file is received,
 * its filename is appended to the stored directory string.
 */
public InputSource resolveEntity( String strPublicId, String strSystemId ) throws IOException, SAXException 
{
//	Trace.functionEntry( this, "resolveEntity" );

	InputSource inputSource = null;
	
//	Trace.stringValue( "publicId", strPublicId );
//	Trace.stringValue( "systemId", strSystemId );

	if( strSystemId != null )
	{
		if( strSystemId.lastIndexOf( ".xml" ) == strSystemId.length() - 4 )
		{
			int iIndex = strSystemId.lastIndexOf( "\\" );

			if( iIndex > -1 )
			{
				_strDirectory = strSystemId.substring( 0, iIndex );
			}
			else
			{
				_strDirectory = System.getProperty( "user.dir" );
			}
		}

		else if( _strDirectory != null && strSystemId.lastIndexOf( ".dtd" ) == strSystemId.length() - 4 )
		{
			int iIndex = strSystemId.lastIndexOf( "/" );

			if( iIndex > -1 )
			{
				
				String strFilename = strSystemId.substring( iIndex + 1 );
				String strFilespec = _strDirectory + "\\" + strFilename;

				// Create a reader
				//----------------
				BufferedReader bufferedReader = null;
	
				try
				{
					// Attempt to open the file in read mode
					//--------------------------------------
					FileReader fileReader = new FileReader( strFilespec );

					bufferedReader = new BufferedReader( fileReader );
				}

				catch( IOException ex )
				{
					ScriptReporter.reportException( ex );
					throw( ex );
				}

				// Create a sax input source
				//--------------------------
				if( bufferedReader != null )
				{
					inputSource = new InputSource( bufferedReader );
				}

				if( inputSource == null )
				{
					ScriptReporter.reportError( ScriptStrings.getString( "S_Cannot_find_file" ), strFilespec );
				}
			}
		}
	}	

//	Trace.object( "inputSource", inputSource );
//	Trace.functionExit( this, "resolveEntity" );
	return inputSource;
}
/**
 */
public void setDocumentLocator( Locator locator) 
{	
//	Trace.functionEntry( this, "setDocumentLocator" );
//	Trace.functionExit( this, "setDocumentLocator" );
}
	/**
	 * Receive notification of a skipped entity.
	 */
public void skippedEntity(java.lang.String name) throws org.xml.sax.SAXException {}
/**
 */
public void startDocument() throws SAXException 
{
//	Trace.functionEntry( this, "startDocument" );

	// Create an element stack
	// Push on startElement, pop on endElement
	//----------------------------------------
	_stackNodes = new Stack();

	// Create a root node
	// The root node is not a part of the data tree
	//---------------------------------------------
//	_nodeRoot = _nodeCurrent = new XGRXmlDataNode( null, "ROOT", null, IXGRDataNodeXml.UNKNOWN );

	// Put it on the stack
	//--------------------
	_stackNodes.push( _nodeRoot );

//	Trace.functionExit( this, "startDocument" );
}
	/**
	 * Receive notification of the beginning of an element.
	 */
public void startElement(String namespaceURI, String strElementName, String qName, Attributes attributes) throws SAXException {
//	Trace.functionEntry( this, "startElement" );

//	Trace.string( strElementName );

	// Create a new node
	//------------------
	XGRXmlDataNode nodeChild = new XGRXmlDataNode( _nodeCurrent, strElementName, null, IXGRDataNodeXml.ELEMENT );

//	Trace.string( "Added " + strElementName + " to " + _nodeCurrent.getName() );
	
	// Put it on the stack
	//--------------------
	_stackNodes.push( _nodeCurrent );

	_nodeCurrent = nodeChild;

	// Add property nodes to the element node
	//---------------------------------------
	for( int i=0; i<attributes.getLength(); ++i )
	{
		// Create a new node and add it to its parent
		//
		// attributes.getType(i)  returns "CDATA" );
		// attributes.getName(i)  returns the name of the property
		// attributes.getValue(i) returns the value of the property
		//---------------------------------------------------------
		new XGRXmlDataNode( nodeChild, 
						    attributes.getLocalName( i ),
						    attributes.getValue( i ),
						    IXGRDataNodeXml.ATTRIBUTE );

		// Debug only
		//-----------
		// Trace.string( "-----------------------" );
		// Trace.string( attributes.getName ( i ) );
		// Trace.string( attributes.getType ( i ) );
		// Trace.string( attributes.getValue( i ) );
	}
	
//	Trace.functionExit( this, "startElement" );
	
}
	/**
	 * Begin the scope of a prefix-URI Namespace mapping.
	 */
public void startPrefixMapping(java.lang.String prefix, java.lang.String uri) throws org.xml.sax.SAXException {}
/**
 */
public void warning( SAXParseException ex )
{
//	Trace.functionEntry( this, "warning" );
	
	ScriptReporter.reportSAXParseException( ex );

//	Trace.functionExit( this, "warning" );
}
}
