package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

/**
 * A contribution manager that manages contribution items in a tree
 * strucutre. Each element in the tree can be accessed by a path. The
 * path consists of the contribution node's ids separated by a period.<p>
 *
 * <b>The manager does not guarantee</b> that an contribution item is
 * only inserted once. If you search for an contribution item by id
 * and two contribution items with the same id exists always the item 
 * that was inserted first is returned.
 */
public abstract class ScriptContributionNodeManager {

	ScriptDefaultContributionNode fRoot= new ScriptDefaultContributionNode("");
	
	/** The path separator character */
	String fSeparator;
	
	/** pre-order traversal means visit the root first, then the children */
	public static final int PRE_ORDER = 0;

	/** post-order means visit the children, and then the root */
	public static final int POST_ORDER = 1;
	
	/**
	 * Creates a new ContributionNodeManager. Path separator character
	 * is the dot
	 */
	public ScriptContributionNodeManager() {
		super();
		fSeparator= ".";
	}
	/**
	 * Creates a new ContributionNodeManager. Path separator character
	 * is the given character.
	 */
	public ScriptContributionNodeManager(char separator) {
		super();
		fSeparator= new String(new char[] {separator});
	}
	/**
	 * Adds the given contribution node to the contribution node
	 * determined by the given path. <code>True</code> is returned
	 * if adding was successful.
	 */
	public boolean addTo(String path, IScriptContributionNode node) {
		IScriptContributionNode target= find(path);
		if (target == null)
			return false;
		target.add(node);
		return true;	
	}
	/**
	 * Adds a new contribution item.
	 */
	public void addToRoot(IScriptContributionNode node) {
		
		fRoot.add(node);
	}
	protected void buildSequence(IScriptContributionNode node, List sequence, int order) {
		if (order == PRE_ORDER)
			sequence.add(node);
		Iterator e= node.getSubNodes();
		while (e.hasNext()) {
			buildSequence((IScriptContributionNode)e.next(), sequence, order);
		}
		if (order == POST_ORDER)
			sequence.add(node);	
	}
	/**
	 * Finds the contribution it determined by the given path
	 */
	public IScriptContributionNode find(String path) {
		
		StringTokenizer stok= new StringTokenizer(path, fSeparator);
		IScriptContributionNode node= fRoot;
		while(stok.hasMoreTokens()) {
			String id= stok.nextToken();
			node= node.findSubNode(id);
			if (node == null)
				return null;
		}
		return node;
	}
	/**
	 * Returns all contribution nodes added to this contribution manager.
	 * @param order Specifies the ordering of the nodes. Valid values are
	 *	<code>PRE_ORDER</code> and <code>POST_ORDER</code>.
	 *
	 */
	public List getElements(int order) {
		
		ArrayList sequence= new ArrayList();
		Iterator rootPages= getRoot().getSubNodes();
		while(rootPages.hasNext())
			buildSequence((IScriptContributionNode)rootPages.next(), sequence, order);
		return sequence;
	}
	/**
	 * Returns the root node
	 */
	protected IScriptContributionNode getRoot() {
		return fRoot;
	}
	/**
	 * Removes the contribution item specified by the given path. If
	 * a node is present under the path, the node is returned; otheriwse
	 * <code>null</code>.
	 */
	public IScriptContributionNode remove(String path) {
		
		int index= path.lastIndexOf(fSeparator);
		if (index == -1)
			return fRoot.remove(path);
		// Make sure that the last character in the string isn't the "."
		
		String parentPath= path.substring(0, index);
		String id= path.substring(index + 1);
		IScriptContributionNode parentNode= find(parentPath);
		if (parentNode == null)
			return null;
		return parentNode.remove(id); 
	}
	/**
	 * Removes the given contribution item if it is managed by
	 * this contribution manager.
	 */
	public boolean remove(IScriptContributionNode node) {
		if (node == null)
			return false;
		return fRoot.remove(node);	
	}
	/**
	 * Removes all contribution items.
	 */
	public void removeAll() {
		fRoot= new ScriptDefaultContributionNode("");
	}
}
