package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.util.Iterator;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferencePage;
import org.eclipse.jface.preference.IPreferencePageContainer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.util.Assert;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.widgets.Widget;

/**
 * A small framework designed to provide hierarchical access to several
 * pages of preference panels. Each page is represented by a node in the tree
 * on the left portion of the composite window. When the node is selected, the page
 * is shown on the right of the composite window.
 */
public class ScriptPreferenceViewer extends Composite
		implements IPreferencePageContainer, SelectionListener
{
	Composite _compositeParent;
	private IPreferenceStore fPreferenceStore;
	private ScriptPreferencePage fCurrentPage;
	private ScriptPreferenceManager fPreferenceManager;
	
	private Tree fTree;
	private Composite fTitleContainer;
	private Label fTitle;
	private Composite fPageContainer;
	private Point fUserPreferredPageSize= new Point(1000,1000);
	private Label fMessageLine;
	private TreeItem fCurrentTreeItem;

	private String fErrorMessage;
	private String fMessage;
	
	static class PageLayout extends Layout {
		public void layout(Composite composite, boolean force) {
			Rectangle rect = composite.getClientArea();
			Control [] children = composite.getChildren();
			for (int i= 0; i < children.length; i++) {
				children[i].setSize(rect.width, rect.height);
			}
		}
		public Point computeSize(Composite composite, int wHint, int hHint, boolean force) {
			Rectangle rect = composite.getClientArea ();
			if (wHint != SWT.DEFAULT) rect.width = wHint;
			if (hHint != SWT.DEFAULT) rect.height = hHint;
			return new Point (rect.width, rect.height);
		}	
	}

	public ScriptPreferenceViewer(Composite compositeParent, int iStyle
								  , ScriptPreferenceManager manager)
	{
		super(compositeParent, iStyle);
		Assert.isNotNull(manager);
		fPreferenceManager= manager;
	}
	/**
	 * Adjust the gui according to the state of the dialog's pages.
	 */
	private void adjustGUI() {
		boolean b= isCurrentPageValid();
	}
	/**
	 * Creates a Tree/TreeItem structure that reflects the page hierarchy.
	 */
	private void buildTree(Composite parent) {
		if (fTree != null)
			fTree.dispose();
	
		fTree = new Tree(parent, SWT.BORDER);
		
		fTree.addSelectionListener( this );

		IScriptContributionNode node= fPreferenceManager.getRoot();
		Iterator e= node.getSubNodes();
		while (e.hasNext()) {
			buildTreeItemFor(fTree, (IScriptPreferenceNode)e.next());
		}
	}
	/**
	 * Creates a TreeItem structure that reflects to the page hierarchy.
	 */
	private void buildTreeItemFor(Widget parent, IScriptPreferenceNode node) {
		
		TreeItem item= null;
		if (parent instanceof Tree)
			item= new TreeItem((Tree)parent, SWT.DEFAULT);
		else
			item= new TreeItem((TreeItem)parent, SWT.DEFAULT);
			
		item.setData(node);
		item.setText(node.getLabel());
		Image image= node.getImage();
		if (image != null) {
			item.setImage(image);
		}	
	
		Iterator subNodes= node.getSubNodes();
		while (subNodes.hasNext()) {
			buildTreeItemFor(item, (IScriptPreferenceNode)subNodes.next());
		}	
	}
	/**
	 * Create the Button bar visible at the bottom of this window.
	 */
	private Composite createButtonBar(Composite parent) {
		Composite result= new Composite(parent, SWT.NULL);
		GridLayout layout= new GridLayout();
		layout.numColumns= 2;
		layout.marginHeight= 0; layout.marginWidth= 0;	
		result.setLayout(layout);
	
		GridData gd;
		
		fMessageLine= new Label(result, SWT.NULL);
//		FontUtil.applyDialogFontTo(fMessageLine);
		fMessageLine.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ));
		fMessageLine.setText("");
	
		result.layout(true);
		return result;
	}
/**
 * Create the inner content of the preference dialog
 */
public Control createContents(Composite compositeParent ) {

	_compositeParent = compositeParent;
	int c;
	if(SWT.getPlatform().equals("motif"))
		c = SWT.COLOR_WIDGET_DARK_SHADOW;
	else
		c = SWT.COLOR_WIDGET_NORMAL_SHADOW;

	Color titleBackground = getDisplay().getSystemColor(c);
	int titleHeightMargin = 5;

	GridData gd;
	gd = new GridData(GridData.FILL_BOTH);
	setLayoutData(gd);
	GridLayout layout = new GridLayout();
	layout.numColumns = 2;
	setLayout(layout);

	// Build the tree an put it into the composite.
	buildTree(this);
	gd = new GridData(GridData.FILL_VERTICAL);
	gd.widthHint = 150;
	gd.verticalSpan = 2;
	fTree.setLayoutData(gd);

	// Build page title's label
	fTitleContainer = new Composite(this, SWT.NULL);
	fTitleContainer.setBackground(titleBackground);
	layout = new GridLayout();
	//layout.marginWidth= 0; layout.horizontalSpacing= 0;
	fTitleContainer.setLayout(layout);
	fTitleContainer.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

	fTitle = new Label(fTitleContainer, SWT.CENTER);
	fTitle.setAlignment(SWT.CENTER);
	fTitle.setBackground(titleBackground);
	fTitle.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

	// Build the Page container
	fPageContainer = createPageContainer(this);
//	fPageContainer.setSize(fUserPreferredPageSize);
	fPageContainer.setLayoutData(new GridData(GridData.FILL_BOTH));

	// Build the separator line
	Label separator = new Label(this, SWT.HORIZONTAL | SWT.SEPARATOR);
	gd = new GridData(GridData.FILL_HORIZONTAL);
	gd.horizontalSpan = 2;
	separator.setLayoutData(gd);

	// Build the Button Bar 
	Composite buttonBar = createButtonBar(this);
	gd = new GridData(GridData.FILL_HORIZONTAL);
	gd.horizontalSpan = 2;
	buttonBar.setLayoutData(gd);

	layout(true);
	// System.out.println("Page Container size: " + fPageContainer.getSize());

	selectFirstItem();

	return this;
}
	/**
	 * Creates the inner content that holds the pages' UI component.
	 */
	private Composite createPageContainer(Composite parent) {
		Composite result= new Composite(parent, SWT.NULL);
		result.setLayout(new PageLayout());
		return result;
	}
	public ScriptPreferenceManager getPreferenceManager()
	{
		return fPreferenceManager;
	}
	/**
	 * Returns my preference bunble.
	 */
	public IPreferenceStore getPreferenceStore() {
		return fPreferenceStore;
	}
	public Control getWindow()
	{
		return fPageContainer;
	}
	/**
	 * Check whether the current page is valid or not.
	 */
	private boolean isCurrentPageValid() {
		if (fCurrentPage == null)
			return true;
		else
			return fCurrentPage.isValid();	
	}
	/**
	 * Select the page in the page hierarchy which is determined by
	 * <code>fCurrentTreeItem</code>.
	 */
	private void selectCurrentPageAgain() {
		fTree.setSelection(new TreeItem[] { fCurrentTreeItem });
		fCurrentPage.setVisible(true);
	}
	/**
	 * Select the first item in the tree.
	 */
	protected void selectFirstItem() {
		if (fTree != null) {
			int count= fTree.getItemCount();
			if (count > 0) {
				TreeItem[] items= fTree.getItems();
				Object data= items[0].getData();
				if (data instanceof IScriptPreferenceNode) {
					fTree.setSelection(new TreeItem[] { items[0] });
					turnToPage((IScriptPreferenceNode)data);
				}
			}	
		}	
	}
	/**
	 * Sets the preference bunble for this preference dialog.
	 */
	public void setPreferenceBundle(IPreferenceStore prefs) {
		Assert.isNotNull(prefs);
		fPreferenceStore= prefs;
	}
	/**
	 * Show the "Page Flipping abort" dialog.
	 */
	private void showPageFlippingAbortDialog() {

			MessageDialog dialog= new MessageDialog(
			getShell(),
			"AbortPageFlippingDialog.title",
			null,
			JFaceResources.getString("AbortPageFlippingDialog.message"),
			SWT.ICON_WARNING,
			new String [] {IDialogConstants.OK_LABEL},
			0);
		dialog.open();	
	}
/**
 * Flip to another page. The methods returns true if the page flip
 * was successful otherwise false.
 */
protected boolean turnToPage(IScriptPreferenceNode node) {
	
	if (node != null) {
		IPreferencePage newPage = node.getPreferencePage();
		if (newPage == fCurrentPage)
			return true;

		Control currentWindow = null;
		if (fCurrentPage != null) {
			if (!fCurrentPage.leavePage())
				return false;
			currentWindow = fCurrentPage.getControl();
		};

		// Ensure that the page control has been created
		// (this allows lazy page control creation)
		if (newPage.getControl() == null) 
			newPage.createControl(fPageContainer);

		
		// Get the new page	
		Control content = newPage.getControl();
		if (content == null)
			return false;

		String title = newPage.getTitle();
		if (title == null)
			title = "";
		fTitle.setText(title);
		fMessageLine.setText("");

		// Resize page container
		//----------------------
		Point contentSize = ((ScriptPreferencePage) newPage).computeSizePage();
		fPageContainer.setSize(contentSize);

		content.setEnabled(true);

		((ScriptPreferencePage) newPage).enterPage();

		content.setVisible(true);

		if (currentWindow != null) {
			currentWindow.setEnabled(false);
			currentWindow.setVisible(false);
		}

		// Resize scrollbars
		//------------------
		pack(true);
		((ScriptViewport) getParent()).updateScrollbars();

		fCurrentPage = (ScriptPreferencePage) newPage;

		return true;
	}
	else {
		return false;
	}
}
/*
 * Method declared on IPreferenceContainer
 */
public void updateButtons() {
	// do nothing
}
/*
 * Method declared on IPreferencePageContainer.
 */
public void updateErrorMessage() {
	fMessageLine.setText(fCurrentPage.getErrorMessage());
}
/*
 * Method declared on IPreferencePageContainer.
 */
public void updateMessage() {
	fMessageLine.setText(fCurrentPage.getMessage());
}
/*
 * Method declared on IPreferencePageContainer.
 */
public void updateTitle() {
	String s = fCurrentPage.getTitle();
	if (s == null)
		s = "";
	fTitle.setText(s);
}
/**
 * Sent when default selection occurs in the control.
 * <p>
 * For example, on some platforms default selection occurs
 * in a List when the user double-clicks an item.
 * </p>
 *
 * @param e an event containing information about the default selection
 */
public void widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent e) {}
/**
 * widgetSelected method comment.
 */
public void widgetSelected(org.eclipse.swt.events.SelectionEvent event) {

	Object data = event.item.getData();
	if (data instanceof IScriptPreferenceNode) {
		if (!isCurrentPageValid()) {
			showPageFlippingAbortDialog();
			selectCurrentPageAgain();
		}
		else if (!turnToPage((IScriptPreferenceNode) data)) {
			// Page flipping wasn't successful
			showPageFlippingAbortDialog();
			selectCurrentPageAgain();
		}
		else {
			// Everything went well
			fCurrentTreeItem = (TreeItem) event.item;
		}
	}
}
}
