package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This class is an amalgamation of JFace's SmartGuideDialog,
 * and SmartGuide classes.  While the JFace SmartGuideDialog is
 * a StandardShell, this class is a Composite, and can be hosted by
 * another composite.
 */
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import java.util.*;
import org.eclipse.jface.*;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.*;
import java.util.*;
import org.eclipse.jface.wizard.ProgressMonitorPart;

public class ScriptWizard extends Composite
{
	// Constants indicating what action has triggers page flipping
	public static final int CANCEL   = 0;
	public static final int PREVIOUS = 1;
	public static final int NEXT     = 2;
	public static final int FINISH   = 3;

	protected boolean          _bHasPageTitleBar          = true;
	protected boolean          _bHasProgressMonitor       = true;
	protected boolean          _bHelpIsAvailable          = true;
	protected boolean          _bForcePreviousNextButtons = true;
	protected boolean          _bSendCancelToAllPages     = false;
	protected boolean          _bSendDoneToAllPages       = false;
	protected int              _iPageHeight               = SWT.DEFAULT;
	protected int              _iPageWidth                = SWT.DEFAULT;
	protected Button           _buttonCancel              = null;
	protected Button           _buttonFinish              = null;
	protected Button           _buttonHelp                = null;
	protected Button           _buttonNext                = null;
	protected Button           _buttonPrevious            = null;
	protected Composite        _compositeButtonBar        = null;
	protected Composite        _compositePages            = null;
	protected Composite        _compositeTitleBar         = null;
	protected Hashtable        _hashPageSizes             = new Hashtable();
	protected Image            _image                     = null;
	protected Image            _imagePageDefault          = null;
	protected Label            _labelImage                = null;
	protected Label            _labelTitle                = null;
	protected Label            _labelDescription          = null;
	protected Label            _messageLine               = null;
	protected PageContainerFillLayout _pageContainerLayout = null;
	protected ProgressMonitorPart _progressMonitorPart    = null;
	protected ScriptWizardPage _pageCurrent           = null;
	protected SelectionAdapter _cancelListener            = null; 
	protected Vector           _vectorPages               = new Vector();
	protected Color            _colorTitleBar             = null;

	/**
	 * A layout for a container which includes several pages, like
	 * a notebook, wizard, or preference dialog. The size computed by
	 * this layout is the maximum width and height of all pages currently
	 * inserted into the container.
	 */
	protected class PageContainerFillLayout extends Layout {

	    /**
	     * The margin width; <code>5</code> pixels by default.
	     */
	    public int marginWidth = 5;

	    /**
	     * The margin height; <code>5</code> pixels by default.
	     */
	    public int marginHeight = 5;

	    /**
	     * The minimum width; <code>0</code> pixels by default.
	     */
	    public int minimumWidth = 0;

	    /**
	     * The minimum height; <code>0</code> pixels by default.
	     */
	    public int minimumHeight = 0;

	    /**
		 * Creates new layout object.
		 *
		 * @param mw the margin width
		 * @param mh the margin height
		 * @param minW the minimum width
		 * @param minH the minimum height
		 */
		public PageContainerFillLayout(int mw, int mh, int minW, int minH) {
		    marginWidth = mw;
		    marginHeight = mh;
		    minimumWidth = minW;
		    minimumHeight = minH;
		}
		/*
		 * Method declared on Layout.
		 */
		public Point computeSize(Composite composite, int wHint, int hHint, boolean force) {
		    if (wHint != SWT.DEFAULT && hHint != SWT.DEFAULT)
		        return new Point(wHint, hHint);

		    Point result = null;
		    Control[] children = composite.getChildren();
		    if (children.length > 0) {
		        result = new Point(0, 0);
		        for (int i = 0; i < children.length; i++) {
		            Point cp = children[i].computeSize(wHint, hHint, force);
		            result.x = Math.max(result.x, cp.x);
		            result.y = Math.max(result.y, cp.y);
		        }
		        result.x = result.x + 2 * marginWidth;
		        result.y = result.y + 2 * marginHeight;
		    } else {
		        Rectangle rect = composite.getClientArea();
		        result = new Point(rect.width, rect.height);
		    }
		    result.x = Math.max(result.x, minimumWidth);
		    result.y = Math.max(result.y, minimumHeight);

		    if (wHint != SWT.DEFAULT)
		        result.x = wHint;
		    if (hHint != SWT.DEFAULT)
		        result.y = hHint;
		    return result;
		}
		/**
		 * Returns the client area for the given composite according to this layout.
		 */
		public Rectangle getClientArea(Composite c) {
		    Rectangle rect = c.getClientArea();
		    rect.x = rect.x + marginWidth;
		    rect.y = rect.y + marginHeight;
		    rect.width = rect.width - 2 * marginWidth;
		    rect.height = rect.height - 2 * marginHeight;
		    return rect;
		}
		/*
		 * Method declared on Layout.
		 */
		public void layout(Composite composite, boolean force) {
		    Rectangle rect = getClientArea(composite);
		    Control[] children = composite.getChildren();
		    for (int i = 0; i < children.length; i++) {
		        children[i].setBounds(rect);
		    }
		}
		/**
		 * Lays outs the page according to this layout.
		 *
		 * @param w the control
		 */
		public void layoutPage(Control w) {
		    w.setBounds(getClientArea((Composite) w.getParent()));
		}
		/**
		 * Sets the location of the page so that its origin is in the
		 * upper left corner.
		 *
		 * @param w the control
		 */
		public void setPageLocation(Control w) {
		    w.setLocation(marginWidth, marginHeight);
		}
	}
/**
 *
 */
public ScriptWizard( Composite compositeParent, int iStyle )
{
	super( compositeParent, iStyle );
	_pageContainerLayout = new PageContainerFillLayout(5, 5, 300, 225);
	
//	Trace.functionEntry( this, "ScriptWizard" );

	_cancelListener = new SelectionAdapter() 
	{
		public void widgetSelected(SelectionEvent e) 
		{
			cancelPressed();
		}
	};

//	Trace.functionExit( this, "ScriptWizard" );
}
/**
 * Adds a new page to the wizard
 */
public void addPage(ScriptWizardPage page)
{
//	Trace.functionEntry( this, "addPage" );
	
	if( _vectorPages == null )
	{
		_vectorPages= new Vector();
	}
	
	if( _vectorPages.contains( page ) == false )	
	{
		_vectorPages.addElement(page);
		page.setSmartGuide(this);
	}

//	Trace.functionExit( this, "addPage" );
}
/**
 * Checks if all pages are complete.
 */
public boolean areAllPagesComplete() 
{
	if (_vectorPages == null) 
		return true;
		
	int size= _vectorPages.size();
	boolean result= true;
	
	for (int i= 0; i < size; i++) 
	{
		result= result && ((ScriptWizardPage)_vectorPages.elementAt(i)).isPageComplete();
		if (!result)
			break;
	}
	
	return result;
}
	/**
	 * The back button has been pressed.
	 */
	protected void backPressed() {
		ScriptWizardPage page= _pageCurrent.getPreviousPage();
		turnToPage(page, PREVIOUS);
	}
	/**
	 * Called when smart guide is canceled. If the method 
	 * returns true, smart guide will close; otherwise, it 
	 * will stay active.
	 * This is a callback method that should be overriden
	 * in the smart guide subclass.
	 */
	protected boolean cancel() {
		return true;
	}
	/**
	 * The cancel button has been pressed.
	 */
	protected void cancelPressed() {
		// Close the dialog. The check whether the dialog can be
		// closed or not is done in <code>handleWindowClosing</code>.
		// This ensures that the check is also evaluated when the user
		// presses the window's close button.
		close();
	}
/**
 */
public void close() {}
	/**
	 * Creates the smard guide's inner content. There is no need for sublcasses to
	 * redefine this method as long as they do not want to change the layout of
	 * the smart guide.
	 */
	public Control createContents(Composite parent) 
	{
//		Trace.functionEntry( this, "createContents" );
		GridData gd;
		GridLayout layout= new GridLayout();
		layout.marginHeight= 0; layout.marginWidth= 0;
		layout.horizontalSpacing= 0; layout.verticalSpacing= 0;
		setLayout(layout);
	
		if ( _bHasPageTitleBar) 
		{
			_compositeTitleBar= createTitleBar( this );
			_compositeTitleBar.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ));
			// Build the separator line
			Label separator= new Label( this, SWT.HORIZONTAL | SWT.SEPARATOR);
			separator.setLayoutData(new GridData( GridData.FILL_HORIZONTAL ));
		}	
			
		// Build the Page container
		_compositePages= createPageContainer( this );
		gd= new GridData( GridData.FILL_BOTH );
		gd.widthHint= _iPageWidth; gd.heightHint= _iPageHeight;
		_compositePages.setLayoutData(gd);
	
		// Insert a progress monitor if one is needed
		if ( _bHasProgressMonitor) 
		{
			GridLayout pmlayout= new GridLayout();
			pmlayout.numColumns= 1;
			_progressMonitorPart= new ProgressMonitorPart( this, pmlayout, SWT.DEFAULT);
			_progressMonitorPart.setLayoutData(new GridData( GridData.FILL_HORIZONTAL ));
			_progressMonitorPart.setVisible(false);
		}	
	
		// Build the separator line
		Label separator= new Label( this, SWT.HORIZONTAL | SWT.SEPARATOR);
		separator.setLayoutData(new GridData( GridData.FILL_HORIZONTAL ));
				
		// Build the Button Bar 
		_compositeButtonBar= createMessageLineAndButtons( this );
		_compositeButtonBar.setLayoutData(new GridData( GridData.FILL_HORIZONTAL ));
		// Lazy creation of pages.
		createPages();
//		updateSizeOfTitleBar();
		
//		triggerAboutToOpen();	
			
		// Create all known pages.	
		insertPages();
		
//		Trace.functionExit( this, "createContents" );
		return this;
	}
	/** 
	 * Creates the Bar visible at the bottom of the wizard.
	 */
	private Composite createMessageLineAndButtons(Composite parent) {
		GridData gd;
		Composite result= new Composite(parent, SWT.NULL);
//		Font font= parent.getFont();
		GridLayout layout= new GridLayout();
		layout.marginWidth= 5; layout.numColumns= 2;
		layout.horizontalSpacing= 0; layout.marginHeight=6;
		result.setLayout(layout);
	
		_messageLine= new Label(result, SWT.NULL);
		_messageLine.setLayoutData(new GridData( GridData.FILL_HORIZONTAL ));
		_messageLine.setText("");
	
		Composite buttonBar= new Composite(result, SWT.NULL);
		layout= new GridLayout();
		layout.numColumns= 2; layout.makeColumnsEqualWidth= true;
		layout.marginWidth= 0; layout.marginHeight= 0;
		layout.horizontalSpacing= 0; layout.verticalSpacing= 0;
		buttonBar.setLayout(layout);
		gd= new GridData(); gd.horizontalIndent= 5;
		buttonBar.setLayoutData(gd);
		
		if (false) {
			layout.numColumns++;
			_buttonHelp= new Button(buttonBar, SWT.PUSH);
//			_buttonHelp.setFont(font);
			_buttonHelp.setText(IDialogConstants.HELP_LABEL);
			_buttonHelp.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					helpPressed();
				}
			});
			gd= new GridData( GridData.FILL_HORIZONTAL );
			_buttonHelp.setLayoutData(gd);
		}	
	
		if (getPageCount() > 1 || _bForcePreviousNextButtons) {
			layout.numColumns += 2;
			_buttonPrevious= new Button(buttonBar, SWT.PUSH);
//			_buttonPrevious.setFont(font); 
			_buttonPrevious.setText(IDialogConstants.BACK_LABEL);
			_buttonPrevious.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					backPressed();
				}
			});
			gd= new GridData( GridData.FILL_HORIZONTAL );
			if (layout.numColumns > 2)
				gd.horizontalIndent= 8;

			_buttonPrevious.setLayoutData(gd);
			
			_buttonNext= new Button(buttonBar, SWT.PUSH);
//			_buttonNext.setFont(font); 
			_buttonNext.setText(IDialogConstants.NEXT_LABEL);
			_buttonNext.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					nextPressed();
				}
			});	
			_buttonNext.setLayoutData(new GridData( GridData.FILL_HORIZONTAL ));
		}
		
		_buttonFinish= new Button(buttonBar, SWT.PUSH);
//		_buttonFinish.setFont(font);
		_buttonFinish.setText(IDialogConstants.FINISH_LABEL);
		_buttonFinish.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				finishPressed();
			}
		});	
		gd= new GridData( GridData.FILL_HORIZONTAL );
		gd.horizontalIndent= 8;
		_buttonFinish.setLayoutData(gd);
		
		_buttonCancel= new Button(buttonBar, SWT.PUSH);
//		_buttonCancel.setFont(font);
		_buttonCancel.setText(IDialogConstants.CANCEL_LABEL);
		_buttonCancel.addSelectionListener(_cancelListener);	
		gd= new GridData( GridData.FILL_HORIZONTAL );
		gd.horizontalIndent= 8;
		_buttonCancel.setLayoutData(gd);
		
		return result;
	}
/**
 * Creates the container that holds all pages.
 */
protected Composite createPageContainer(Composite parent) 
{
//	Trace.functionEntry( this, "createPageContainer" );
	
	Composite result= new Composite(parent, SWT.NULL);
	result.setLayout(_pageContainerLayout);

//	Trace.functionExit( this, "createPageContainer" );
	return result;
}
	/**
	 * Hook method for lazy page creation. Subclasses should reimplement
	 * this method if the want to create their pages just before the
	 * wizard is shown.
	 */
	protected void createPages() {
	}
	/**
	 * Creates the wizard's title bar.
	 */
	private Composite createTitleBar(Composite parent) {
		
		GridData gd;
		Composite result= new Composite(parent, SWT.NONE);
		Display display= result.getDisplay();
		Font font= parent.getFont();
	
		Color bg;
		RGB rgb= null;
		if (rgb != null) {
			if (_colorTitleBar != null)
				_colorTitleBar.dispose();
			_colorTitleBar= new Color(display, rgb);
			bg= _colorTitleBar;
		} else {
			bg= display.getSystemColor(SWT.COLOR_WHITE);
		}	
		
		result.setBackground(bg);
		GridLayout layout= new GridLayout();
		layout.numColumns= 2; layout.marginHeight= 0;
		layout.marginWidth= 0;
		result.setLayout(layout);
		
		Composite textBox= new Composite(result, SWT.NONE);
		textBox.setBackground(bg);
		layout= new GridLayout();
		textBox.setLayout(layout);
		
		textBox.setLayoutData(new GridData( GridData.FILL_BOTH ));
		
		_labelTitle= new Label(textBox, SWT.LEFT);
		_labelTitle.setBackground(bg);
//		_labelTitle.setFont(JFaceResources.getBannerFont());
		_labelTitle.setText("");
		gd= new GridData( GridData.FILL_HORIZONTAL );
		gd.verticalAlignment= gd.BEGINNING;
		_labelTitle.setLayoutData(gd);
	
		
		_labelDescription= new Label(textBox, SWT.LEFT);
		_labelDescription.setBackground(bg);
//		_labelDescription.setFont(font);
		gd= new GridData( GridData.FILL_BOTH );
		gd.horizontalIndent= 5;
		_labelDescription.setLayoutData(gd);

		_labelImage= new Label(result, SWT.LEFT);
//		_labelImage.setFont(font);
		_labelImage.setBackground(bg);
		_labelImage.setText("");
		gd= new GridData(); gd.horizontalAlignment= gd.END;
		_labelImage.setLayoutData(gd);
		
		return result;
	}
/**
 * Disables the currently displayed page.
 */
private void disablePage( ScriptWizardPage page )
{
//	Trace.functionEntry( this, "disablePage" );
	if (page == null)
		return;
		
	Control w= page.getControl( _compositePages );
	w.setVisible(false);
	w.setEnabled(false);
	page.setMessageLine(null);

//	Trace.functionExit( this, "disablePage" );
}
	boolean doCancelOnPages() {
		if ( _bSendCancelToAllPages == true) {
			boolean result= true;
			if (_vectorPages != null) {
				int size= _vectorPages.size();
				for (int i= 0; i < size; i++) {
					result= result && ((ScriptWizardPage)_vectorPages.elementAt(i)).cancel();
					if (!result)
						break;
				}	
			}
			return result;	
		} else {
			return _pageCurrent.cancel();
		}		
	}
	/**
	 * The user has pressed the finish button. If the method returns 
	 * true, the smart guide will close; otherwise, it will stay up. 
	 */
	protected boolean done() {
		return true;
	}
/**
 * Enables the currently displayed page.
 */
private void enablePage( ScriptWizardPage page, int direction) 
{
//	Trace.functionEntry( this, "enablePage" );
		
	if (page == null)
		return;
		
	page.setMessageLine( _messageLine );
	Control w= page.getControl( _compositePages );
	w.setEnabled(true);
		
	// Page gets visible the first time.
	if ( _hashPageSizes.get(w) == null) 
	{
//		Point contentSize= w.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
		Point contentSize = new Point(100, 100);		
		_hashPageSizes.put(w, contentSize);
	}	
		
	page.enter(direction);
	w.setVisible(true);

//	Trace.functionExit( this, "enablePage" );
}
	/**
	 * The finish button has been pressed.
	 */
	protected void finishPressed() {
		if (_pageCurrent != null) {
			// Checks whether the current page allows leaving and the pages allow
			// finishing.
			if (!_pageCurrent.leave(FINISH) || performDoneOnPages() == false )
			{
				return;
			}	
		}	
		if( done()) {
//		 	triggerAboutToClose();
			// Hard close the dialog if done was successful. We call <code>hardClose</code>
			// because the <code>handleWindowClosing</code> hook called by close performs a 
			// cancel check, which isn't necessary when <code>finish</code> was successful.
//			hardClose();
		}	
	}
	/**
	 * Returns the current page or null if no page is shown.
	 */
	public ScriptWizardPage getCurrentPage() {
		return _pageCurrent;
	}
	/**
	 * Returns the default page image.
	 */
	public Image getDefaultPageImage() {
		return null;
	}
	/**
	 * Returns the height of the given image.
	 */
	private int getImageHeight(Image image) {
		return image.getBounds().height;	
	}
/**
 * Returns the successor of the given page. Usually this is the page that 
 * was added after the given page to this smart guide.
 */
public ScriptWizardPage getNextPage( ScriptWizardPage page )
{
	int index;
		
	if( page == null || (index= _vectorPages.indexOf(page)) == -1 )
		return null;
	
	if( index < _vectorPages.size() - 1 )
		return (ScriptWizardPage)_vectorPages.elementAt( index + 1 );
	else
		return null;	
}
/**
 */
public Composite getPageComposite() {
	return _compositePages;
}
/**
 * Gets the number of pages in the smart guide.
 */
public int getPageCount()
{
	return _vectorPages != null ? _vectorPages.size() : 0;
}
/**
 * Returns the predecessor of the given page. Usually this is the page that was added 
 * right before the given page to this smart guide.
 */
public ScriptWizardPage getPreviousPage( ScriptWizardPage page )
{
	int index;
		
	if( page == null || ( index = _vectorPages.indexOf( page ) ) == -1 )
		return null;
	
	if( index > 0 )
		return (ScriptWizardPage)_vectorPages.elementAt( index - 1 );
	else
		return null;	
}
	/**
	 * Returns the height of the title text label.
	 */
	private int getTitleTextHeight() {
		// Since a font doesn't offer a method to get its height
		// we currently use a GC.
		GC gc= new GC( _labelTitle );
		Point extent= gc.textExtent("X");
		gc.dispose();
		return (extent.y + 2 * 5);
	}
	/**
	 * The user has pressed the help button.
	 */
	protected void helpPressed() {
		if (_pageCurrent != null) {
			_pageCurrent.displayHelp();
		}
	}
	/**
	 * Inserts all pages' UI components into my page container
	 */
private void insertPages() 
{
//	Trace.functionEntry( this, "insertPages" );
	
	Control w;

	for( int i=0; i<_vectorPages.size(); ++i )
	{
		ScriptWizardPage page = (ScriptWizardPage)_vectorPages.elementAt(i);
		w= page.getControl( _compositePages );
		w.setVisible(false);
		w.setEnabled(false);
	}

//	Trace.functionExit( this, "insertPages" );
}
	/**	
	 * The next button has been pressed.
	 */
	protected void nextPressed() {
		ScriptWizardPage page= _pageCurrent.getNextPage();
		turnToPage(page, NEXT);
	}
	/**
	 * Depending on the <code>sendCancelToAllPage</code> flag, <code>cancel</code> is
	 * called for the currently active page or for all pages.
	 */	
	protected boolean performCancelOnPages() {
		ScriptWizardPage currentPage= getCurrentPage();
		if (currentPage != null) {
			// Checks whether the current page allows leaving and the pages allow
			// canceling.
			if (!currentPage.leave(0) || 
					!doCancelOnPages()) {
				return false;
			}	
		}	
		return cancel();
	}
	/**
	 * Depending on the <code>sendDoneToAllPage</code> flag, <code>done</code> is
	 * called for the currently active page or for all pages.
	 */	
	protected boolean performDoneOnPages() {
		if ( _bSendDoneToAllPages == true ) { 
			boolean result= true;
			if ( _vectorPages != null) {
				int size= _vectorPages.size();
				for (int i= 0; i < size; i++) {
					result= result && ((ScriptWizardPage)_vectorPages.elementAt(i)).done();
					if (!result)
						break;
				}	
			}
			return result;	
		} else {
			return _pageCurrent.done();
		}		
	}
/**
 */
public void turnToFirstPage()
{
	if( _vectorPages.size() > 0 )
	{
		turnToPage( (ScriptWizardPage)_vectorPages.elementAt(0), NEXT );
	}
}
/**
 * Flips to the given page.
 */
protected void turnToPage( ScriptWizardPage page, int direction ) 
{
//	Trace.functionEntry( this, "turnToPage" );
		
	if (page == null)
		return;		
		
	ScriptWizardPage oldPage= _pageCurrent;
	// Check if we can leave the current page
	if (_pageCurrent != null) {
		if (!_pageCurrent.leave(direction))
			return;	
	}
	// Remember my previous page if we flip forward.
	if (direction == NEXT)
		page.setPreviousPage( _pageCurrent );
			
	// If we go back reset my previous page	
	else if (direction == PREVIOUS)
		_pageCurrent.setPreviousPage(null);	
		
	_pageCurrent= page;
	
	updateTitle();
	updateImage();
	updateDescription();
			
	enablePage(_pageCurrent, direction);
	disablePage(oldPage);
	updateButtons();

//	Trace.functionExit( this, "disablePage" );
}
	/**
	 * Adjusts the back, next, finish and cancel button to reflect the
	 * state of the currently active page.
	 */
	private void updateButtons() {

		if (_pageCurrent == null)
			return;
	
		if (_buttonPrevious != null)
			_buttonPrevious.setEnabled(_pageCurrent.getPreviousPage() != null);
		if (_buttonNext != null)
			_buttonNext.setEnabled(_pageCurrent.canFlipToNextPage());
		_buttonFinish.setEnabled(areAllPagesComplete());
	}
/**
 * Updates the desciption line.
 */
private void updateDescription() {
	// No current page. Do nothing
	if( _pageCurrent == null )
		return;
			
	String s= _pageCurrent.getDescription();
	
	if (s == null)
		s= "";
		
	_messageLine.setText(s);
}
	/**
	 * Updates the Icon.
	 */
	private void updateImage() {
		// No current page or no title bar. Do nothing
		if (_pageCurrent == null || !_bHasPageTitleBar)
			return;
			
		Image image= _pageCurrent.getImage();
		if (image == null) {
			_labelImage.setVisible(false);
		} else {
			_labelImage.setImage(image);
			_labelImage.setVisible(true);
		}	
	}
	/**
	 * Updates the title text.
	 */
	private void updateTitle() {
		// No current page or no title bar. Do nothing
		if( _pageCurrent == null || _bHasPageTitleBar == false )
			return;
			
		String s= _pageCurrent.getTitle();
		// SWT doesn't handle null strings
		if (s == null)
			s= "";
		_labelTitle.setText(s);
		_compositeTitleBar.layout(true);	
	}
}
