package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This is the preference page for the "Scripts" category.
 * This page allows a user to specify which plugins are to be available
 * to scripts
 */
import java.io.IOException;
import java.util.Hashtable;
import java.util.TreeSet;

import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.IPluginRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.model.PluginDescriptorModel;
import org.eclipse.core.runtime.model.PluginPrerequisiteModel;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.viewers.ColumnLayoutData;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.scripting.internal.ScriptPlugin;
import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

public class ScriptDevelopmentPreferencePage extends PreferencePage implements IWorkbenchPreferencePage, SelectionListener {
	
	protected Table _tablePluginLabels = null;
	protected Hashtable _hashLabelsToIDs = null;

	/**
	 * Displays list of plugins for user selection
	 */
	protected Control createContents(Composite compositeParent) {
		
		// Content
		//--------
		Composite compositeContent = new Composite(compositeParent, SWT.NULL);

		GridLayout layout = new GridLayout();
		compositeContent.setLayout(layout);

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		compositeContent.setLayoutData(gridData);

		// Label: Pre-specified Locations
		//-------------------------------
		Label label = new Label(compositeContent, SWT.NULL);
		label.setText(ScriptStrings.getString("S_Select_plug-ins_to_be_exposed_to_scripts"));

		// Table: Pre-specified Locations
		//------------------------------
		_tablePluginLabels = new Table(compositeContent, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER | SWT.CHECK | SWT.V_SCROLL | SWT.H_SCROLL);

		String[] columnTitles = new String[] { ScriptStrings.getString("")};
		int[] iColumnWeight = { 100 };
		TableLayout layoutTable = new TableLayout();

		for (int i = 0; i < columnTitles.length; i++) {
			TableColumn tableColumn = new TableColumn(_tablePluginLabels, SWT.NONE);
			tableColumn.setText(columnTitles[i]);
			ColumnLayoutData cLayout = new ColumnWeightData(iColumnWeight[i], true);
			layoutTable.addColumnData(cLayout);
		}

		_tablePluginLabels.setLinesVisible(true);
		_tablePluginLabels.setHeaderVisible(false);
		_tablePluginLabels.setLayout(layoutTable);
		
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.heightHint = 300;
		gridData.widthHint = 400;
		_tablePluginLabels.setLayoutData(gridData);
		_tablePluginLabels.addSelectionListener(this);

		initializeContent();

		return compositeContent;
	}
	/**
	 * Does nothing.
	 */
	public void init(IWorkbench aDesktop) {
	}
	/**
	 * Provides content for displayed list of plugin labels
	 */
	protected void initializeContent(){

		// Create links from plugin label keys to plugin ID values
		_hashLabelsToIDs = new Hashtable();
		
		// Create links from plugin ID keys to table item values
		Hashtable hashIDsToTableItems = new Hashtable();

		// Obtain all plugin descriptors
		IPluginRegistry registry = Platform.getPluginRegistry();
		IPluginDescriptor[] descriptors = registry.getPluginDescriptors();
		
		// Create an ordered list		
		TreeSet treeSet = new TreeSet();
		
		for (int i = 0; i < descriptors.length; ++i) {			
			if (descriptors[i].getLabel() != null) {

				// Place each entry in the ordered list, and
				// label/ID pairs in the hashtable
				// Filter out the user scripts plugin
				if( descriptors[i].getUniqueIdentifier().equals( "org.eclipse.scripting.userscripts" ) == false ){
					treeSet.add( descriptors[i].getLabel() );
					_hashLabelsToIDs.put( descriptors[i].getLabel(), descriptors[i].getUniqueIdentifier() );			
				}
			}
		}
		
		TableItem tableItem = null;
		Object[] objStrings = treeSet.toArray();
				
		for (int i = 0; i < objStrings.length; ++i) {
			if (objStrings[i] != null) {
				tableItem = new TableItem(_tablePluginLabels, SWT.NULL);
				tableItem.setText(0, (String)objStrings[i]);
				hashIDsToTableItems.put( (String)_hashLabelsToIDs.get(objStrings[i]), tableItem );
			}
		}
		
		// Obtain list of import IDs
		String[] straImports = loadImportList();
		
		// Set checked current import items
		// Detect whether previously selected plugins are now missing
		//-----------------------------------------------------------
		boolean bPluginMissing = false;
		
		for( int i=0; i<straImports.length; ++i ){
			
			tableItem = (TableItem)hashIDsToTableItems.get( straImports[i] );
			
			if( tableItem != null )
				tableItem.setChecked(true);
			else
				bPluginMissing = true;
		}
		
		// Set checked and grayed default items
		// org.eclipse.core.boot
		// org.eclipse.core.resources
		// org.eclipse.core.runtime
		// org.eclipse.scripting
		//-------------------------------------
		tableItem = (TableItem)hashIDsToTableItems.get( "org.eclipse.core.boot" );
		if( tableItem != null ){
				tableItem.setChecked(true);
				tableItem.setGrayed(true);
		}			
		
		tableItem = (TableItem)hashIDsToTableItems.get( "org.eclipse.core.resources" );
		if( tableItem != null ){
				tableItem.setChecked(true);
				tableItem.setGrayed(true);
		}			
		
		tableItem = (TableItem)hashIDsToTableItems.get( "org.eclipse.core.runtime" );
		if( tableItem != null ){
				tableItem.setChecked(true);
				tableItem.setGrayed(true);
		}			
		
		tableItem = (TableItem)hashIDsToTableItems.get( "org.eclipse.scripting" );
		if( tableItem != null ){
				tableItem.setChecked(true);
				tableItem.setGrayed(true);
		}
	
		// If a previously selected plugin is now missing,
		// save the now correct list, and display a restart required message
		//------------------------------------------------------------------
		if( bPluginMissing == true ){
			
			saveImportList();

			String strMessage = ScriptStrings.getString( "S_The_removal_of_one_or_more_selected_plug-ins_has_been_detected" );
			strMessage += "\n";
			strMessage += ScriptStrings.getString( "S_You_must_restart_the_workbench_to_activate_the_correction");
			
			MessageDialog.openInformation(getShell(), ScriptStrings.getString("S_Scripts"), strMessage);
		}
	}
	/**
	 * Obtains list of selected plugins as specified in the user scripts plugin
	 */
	protected String[] loadImportList(){
		ScriptPlugin scriptPlugin = ScriptPlugin.getPluginInstance();
		
		PluginDescriptorModel descriptorModel = scriptPlugin.getUserScriptsPluginManifest();
		PluginPrerequisiteModel[] prerequisiteModels = descriptorModel.getRequires();
		
		String[] straIDs = null;
		
		if( prerequisiteModels != null ){
			
			straIDs = new String[ prerequisiteModels.length ];
		
			for( int i=0; i<prerequisiteModels.length; ++i ){
				straIDs[i] = prerequisiteModels[i].getPlugin();
			}
		}
		
		else{
			straIDs = new String[0];
		}
		
		return straIDs;
	}
	/**
	 * When the Apply button is pressed, the selection is saved.
	 */
	protected void performApply(){
		saveImportList();
		MessageDialog.openInformation(getShell(), ScriptStrings.getString("S_Scripts"), ScriptStrings.getString("S_You_must_restart_the_workbench_to_activate_any_changes"));
	}
	/**
	 * When the Ok button is pressed, the selection is saved.
	 */
	public boolean performOk(){
		saveImportList();
		MessageDialog.openInformation(getShell(), ScriptStrings.getString("S_Scripts"), ScriptStrings.getString("S_You_must_restart_the_workbench_to_activate_any_changes"));
		return true;
	}
	/**
	 * Sets list of imports
	 */
	protected void saveImportList(){
		
		// Obtain the user scripts plugin manifest
		ScriptPlugin scriptPlugin = ScriptPlugin.getPluginInstance();
		
		PluginDescriptorModel descriptorModel = scriptPlugin.getUserScriptsPluginManifest();

		// Create a list of plugin IDs from the checked table items		
		TableItem[] tableItems = _tablePluginLabels.getItems();
		TreeSet treeSetIDs = new TreeSet();
		String strID = null;
		
		for( int i=0; i<tableItems.length; ++i ){
			if( tableItems[i].getChecked() == true ){
				strID = (String)_hashLabelsToIDs.get( tableItems[i].getText(0) );
				if( strID != null ) {
					treeSetIDs.add( strID );
				}
			}
		}

		// Convert the list of plugin IDs to an ordered array of models
		PluginPrerequisiteModel[] prerequisiteModels = new PluginPrerequisiteModel[ treeSetIDs.size() ];		

		Object[] objArray = treeSetIDs.toArray();
		
		for( int i=0; i<treeSetIDs.size(); ++i ){
			prerequisiteModels[i] = new PluginPrerequisiteModel();
			prerequisiteModels[i].setPlugin( (String)objArray[i] );
		}

		// Register the array of prerequisites		
		descriptorModel.setRequires( prerequisiteModels );
		
		// Save the plugin file
		try{
			scriptPlugin.saveUserScriptsPluginManifest();
		}
		catch( IOException ex ){
		}
		
		return;
	}
	public void widgetDefaultSelected( SelectionEvent e ){		
		widgetSelected( e );
	}
	public void widgetSelected( SelectionEvent e ){
		TableItem[] tableItems = _tablePluginLabels.getItems();
		
		for( int i=0; i<tableItems.length; ++i ){
			if( tableItems[i].getGrayed() == true && tableItems[i].getChecked() == false ){
				tableItems[i].setChecked( true );
				return;
			}
		}
	}
}
