package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * 
 */
import java.io.ByteArrayInputStream;
import java.io.InputStream;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentListener;
import org.eclipse.ui.editors.text.FileDocumentProvider;
import org.eclipse.ui.part.FileEditorInput;

public class ScriptUIEditorDocumentProvider extends FileDocumentProvider implements IDocumentListener{
	protected ScriptUIEditorModelElement _elementRoot = null;
	protected ScriptUIEditor             _editor      = null;

	protected final static String _strXmlHeader = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"xhtml1-transitional.dtd\">\n";

/**
 */
public ScriptUIEditorDocumentProvider() {
	super();
}
	/**
	 * The manipulation described by the document event will be performed.
	 */
public void documentAboutToBeChanged(org.eclipse.jface.text.DocumentEvent event) {}
	/**
	 * The manipulation described by the document event has been performed.
	 */
public void documentChanged(DocumentEvent event) {
	if( _editor != null )
	{
		_editor.textChanged( event );
	}
}
/**
 * This is a copy of the super class' method, except an XML header is
 * prepended to the document content.
 */
protected void doSaveDocument(IProgressMonitor monitor, Object element, IDocument document) throws CoreException {

	boolean bSave = true;

	if (element instanceof FileEditorInput) {

		// Apply text editor changes if any to do
		//---------------------------------------
		String strErrorMessage = _editor.doApplyTextChanges();
		
		if (strErrorMessage != null) {
			
			// Set the editor's message line
			//------------------------------
			_editor.setErrorMessage( strErrorMessage );
			
			// Display message box
			//--------------------
			bSave = MessageDialog.openQuestion(_editor.getShell(), "Errors were detected", strErrorMessage + "\n\n" + "Do you still wish to save?");
		}

		if (bSave == true) {

			StringBuffer strbContent = new StringBuffer();
			strbContent.append(_strXmlHeader);
			strbContent.append(document.get());

			InputStream stream = new ByteArrayInputStream(strbContent.toString().getBytes());

			IFile file = ((FileEditorInput)element).getFile();

			if (file.exists())
				file.setContents(stream, false, true, monitor);
			else
				file.create(stream, false, monitor);
		}
	}

	return;
}
/**
 * Tells the editor that an attribute of an element has changed.  The
 * editor will update the various views.
 */
public void elementAttributeChanged(ScriptUIEditorModelElement element) {
	_editor.doElementAttributeChange( element );
}
/**
 * Generates new string for the element, then replaces the current element
 * text with the new string, then generates new line numbers.
 */
public void elementChanged(Object objEditorInput, Object element) {

	IDocument document = getDocument(objEditorInput);

	if (document != null) {

		if (element instanceof ScriptUIEditorModelElement) {

			ScriptUIEditorModelElement modelElement = (ScriptUIEditorModelElement) element;

			// Obtain html element if this is the root
			//----------------------------------------
			if (modelElement == _elementRoot) {
				modelElement = modelElement.getChildElementAt(0);
			}

			try {
				// Determine current line indentation
				//-----------------------------------
				int iOffset = document.getLineOffset( modelElement._iLineNumberFirst );
				int iLength = document.getLineLength( modelElement._iLineNumberFirst );
				
				String strLineCurrent = document.get(iOffset, iLength);

				int iIndentation = 0;
				for (int i = 0; i < strLineCurrent.length(); ++i) {
					if (strLineCurrent.charAt(i) == ' ')
						iIndentation++;
					else
						break;
				}

				// Determine where to put the string in the document
				// before the element changes
				//--------------------------------------------------
				int iCharFirst = document.getLineOffset(modelElement._iLineNumberFirst);				
				int iCharLast = 0;
				
				try{
					iCharLast = document.getLineOffset(modelElement._iLineNumberLast) + document.getLineLength( modelElement._iLineNumberLast ) - 1;
				}
				catch( BadLocationException ex ){
					iCharLast = document.getLength() - 1;
				}

				// Obtain replacement string for the element
				//------------------------------------------
				StringBuffer strb = new StringBuffer(10000);
				modelElement.printElementString(strb, modelElement._iLineNumberFirst, iIndentation);

				// Set the replacement string
				//---------------------------
				document.removeDocumentListener(this);

				iLength = iCharLast - iCharFirst + 1;
				
				// Ensure length is in range
				//--------------------------
				if( iCharFirst == 0 )
				{
					int iDocumentLength = document.getLength();
					iLength = Math.min( iLength, iDocumentLength );
				}
				
				document.replace(iCharFirst, iLength, strb.toString());
				document.addDocumentListener(this);

				// Generate new line numbers
				//--------------------------
				ScriptUIEditorModel model = modelElement.getModel();
				model.generateLineNumbers();
			}
			catch (Exception ex) {
			}
		}
	}
}
/**
 */
public ScriptUIEditorModelElement getElementRoot() {
	return _elementRoot;
}
/**
 * Intitializes the given document with the given stream.
 */
protected void setDocumentContent(IDocument document, InputStream contentStream) throws CoreException {

	// Generate an XML model
	//----------------------
	ScriptUIEditorXmlParser parser = new ScriptUIEditorXmlParser();

	try {
		parser.setInputStream( contentStream );
		parser.parse();
	}
	catch (Exception ex) {
	}

	// Generate an XHTML model
	//------------------------

	ScriptUIEditorXmlElement elementXmlRoot = parser.getRootElement();
	ScriptUIEditorModel model = new ScriptUIEditorModel(this);

	if (elementXmlRoot != null) {
		try {
			model.parse(elementXmlRoot);
		}
		catch (Exception ex) {
		}
	}

	// Ensure core elements are in the model
	//--------------------------------------	
	model.makeValid();

	// Generate the document content
	//------------------------------
	_elementRoot = model.getRootElement();
	StringBuffer strbDocument = new StringBuffer(10000);

	_elementRoot.printElementString(strbDocument);

	// Set the document content
	//-------------------------
	document.set(strbDocument.toString());

	document.addDocumentListener(this);
}
/**
 */
public void setEditor(ScriptUIEditor editor) {
	_editor = editor;
}
}
