package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.widgets.TreeItem;

/**
 * Implements drop behaviour for drag and drop operations
 * that land on the resource navigator.
 */
public class ScriptUIEditorTreeViewerDropAdapter extends ViewerDropAdapter {

	protected TreeViewer _treeViewer = null;
	protected ScriptUIEditorContentOutlinePage _outlinePage = null;
	
	protected TransferData currentTransfer = null;
	
	protected long hoverStart = 0;
	protected static final long hoverThreshold = 10000;
/**
 * NavigatorDropAction constructor comment.
 */
public ScriptUIEditorTreeViewerDropAdapter(TreeViewer treeViewer, ScriptUIEditorContentOutlinePage outlinePage) {
	super(treeViewer);

	_treeViewer  = treeViewer;
	_outlinePage = outlinePage;
}
/**
 * The mouse has moved over the drop target.  If the
 * target item has changed, notify the action and check
 * that it is still enabled.
 */
public void dragOver(DropTargetEvent event) {
	try {
		//this method implements the UI behaviour that when the user hovers 
		//over an unexpanded tree item long enough, it will auto-expand.
		Object oldTarget = getCurrentTarget();
		super.dragOver(event);
		if (oldTarget != getCurrentTarget()) {
			hoverStart = System.currentTimeMillis();
		} else {
			//if we've been hovering over this item awhile, expand it.
			if (hoverStart > 0 && (System.currentTimeMillis() - hoverStart) > hoverThreshold) {
				expandSelection((TreeItem) event.item);
				hoverStart = 0;
			}
		}
	} catch (Throwable t) {
		handleException(t, event);
	}
}
/**
 * Expands the selection of the given tree viewer.
 */
protected void expandSelection(TreeItem selection) {
	if (selection == null)
		return;
	if (!selection.getExpanded()) {
		TreeViewer treeViewer = (TreeViewer) getViewer();
		Object[] expanded = treeViewer.getExpandedElements();
		Object[] array = new Object[expanded.length + 1];
		System.arraycopy(expanded, 0, array, 0, expanded.length);
		array[array.length - 1] = selection.getData();
		treeViewer.setExpandedElements(array);
	}
}
/**
 * Invoked when an action occurs. 
 * Argument context is the Window which contains the UI from which this action was fired.
 * This default implementation prints the name of this class and its label.
 * @see IAction#actionPerformed
 */
public boolean performDrop(Object data) {

	if (currentTransfer == null || getCurrentTarget() == null) {
		return false;
	}
	
	if (ScriptUIEditorIntegerTransfer.getInstance().isSupportedType(currentTransfer)) {

		if (getCurrentTarget() instanceof ScriptUIEditorModelElement && data instanceof Integer) {
			final int iHashSource = ((Integer)data).intValue();

			final Object current = getCurrentTarget();
			final boolean bAddAsChild = (getCurrentLocation() == LOCATION_ON);
			final boolean bAddAfter   = (getCurrentLocation() == LOCATION_AFTER);
			
/*			BusyIndicator.showWhile( new Runnable() { 
			public void run() { */
		
			_outlinePage.performDrop( iHashSource, (ScriptUIEditorModelElement)current, bAddAsChild, bAddAfter );
/*						}
		}); */
			return true;
		}        
	}
	
	return false;
}
/**
 * This method is used to notify the action that some aspect of
 * the drop operation has changed.
 */
public boolean validateDrop(Object target, int operation, TransferData transferType) {

	currentTransfer = transferType;
	
	if (ScriptUIEditorIntegerTransfer.getInstance().isSupportedType(currentTransfer)) {

		if (getCurrentLocation() == LOCATION_NONE)
			return false;

		if (getCurrentTarget() instanceof ScriptUIEditorModelElement) {

			// Target
			//-------
			ScriptUIEditorModelElement elementTarget = (ScriptUIEditorModelElement) getCurrentTarget();
			boolean bAddAsChild = (getCurrentLocation() == LOCATION_ON);
			boolean bAddAfter = (getCurrentLocation() == LOCATION_AFTER);

			// Source: No clean way to find out what the source is
			// The tree viewer selection changes as the drag is performed
			//-----------------------------------------------------------
			return true;
/*			
			Object obj = getSelectedObject();
			if (obj instanceof ScriptUIEditorModelElement) {
				ScriptUIEditorModelElement elementSource = (ScriptUIEditorModelElement) obj;
				boolean bValid = ScriptUIEditorModel.isMoveValid(elementSource, elementTarget, bAddAsChild, bAddAfter);
				return bValid;
			}
*/
		}
	}

	return false;
}
}
