package org.eclipse.vcm.internal.core.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.WeakHashMap;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.vcm.internal.core.Handle;
import org.eclipse.vcm.internal.core.IVCMStatus;
import org.eclipse.vcm.internal.core.Policy;
import org.eclipse.vcm.internal.core.ResourceComparator;
import org.eclipse.vcm.internal.core.SharingManager;
import org.eclipse.vcm.internal.core.StateInfo;
import org.eclipse.vcm.internal.core.VCMException;
import org.eclipse.vcm.internal.core.VCMStatus;
import org.eclipse.vcm.internal.core.adapter.ICatchupListener;
import org.eclipse.vcm.internal.core.adapter.IReleaseListener;
import org.eclipse.vcm.internal.core.adapter.IRepositoryAdapter;
import org.eclipse.vcm.internal.core.adapter.IServerResource;
import org.eclipse.vcm.internal.core.adapter.IServerResourceTree;
import org.eclipse.vcm.internal.core.adapter.IServerWorkspace;
import org.eclipse.vcm.internal.core.adapter.IVersionInfo;
import org.eclipse.vcm.internal.core.base.IChangeSet;
import org.eclipse.vcm.internal.core.base.IProjectEdition;
import org.eclipse.vcm.internal.core.base.IRepositoryLocation;
import org.eclipse.vcm.internal.core.base.IResourceEdition;
import org.eclipse.vcm.internal.core.base.ISharingManager;
import org.eclipse.vcm.internal.core.base.IStateInfo;
import org.eclipse.vcm.internal.core.base.ISubscriber;
import org.eclipse.vcm.internal.core.base.ISyncInfo;
import org.eclipse.vcm.internal.core.base.ITeamStream;
import org.eclipse.vcm.internal.core.base.VCMPlugin;
import org.eclipse.vcm.internal.core.ccvs.client.CVSException;
import org.eclipse.vcm.internal.core.ccvs.client.CVSServerException;
import org.eclipse.vcm.internal.core.ccvs.client.Client;
import org.eclipse.vcm.internal.core.ccvs.client.Request;
import org.eclipse.vcm.internal.core.ccvs.requests.Add;
import org.eclipse.vcm.internal.core.ccvs.requests.Checkout;
import org.eclipse.vcm.internal.core.ccvs.requests.Commit;
import org.eclipse.vcm.internal.core.ccvs.requests.Diff;
import org.eclipse.vcm.internal.core.ccvs.requests.History;
import org.eclipse.vcm.internal.core.ccvs.requests.ICheckedInListener;
import org.eclipse.vcm.internal.core.ccvs.requests.IEntryContentIterator;
import org.eclipse.vcm.internal.core.ccvs.requests.IFileContent;
import org.eclipse.vcm.internal.core.ccvs.requests.IHistoryListener;
import org.eclipse.vcm.internal.core.ccvs.requests.ILogListener;
import org.eclipse.vcm.internal.core.ccvs.requests.ITagListener;
import org.eclipse.vcm.internal.core.ccvs.requests.IUpdateMessageListener;
import org.eclipse.vcm.internal.core.ccvs.requests.IUpdatedListener;
import org.eclipse.vcm.internal.core.ccvs.requests.Log;
import org.eclipse.vcm.internal.core.ccvs.requests.RTag;
import org.eclipse.vcm.internal.core.ccvs.requests.Status;
import org.eclipse.vcm.internal.core.ccvs.requests.Tag;
import org.eclipse.vcm.internal.core.ccvs.requests.Update;
import org.eclipse.vcm.internal.core.ccvs.utils.DateUtil;
import org.eclipse.vcm.internal.core.ccvs.utils.DirectoryEntryIterator;
import org.eclipse.vcm.internal.core.ccvs.utils.PathComparator;
import org.eclipse.vcm.internal.core.ccvs.utils.StreamUtil;
import org.eclipse.vcm.internal.core.ccvs.utils.StringPair;

public class CVSAdapter implements IRepositoryAdapter {
	public static final String FOLDER_REVISION = CVSPlugin.getResourceString("na");
	IRepositoryLocation location;
	Map workspaceMap = new WeakHashMap();
	private static IUserAuthenticator authenticator;
	
	// locally remembered remote directories
	// keys: String (project name)
	// values: SortedSet of IPath (directory paths)
	Map knownRemoteDirectories = new HashMap(11);
	
	// a cache for tag information:
	// keys: StringPair (module name, tag name)
	// values: StateInfo objects for that tag
	Map tagInfoCache = null;
	
	// for testing to remember the number of server round-trips
	private int roundTripCnt = 0;
	
public CVSAdapter(IRepositoryLocation location) {
	this.location = location;
}
static void addCVSEntry(IPath tempDirPath, String fileName, String revision, String date, String whatIsThis, String tagName, boolean createEmptyFile) throws IOException {
	File tempDir = new File(tempDirPath.toOSString());
	File tempCVSDir = new File(tempDir, "CVS");
	File entries = new File(tempCVSDir, "Entries");
	FileWriter writer = new FileWriter(entries.getAbsolutePath(), true); // append
	writer.write("/" + fileName + "/" + revision + "/" + date + "/" + whatIsThis + "/" + tagName);
	writer.write('\n');
	writer.close();
	if (createEmptyFile) {
		File emptyFile = new File(tempDir, fileName);
		emptyFile.createNewFile();
	}
}
public static void addCvsIgnoreEntry(IFile cvsIgnoreFile, IResource ignoreResource) throws IOException, CoreException {
	String ignoreResourceName = ignoreResource.getName();
	if (cvsIgnoreFile.exists()) {
		List existingEntries = readCvsIgnoreFile(cvsIgnoreFile.getContents(false));
		if (!existingEntries.contains(ignoreResourceName)) {
			// Append to end of file, add an entry on each line. 
			FileWriter writer = new FileWriter(cvsIgnoreFile.getLocation().toString(), true);
			writer.write(ignoreResourceName + "\n");
			writer.close();
		}
	} else {
		InputStream in = new ByteArrayInputStream(ignoreResourceName.getBytes());
		cvsIgnoreFile.create(in, true, null);
	}
}
private void addKnownRemoteDirectory(String projectName, IPath path) {
	SortedSet remoteDirs = (SortedSet) knownRemoteDirectories.get(projectName);
	boolean newRemoteDirsCreated = false;
	if (remoteDirs == null) {
		remoteDirs = new TreeSet(new PathComparator());
		newRemoteDirsCreated = true;
	}
	// add path and all its parents
	for (int i = 1; i <= path.segmentCount(); i++) {
		remoteDirs.add(path.uptoSegment(i));
	}
	if (newRemoteDirsCreated)
		knownRemoteDirectories.put(projectName, remoteDirs);
}
/**
 * For each new remote directory this operation will recursively query the cvs server for 
 * their contents. The cvs server does not provide a 'deep' update command for new directories 
 * thus this operation is called to perform the 'deep' refresh. Remote directories are
 * remembered so that the next refresh operation will include these known new directories.
 * 
 * The deep refresh is performed in a single round trip. The caller is responsible for creating
 * and disconnecting the connection.
 * 
 * Pruning of empty directories is performed by this operation. The resource tree builder is
 * not told about empty directories.
 */
private void addNewDirectoriesContents(Client client, Context context, List newRemoteDirectories, CVSResourceTreeBuilder builder, IProgressMonitor monitor) throws VCMException {
	for (Iterator it = newRemoteDirectories.iterator(); it.hasNext();) {
		IPath newDirPath = (IPath) it.next();

		IServerResource[] members = context.internalFetchMembers(client, newDirPath, monitor);

		boolean addThisDirectory = false;

		List memberDirs = new ArrayList(10);
		for (int i = 0; i < members.length; i++) {
			IPath memberPath = newDirPath.append(members[i].getName());
			if (members[i].getType() == IResource.FILE) {
				builder.fileStatus(memberPath, ((StateId) members[i].getStateId()).getId());
				addThisDirectory = true;
			} else {
				memberDirs.add(memberPath);
			}
		}

		if (addThisDirectory) {
			addKnownRemoteDirectory(context.getPathPrefix().lastSegment(), newDirPath);
		}

		// recurse:
		addNewDirectoriesContents(client, context, memberDirs, builder, monitor);
	}
}
public void addProjectFromVersion(String teamStreamName, String projectName, String projectVersionName, boolean replace, IProgressMonitor monitor) throws VCMException {
	// In cvs all projects are in all branches so we can't check to see if the
	// project is already in the team stream.
	monitor = Policy.monitorFor(monitor);
	String[] args = null;

	// In cvs you can replace a branch with another by moving the branch tag
	// from one revision to the next. The revisions on the initial branch are
	// still there but the branch tag has moved. The -F flag does the move
	// if the branch tag already exists.
	if (replace)
		args = new String[] { "-b", "-F" };
	else
		args = new String[] { "-b" };

	Request rtag = new RTag(projectName, projectVersionName, teamStreamName, args);
	Client client = new Client(this, monitor);
	try {
		client.execute(rtag);
	} finally {
		client.disconnect();
	}
}
private void addTagInfoEntry(String moduleName, String tag, String author, Date creationDate) {
	if(tagInfoCache==null)
		tagInfoCache = new HashMap();
	tagInfoCache.put(new StringPair(moduleName, tag), new StateInfo(new StateId(FOLDER_REVISION), creationDate, author, CVSPlugin.getResourceString("na"), tag, ""));
}
protected void checkOK(IStatus status) throws CVSException {
	if (!status.isOK())
		throw new CVSException(status);
}
// key type: IFolder, value type: List of IResource (IFile or IFolder, not IProject)
Map computeParentSlices(List resources) {
	Map result = new HashMap(3);
	for (Iterator it = resources.iterator(); it.hasNext();) {
		IResource resource = (IResource) it.next();
		List resourcesInDir = (List) result.get(resource.getParent());
		if (resourcesInDir == null) {
			resourcesInDir = new ArrayList(3);
			result.put(resource.getParent(), resourcesInDir);
		}
		resourcesInDir.add(resource);
	}
	return result;
}
// key type: IProject, value type: SortedSet of IResource
Map computeProjectSlices(IResource[] resources) throws CoreException {
	Map result = new HashMap(3);
	for (int i = 0; i < resources.length; i++) {
		SortedSet projectResources = (SortedSet) result.get(resources[i].getProject());
		if (projectResources == null) {
			projectResources = new TreeSet(new ResourceComparator());
			result.put(resources[i].getProject(), projectResources);
		}
		projectResources.add(resources[i]);
	}
	return result;
}
void createCVSFile(File dir, String fileName, String fileContent) throws IOException {
	File file = new File(dir, fileName);
	PrintWriter fileWriter = new PrintWriter(new FileWriter(file));
	fileWriter.println(fileContent);
	fileWriter.close();
}
public static void createCVSSubdir(File parentDir, String repositoryRoot, IPath repositoryRelativePath) throws IOException {
	File cvsDir = new File(parentDir, "CVS");
	cvsDir.mkdir();
	File entries = new File(cvsDir, "Entries");
	entries.createNewFile();
	File root = new File(cvsDir, "Root");
	FileWriter writer = new FileWriter(root);
	writer.write(repositoryRoot);
	writer.close();
	File repository = new File(cvsDir, "Repository");
	writer = new FileWriter(repository);
	writer.write(repositoryRelativePath.toString());
	writer.close();
}
public IStatus createProjectVersion(SortedSet resources, String versionName, IProjectEdition[] referencedProjectVersions, IProgressMonitor monitor) throws CoreException {
	
	final IProgressMonitor m = Policy.monitorFor(monitor);
	IResource resource = (IResource)resources.first();
	if( resource != null ) {
	final SharingManager sharingManager = (SharingManager) VCMPlugin.getProvider().getSharingManager();
	
	// First use tag to ensure that revisions from the local workspace are 
	// tagged and not the teamstream resources.
	String projectName = resource.getProject().getName();
	Request tag = new Tag(	new Path(projectName), 
							versionName, 
							new SyncInfoEntryIterator(resources, sharingManager, versionName),
							new ITagListener() {
								public void tagged(IPath file){
									m.subTask(CVSPlugin.getResourceString("CVSAdapter.tagging", new Object[] {file.toString()} ));
									m.worked(1);							
								}} );
								
	// Last chance to change your mind.
	if (m.isCanceled())
		throw new OperationCanceledException();

	Client client = new Client(this,m);
	try {
		client.execute(tag);
	} finally {
		client.disconnect();
	}
	
	// Then rtag with the same tag name to get the history entry.
	Request rtag = new RTag(projectName, versionName, versionName, null);
	client = new Client(this, m);
	try {
		client.execute(rtag);
	} finally {
		client.disconnect();
	}
	
	}
	return new CVSStatus(CVSStatus.OK, "ok");
}
public void createTeamStream(String name, IProgressMonitor monitor) throws VCMException {
	// cvs plugin remembers (and persists) team stream names
	String[] names = CVSPlugin.getInstance().getTeamStreamNames(getRepositoryRoot());
	if (monitor.isCanceled())
		throw new OperationCanceledException();
	if (!Util.arrayIncludes(names, name)) {
		CVSPlugin.getInstance().addTeamStream(getRepositoryRoot(), name);
	}
}
public IServerWorkspace createTeamStreamProjectVersion(IServerWorkspace teamStreamProjectWorkspace, String versionName, IResourceEdition[] referencedProjectEditions, IProgressMonitor monitor) throws VCMException {
	Context ctx = (Context) teamStreamProjectWorkspace;

	// Should we only allow tags on top level modules, or on seperate directories?
	Request rtag = new RTag(ctx.getPathPrefix().lastSegment(), ctx.getTagName(), versionName, null);
	Client client = new Client(this, monitor);
	try {
		client.execute(rtag);
	} finally {
		client.disconnect();
	}
	return new Context(this, ctx.getPathPrefix(), versionName);
}
public void deleteTeamStream(String name, IProgressMonitor monitor) throws CVSException {
	// cvs plugin remembers (and persists) team stream names
	CVSPlugin.getInstance().removeTeamStream(getRepositoryRoot(), name);
}
public void deleteTeamStreamProject(String teamStreamName, String projectName, IProgressMonitor monitor) throws CVSException {
	throw new CVSException(CVSStatus.ERROR, new Path(projectName), CVSPlugin.getResourceString("CVSAdapter.deleteProjects"), null);
}
public void diffReport(IProject project, String teamStreamName, IProgressMonitor monitor) throws CoreException {
	SharingManager sharingManager = (SharingManager) VCMPlugin.getProvider().getSharingManager();

	IProgressMonitor progressMonitor = Policy.monitorFor(monitor);
	progressMonitor.beginTask("Getting differences with server resources", 100);
	try {
		final SortedSet resources = new TreeSet(new ResourceComparator());
		project.accept(new IResourceVisitor() {
			public boolean visit(IResource resource) {
				resources.add(resource);
				return true;
			}
		}, IResource.DEPTH_INFINITE, true);

		progressMonitor.worked(20);

		Client client = new Client(this, Policy.subMonitorFor(monitor, 80));
		Request diffRequest = new Diff(project.getFullPath().makeRelative(), teamStreamName, System.out, new DiffEntryIterator(resources, sharingManager, teamStreamName));
		try {
			client.execute(diffRequest);
		} finally {
			client.disconnect();
		}
	} finally {
		progressMonitor.done();
	}
}
public IVersionInfo[] fetchContainerHistory(final String projectName, final IPath projectRelativePath, IProgressMonitor monitor) throws VCMException {
	final List versionInfos = new ArrayList();
	
	Client client = new Client(this, monitor);

	final String[] teamStreamNames = CVSPlugin.getInstance().getTeamStreamNames(getRepositoryRoot());

	Request request = new History(projectName, null, new IHistoryListener() {
		public void historyLine(String moduleName, String tag, String author, Date creationDate) {
			// team stream names might be referenced as "rtag" tags in the history file, so ignore them
			if (!Util.arrayIncludes(teamStreamNames, tag)) {
				addTagInfoEntry(moduleName, tag, author, creationDate);
				Context context = new Context(CVSAdapter.this, new Path(projectName), tag);
				CVSResource resource;
				if(projectRelativePath.isEmpty()) {
					resource = new CVSModule(context, FOLDER_REVISION);
				} else {
					resource = new CVSResource(context, projectRelativePath, IResource.FOLDER, FOLDER_REVISION);
				}
				StateInfo stateInfo = new StateInfo(new StateId(FOLDER_REVISION), creationDate, author, CVSPlugin.getResourceString("na"), tag, "");
				versionInfos.add(new VersionInfo(resource, context, stateInfo, Collections.EMPTY_LIST));
			}
		}
	});
	try {
		client.execute(request);
	} finally {
		client.disconnect();
	}

	if(versionInfos.size()>0 && !projectRelativePath.isEmpty()) {
		// make sure that the directory exists
		Context c = new Context(this, new Path(projectName), "HEAD");
		IServerResource testResource = c.fetchResourceAt(projectRelativePath, monitor);
		if(testResource==null)
			return new IVersionInfo[0];
	}
	
	return (IVersionInfo[])versionInfos.toArray(new IVersionInfo[versionInfos.size()]);
}
public IVersionInfo[] fetchFileHistory(final String projectName, final IPath path, final IServerResource resource, final IProgressMonitor monitor) throws VCMException {

	// keys: String (revision); value: CVSResource
	final Map revisionResources = new HashMap(13);

	// keys: CVSResource, values: Collection of CVSResource
	final Map predecessorMap = new HashMap(13);

	// to calculate predecessors
	final CVSResource[] lastVersion = new CVSResource[] { null };
	final StateInfo[] lastInfo = new StateInfo[] { null };
	
	final List versions = new ArrayList(11);

	IPath projectPath = new Path(projectName);
	Request log = new Log(projectPath.append(path.removeLastSegments(1)), new Path(path.lastSegment()), null, new ILogListener() {
		// Assume that this operation will be called with revisions sorted by revision and by branch		
		public void log(String revision, String author, String date, String comment, String state, String[] tags) {

			// If the equest was canceled don't perform any further work.
			if (monitor.isCanceled())
				return;

			// if this revision does not exist, don't add it to the versions.
			if (state.equals("dead")) {
				if(lastVersion[0]!=null) {
					versions.add(new VersionInfo(lastVersion[0], lastInfo[0], Collections.EMPTY_LIST));
					lastVersion[0] = null;
					lastInfo[0] = null;
				}
				return;
			}

			CVSResource thisVersion = new CVSResource(new Context(CVSAdapter.this, new Path(projectName), revision), path, IResource.FILE, revision);
			StateInfo thisInfo = new StateInfo(new StateId(revision), DateUtil.convertFromLogTime(date), author, comment, revision, CVSResource.tagsToString(tags));
			
			if(lastVersion[0]!=null) {
				if(sameBranch(lastVersion[0], thisVersion)) {
					versions.add(new VersionInfo(lastVersion[0], lastInfo[0], Collections.singleton(thisVersion)));
				} else {
					// the current version is on another branch.
					// set the predecessor of the last version we saw
					String lastRevision = lastVersion[0].stateId;
					if(lastRevision.indexOf(".")==lastRevision.lastIndexOf('.')) {
						versions.add(new VersionInfo(lastVersion[0], lastInfo[0], Collections.EMPTY_LIST));
					} else {
						// this is not the initial revision, so the predecessor can be
						// determined by removing the last two numbers
						String predecessorRevision = lastRevision.substring(0, lastRevision.lastIndexOf('.'));
						predecessorRevision = predecessorRevision.substring(0, predecessorRevision.lastIndexOf('.'));
						versions.add(new VersionInfo(lastVersion[0], lastInfo[0], Collections.singleton(revisionResources.get(predecessorRevision))));
					}
				}
			}
			lastVersion[0] = thisVersion;
			lastInfo[0] = thisInfo;

			revisionResources.put(revision, thisVersion);
		}
	});

	Client client = new Client(this, null);
	try {
		client.execute(log);
	} finally {
		client.disconnect();
	}

	// The operation was canceled, so we shall exit safely here.
	if (monitor.isCanceled())
		throw new OperationCanceledException();

	// set the predecessor of the last version we saw.
	// this code is copied from above - sorry for that...
	if(lastVersion[0]!=null) {
		String lastRevision = lastVersion[0].stateId;
		if(lastRevision.indexOf(".")==lastRevision.lastIndexOf('.')) {
			versions.add(new VersionInfo(lastVersion[0], lastInfo[0], Collections.EMPTY_LIST));
		} else {
			// this is not the initial revision, so the predecessor can be
			// determined by removing the last two numbers
			String predecessorRevision = lastRevision.substring(0, lastRevision.lastIndexOf('.'));
			predecessorRevision = predecessorRevision.substring(0, predecessorRevision.lastIndexOf('.'));
			versions.add(new VersionInfo(lastVersion[0], lastInfo[0], Collections.singleton(revisionResources.get(predecessorRevision))));
		}
	}


	return (IVersionInfo[])versions.toArray(new IVersionInfo[versions.size()]);
}
public String[] fetchProjectNames(IProgressMonitor monitor) throws VCMException {
	List topLevelModules = Arrays.asList(internalFetchMemberNames(Path.EMPTY, "HEAD", monitor)[0]);
	List result = new ArrayList();
	result.addAll(topLevelModules);
	result.remove("CVSROOT");
	return (String[]) result.toArray(new String[result.size()]);
}
public IServerResourceTree fetchRemoteTree(IServerWorkspace workspace, IPath path, int depth, final IProgressMonitor monitor) throws CoreException, VCMException {
	Context context = ((Context) workspace);

	// the path prefix always has one segment
	final String projectName = context.getPathPrefix().lastSegment();

	// tentatively add the parent path as a known remote directory.
	// if this directory does not exist, we will be told
	if (path.segmentCount() > 1)
		addKnownRemoteDirectory(projectName, path.removeLastSegments(1));

	// We can abandon the operation early here..
	if (monitor.isCanceled())
		throw new OperationCanceledException();
		
	IPath[] previouslyKnownRemoteDirectories = getKnownRemoteDirectories(projectName, true);

	if (monitor.isCanceled())
		throw new OperationCanceledException();
		
	IPath fullPath = context.getPathPrefix().append(path);
	Client client = new Client(this, monitor);

	String recurseArg = (depth == IResource.DEPTH_ZERO ? "-l" : "-d");

	final List newRemoteDirectories = new ArrayList();
	final List remoteFiles = new ArrayList();

	Request request = new Update(Path.EMPTY, fullPath, new String[] { recurseArg }, ((Context) workspace).getTagName(), true, new DirectoryEntryIterator(previouslyKnownRemoteDirectories), new IUpdateMessageListener() {
		IPath currentDir = null;
		public void directoryInformation(IPath path, boolean newDirectory) {
			if( validatePath(path) ) {
				if (newDirectory)
					newRemoteDirectories.add(path.removeFirstSegments(1));
				currentDir = path;
			} else {
				Util.logError(CVSPlugin.getResourceString("CVSAdapter.badPath", new Object[] {path.toOSString()}), null);
			}
		}
		public void directoryDoesNotExist(IPath path) {
			if( validatePath(path) ) {
				removeKnownRemoteDirectory(projectName, path.removeFirstSegments(1));
			} else {
				Util.logError(CVSPlugin.getResourceString("CVSAdapter.badPath", new Object[] {path.toOSString()}), null);
			}
		}
		public void fileInformation(char type, String filename) {
			if( validatePath(filename)) {
				IPath filePath = new Path(filename);
				remoteFiles.add(filePath);
				monitor.subTask(filePath.lastSegment().toString());
				monitor.worked(1);
			} else {
				Util.logError(CVSPlugin.getResourceString("CVSAdapter.badPath", new Object[] {filename}), null);
			}
		}
	}, null);
	try {
		client.execute(request);
	} catch(CVSServerException e) {
		// XXX special case handling so that it is possible to release a new project to a stream.
		// (otherwise, the refresh/fetchRemoteTree operation preceding the release will fail.)
		// This is a cvs oddity because if the stream tag is not yet used to tag any other project,
		// the server will report an error ("no such tag"). If the stream tag is already used to tag files
		// of another project, no error will be reported. Since you can release a new project to a
		// stream even if the stream tag is not yet used anywhere, we filter out these errors here.
		if( !isNoTagException(e) ) {
			throw e;
		}
	} finally {
		client.disconnect();
	}

	// We can abandon the operation early here..
	if (monitor.isCanceled())
		throw new OperationCanceledException();

	client = new Client(this, monitor);

	CVSResourceTreeBuilder builder = new CVSResourceTreeBuilder((Context) workspace, path);

	// If we didn't find anything on the server with the update, don't bother running the status command.
	if (!remoteFiles.isEmpty()) {
		request = new Status(getKnownRemoteDirectories(projectName, true), (IPath[]) remoteFiles.toArray(new IPath[0]), null, context.getTagName(), null, builder);
		try {
			client.execute(request);
		} finally {
			client.disconnect();
		}
	}

	if (monitor.isCanceled())
		throw new OperationCanceledException();

	client = new Client(this, monitor);
	try {
		addNewDirectoriesContents(client, context, newRemoteDirectories, builder, monitor);
	} finally {
		client.disconnect();
	}

	return builder.getBuiltTree();
}
public String[] fetchTeamStreamNames(IProgressMonitor monitor) throws CVSException {
	// cvs plugin remembers (and persists) team stream names
	String[] names = CVSPlugin.getInstance().getTeamStreamNames(getRepositoryRoot());
	List result = new ArrayList(names.length + 1);
	result.add("HEAD");
	result.addAll(Arrays.asList(names));
	return (String[]) result.toArray(new String[result.size()]);
}
/**
 * Returns the project names in the given stream.
 */
public String[] fetchTeamStreamProjectNames(String teamStreamName, IProgressMonitor monitor) throws VCMException {
	List topLevelModules = Arrays.asList(internalFetchMemberNames(Path.EMPTY, teamStreamName, monitor)[0]);	
	List result = new ArrayList();
	result.addAll(topLevelModules);
	result.remove("CVSROOT");	
	return (String[]) result.toArray(new String[result.size()]);
}
public static IUserAuthenticator getAuthenticator() {
	if (authenticator == null) {
		authenticator = getPluggedInAuthenticator();
	}
	return authenticator;
}
/**
 * To see if a resource is ignored, look for a file called .cvsignore in it's parent 
 * directory that has an entry with the same name as the resources name.
 *
 * The cvsignore file is a whitespace-separated list of file names and wildcard patterns
 * that CVS should ignore. (See also the -I option to the import command.) Therefore technically, 
 * you can name multiple files or patterns on each line, separated by whitespace, but it 
 * is more readable to keep them to one per line. The whitespace separation rule does, 
 * unfortunately, mean that there's no way to specify a space in a file name, except to use 
 * wildcards. 
 *
 * This variable is appended last to the ignore list during a command. The list is built 
 * up in this order: CVSROOT/cvsignore, the .cvsignore file in your home directory, the 
 * $CVSIGNORE variable, any -I command option, and finally the contents of
 * .cvsignore files in the working copy used as CVS works in each directory. A ! as the
 * ignore specification at any point nullifies the entire ignore list built up to that 
 * point. 
 */
public boolean[] getIgnored(IResource[] resources) throws CoreException {
	boolean[] ignoreResult = new boolean[resources.length];
	Map ignoreMatchers = new HashMap(3);
	
	IPath cvsIgnorePath = new Path("");
	try {
		for (int i = 0; i < resources.length; i++) {
			if (resources[i].getType() == IResource.PROJECT) {
				ignoreResult[i] = false;
				continue;
			}
			IContainer parentResource = resources[i].getParent();
			IFile cvsIgnoreFile = (IFile) parentResource.getFile(new Path(".cvsignore"));
			cvsIgnorePath = cvsIgnoreFile.getLocation();
			if (cvsIgnoreFile != null && cvsIgnoreFile.exists()) {
				// cvsignore exists, if first time visiting parse it
				List matchers;
				if (!ignoreMatchers.containsKey(cvsIgnorePath)) {
					List entries = readCvsIgnoreFile(cvsIgnoreFile.getContents(false));
					// create a string matcher for each entry
					matchers = new ArrayList();
					Iterator iterator = entries.iterator();
					while (iterator.hasNext()) {
						String entry = (String)iterator.next();
						// Don't include empty strings
						if( entry.length() > 0 )
							matchers.add(new StringMatcher(entry, false, false));
					}
					ignoreMatchers.put(cvsIgnorePath, matchers);
				} else {
					matchers = (List) ignoreMatchers.get(cvsIgnorePath);
				}
				Iterator iterator = matchers.iterator();
				while (iterator.hasNext()) {
					if (((StringMatcher)iterator.next()).match(resources[i].getName())) {
						ignoreResult[i] = true;
						break;
					}
				}
			} else {
				// cvsignore does not exist, don't ignore this resource
				ignoreResult[i] = false;
			}
		}
		return ignoreResult;
	} catch (IOException ex) {
		throw new CVSException(ICVSStatus.ERROR, cvsIgnorePath, CVSPlugin.getResourceString("CVSAdapter.cvsignore"), ex);
	}
}
private IPath[] getKnownRemoteDirectories(String projectName, boolean returnFullPaths) {
	SortedSet remoteDirs = (SortedSet) knownRemoteDirectories.get(projectName);
	// revisit this hack to add project dir
	if (remoteDirs == null) {
		try {
			IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
			if(!project.exists() || VCMPlugin.getProvider().getSharingManager().getSharing(project)==null)
				return new IPath[] { new Path(projectName)};
			// the project exists locally and is managed, initialize the remote dirs
			// based on the sync infos.
			final SortedSet newRemoteDirs = new TreeSet(new PathComparator());
			final ISharingManager sharingManager = VCMPlugin.getProvider().getSharingManager();
			try {
				project.accept(new IResourceVisitor() {
					public boolean visit(IResource resource) throws CoreException {
						if(resource.getType()!=IResource.FILE) {
							ISyncInfo syncInfo = sharingManager.getSyncInfo(resource);
							if(syncInfo!=null && syncInfo.getRemote()!=null) {
								// the resource did exist on the server...
								if(resource.getType()!=IResource.PROJECT)
									newRemoteDirs.add(resource.getProjectRelativePath());
								return true;
							}
						}
						return false;
					}
				 }, IResource.DEPTH_INFINITE, true);
			} catch(CoreException ex) {
				// at least we tried to get the remote dirs from the workspace.
				// eat this exception and fall back to the "do one roundtrip per
				// new remote directory behaviour"
				return new IPath[] { new Path(projectName)};
			}
			
			// return with the project name if no directories have
			// remote version yet. This can happen when this is called
			// before a project is released.
			if( newRemoteDirs.isEmpty() )
				return new IPath[] { new Path(projectName) };
			
			knownRemoteDirectories.put(projectName, newRemoteDirs);
			remoteDirs = newRemoteDirs;
		} catch(RuntimeException ex) {
			CVSPlugin.log(new CVSStatus(ICVSStatus.ERROR, new Path(projectName), CVSPlugin.getResourceString("CVSAdapter.internalRecoverableError"), ex));
			return new IPath[] { new Path(projectName)};
		}
	}
	if (returnFullPaths) {
		List result = new ArrayList(remoteDirs.size() + 1);
		result.add(new Path(projectName));
		for (Iterator it = remoteDirs.iterator(); it.hasNext();) {
			IPath path = (IPath) it.next();
			result.add(new Path(projectName).append(path));
		}
		return (IPath[]) result.toArray(new IPath[result.size()]);
	} else {
		return (IPath[]) remoteDirs.toArray(new IPath[0]);
	}
}
public IRepositoryLocation getLocation() {
	return location;
}
private static IUserAuthenticator getPluggedInAuthenticator() {
	IExtension[] extensions = Platform.getPluginRegistry().getExtensionPoint(CVSPlugin.PI_CVS, CVSPlugin.PT_AUTHENTICATOR).getExtensions();
	if (extensions.length == 0)
		return null;
	IExtension extension = extensions[0];
	IConfigurationElement[] configs = extension.getConfigurationElements();
	if (configs.length == 0) {
		CVSPlugin.log(new CVSStatus(ICVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapter.noConfigurationElement", new Object[] {extension.getUniqueIdentifier()}), null));
		return null;
	}
	try {
		IConfigurationElement config = configs[0];
		return (IUserAuthenticator) config.createExecutableExtension("run");
	} catch (CoreException ex) {
		CVSPlugin.log(new CVSStatus(ICVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapter.unableToInstantiate", new Object[] {extension.getUniqueIdentifier()}), ex));
		return null;
	}
}
public IServerWorkspace getProjectVersion(String projectName, String versionName) throws CVSException {
	return new Context(this, new Path(projectName), versionName);
}
String getRepositoryRoot() {
	// assuming location strings of the form
	// :pserver:user@host:/path/to/repository/root
	// move this code to IRepositoryAdapterFactory: checkOK(validateRepositoryLocation(location.getLocation()));
	return location.getLocation();
}
IStateInfo getTagInfoEntry(String moduleName, String tag, IProgressMonitor monitor) throws VCMException {
	if(tagInfoCache==null)
		tagInfoCache = new HashMap();
	IStateInfo result = (IStateInfo) tagInfoCache.get(new StringPair(moduleName, tag));
	if (result==null) {
		// try to fill the cache
		fetchContainerHistory(moduleName, Path.EMPTY, monitor);
		result = (IStateInfo) tagInfoCache.get(new StringPair(moduleName, tag));
	}
	return result;
}
public IServerWorkspace getTeamStreamProject(String teamStreamName, String projectName) throws CVSException {
	return new Context(this, new Path(projectName), teamStreamName);
}
public void incrementRoundTripCount() {
	++roundTripCnt;
}
/**
 * Returns the remote members at the give remote path. This operation creates its own cvs 
 * connection to query the server.
 */
public String[][] internalFetchMemberNames(IPath repositoryRelativePath, String tagName, final IProgressMonitor monitor) throws VCMException {
	Client client = new Client(this, monitor);
	try {
		return internalFetchMemberNames(client, repositoryRelativePath, tagName, monitor);
	} finally {
		client.disconnect();
	}
}
/**
 * Returns the names of remote members at the given remote path. This operation can be run within one 
 * cvs connection. The caller is responsible for connecting and disconnecting the client.
 * 
 * If called several times within the same cvs connection, the update command 
 * will show folders from other directories updated in the same command. These are filtered 
 * out by only considering the folders and files found in the repositoryRelativePath.
 */
public String[][] internalFetchMemberNames(Client client, IPath repositoryRelativePath, String tagName, final IProgressMonitor monitor) throws VCMException {
	final List newRemoteDirectories = new ArrayList();
	final List newRemoteFiles = new ArrayList();
	Request request = new Update(repositoryRelativePath, Path.EMPTY, new String[] { "-d" }, tagName, true, null, new IUpdateMessageListener() {
		public void directoryInformation(IPath path, boolean newDirectory) {
			if( validatePath(path) ) {
				// only consider directory if relative to the repositoryRelativePath
				if (newDirectory && path.segmentCount() == 1)
					newRemoteDirectories.add(path.lastSegment());
				monitor.subTask(path.lastSegment().toString());
				monitor.worked(1);
			} else {
				Util.logError(CVSPlugin.getResourceString("CVSAdapter.badPath", new Object[] {path.toOSString()}), null);
			}
		}
		public void directoryDoesNotExist(IPath path) {
			monitor.worked(1);
		}
		public void fileInformation(char type, String filename) {
			if( validatePath(filename) ) {
				IPath filePath = new Path(filename);	
				// only consider file if relative to the repositoryRelativePath
				if( filePath.segmentCount() == 1 ) {
					String properFilename = filePath.lastSegment();
					newRemoteFiles.add(properFilename);
					monitor.subTask(properFilename);
					monitor.worked(1);
				}
			} else {
				Util.logError(CVSPlugin.getResourceString("CVSAdapter.badPath", new Object[] {filename}), null);
			}
		}
	}, null);
	try {
		client.execute(request);
	} catch (CVSServerException ex) {
		if( ! isNoTagException(ex) )
			throw ex;
		// we now know that this is an exception caused by a cvs bug.
		// if the folder has no files in it (just subfolders) cvs does not respond with the subfolders...
		// workaround: retry the request with no tag to get the directory names (if any)
		if (monitor.isCanceled()) // Can exit if the answer is not required.
			throw new OperationCanceledException();
		return internalFetchMemberNames(client, repositoryRelativePath, "HEAD", monitor);
	}
	return new String[][] {(String[]) newRemoteDirectories.toArray(new String[0]), (String[]) newRemoteFiles.toArray(new String[0])};
}
public void internalLoadShallow(ISubscriber subscriber, IServerWorkspace serverWorkspace, SortedSet resources, final ICatchupListener catchupListener, final IProgressMonitor monitor) throws CoreException {
	if (resources.isEmpty())
		return;
	final IProject project = ((IResource) resources.first()).getProject();
	final Context context = (Context) serverWorkspace;

	// a list of strings (to be passed as arguments to Update below)
	// with the project-relative paths of all files we would like to get
	List allFilePaths = new ArrayList(resources.size());
	for (Iterator it = resources.iterator(); it.hasNext();) {
		IResource resource = (IResource) it.next();
		if (resource.getType() == IResource.FILE)
			allFilePaths.add(resource.getProjectRelativePath().toString());
		else {
			// CVS won't tell us about updated directories thus we have to callback
			// to the listener about folders and projects so that the sync information
			// can be correctly updated for these resource types.
			CVSResource serverResource = null;
			if( resource.getType() == IResource.FOLDER ) {
				serverResource = new CVSResource(context, resource.getProjectRelativePath(), IResource.FOLDER, CVSAdapter.FOLDER_REVISION);
			} else {
				serverResource = new CVSModule(context, CVSAdapter.FOLDER_REVISION);
			}
			catchupListener.resourceCaughtup(resource, serverResource);			
		}		
	}

	final List errors = new ArrayList(5);
	Client client = new Client(this, monitor);
	String[] arguments = (String[]) allFilePaths.toArray(new String[allFilePaths.size()]);
	CatchupEntryIterator entryIterator = new CatchupEntryIterator(resources, subscriber);

	// this listener does the actual work of getting the content and creating the files.
	// it is wrapped here because of the calls to the catchup listener.
	final UpdatedListener updatedListener = new UpdatedListener(project.getLocation(), project.getFullPath(), null, errors, monitor);

	Request request = new Update(project.getFullPath(), null, arguments, context.getTagName(), false, entryIterator, null, new IUpdatedListener() {
		public void fileInformation(IPath updateContextRelativePath, String entryLine, InputStream in, int size) {

			int secondSlashIndex = entryLine.indexOf('/', 1);
			String fileName = entryLine.substring(1, secondSlashIndex);
			String revision = entryLine.substring(secondSlashIndex + 1, entryLine.indexOf('/', secondSlashIndex + 1));

			IFile file = project.getFile(updateContextRelativePath.append(fileName));

			// Make dirs if needed
			IContainer parent = file.getParent();
			try {
				mkdirs(parent, context, catchupListener);
			} catch (CoreException ex) {
				errors.add(new CVSStatus(ICVSStatus.ERROR, file.getLocation(), CVSPlugin.getResourceString("CVSAdapter.createParents"), ex));
			}

			// call the wrapped listener to actually load the file
			updatedListener.fileInformation(updateContextRelativePath, entryLine, in, size);

			try {
				file.refreshLocal(IResource.DEPTH_INFINITE, null);
			} catch (CoreException ex) {
				errors.add(new CVSStatus(ICVSStatus.ERROR, file.getLocation(), CVSPlugin.getResourceString("CVSAdapter.localResource"), ex));
			}

			// call the catchup listener:

			CVSResource serverFile = new CVSResource(context, file.getProjectRelativePath(), IResource.FILE, revision);

			try {
				catchupListener.resourceCaughtup(file, serverFile);
			} catch (CoreException ex) {
				errors.add(new CVSStatus(ICVSStatus.ERROR, file.getLocation(), CVSPlugin.getResourceString("CVSAdapter.syncInfo"), ex));
			}
		}
	});
	
	// Here's a chance to avoid the work.
	try {
		if (!monitor.isCanceled())
			client.execute(request);
	} finally {
		client.disconnect();
	}

	// If the operation was canceled, throw the exception.
	if (monitor.isCanceled())
		throw new OperationCanceledException();

	if (errors.size() > 0) {
		MultiStatus multi = new MultiStatus(VCMPlugin.PI_VCM, IVCMStatus.SERVER_ERROR, (IStatus[]) errors.toArray(new IStatus[errors.size()]), CVSPlugin.getResourceString("CVSAdapter.ioLoad"), null);
		throw new CVSException(multi);
	}
}
public void internalRelease(IServerWorkspace serverWorkspace, SortedSet resources, boolean onlyIfOutgoing, String releaseComment, final IReleaseListener releaseListener, IProgressMonitor monitor) throws CoreException {
	SharingManager sharingManager = (SharingManager) VCMPlugin.getProvider().getSharingManager();
	final IProject project = ((IResource) resources.first()).getProject();
	final Context context = (Context) serverWorkspace;
	List newDirectories = new ArrayList();
	List parents = new ArrayList();
	List parentPaths = new ArrayList();

	for (Iterator it = resources.iterator(); it.hasNext();) {
		IResource resource = (IResource) it.next();
		if (resource.getType() != IResource.FILE) {
			ISyncInfo syncInfo = sharingManager.getSyncInfo(resource);
			
			// Create folders on server if we have local creations or the user releases is workspace folders 
			// to overide incoming deletions.
			if (syncInfo.getOutgoingChange() == ISyncInfo.CREATE || (!onlyIfOutgoing && syncInfo.getIncomingChange() == ISyncInfo.DELETE) ) {
				newDirectories.add(resource.getFullPath().makeRelative());
				parentPaths.add(resource.getFullPath().makeRelative());
			}
		}
		if (resource.getType() != IResource.PROJECT) {
			parents.add(resource.getParent());
			IPath parentPath = resource.getParent().getFullPath().makeRelative();
			if (!parentPaths.contains(parentPath))
				parentPaths.add(resource.getParent().getFullPath().makeRelative());
		}
	}
	
	// No real work done yet, so if the request was canceled it is safe to exit the task.
	if (monitor.isCanceled())
		throw new OperationCanceledException();
		
	// Create directories.	
	if (newDirectories.size() > 0) {
		IPath[] newDirectoryPaths = (IPath[]) newDirectories.toArray(new IPath[newDirectories.size()]);
		IPath[] parentPathArray = (IPath[]) parentPaths.toArray(new IPath[parentPaths.size()]);
		Request addRequest = new Add(Path.EMPTY, new DirectoryEntryIterator(parentPathArray), newDirectoryPaths);
		Client client = new Client(this, monitor);
		try {
			client.execute(addRequest);
			for(Iterator it=newDirectories.iterator(); it.hasNext();) {
				IPath dirPath = (IPath) it.next();
				IResource newDir = ResourcesPlugin.getWorkspace().getRoot().findMember(dirPath);
				IServerResource newServerDir = new CVSResource(context, dirPath.removeFirstSegments(1), IResource.FOLDER, FOLDER_REVISION);
				releaseListener.resourceReleased(newDir, newServerDir);
			}
		} finally {
			client.disconnect();
		}
	}
	
	// Release the directories that were created.	
	for (Iterator it = newDirectories.iterator(); it.hasNext();) {
		IPath projectRelativeNewDir = ((IPath) it.next()).removeFirstSegments(1);
		IResource releasedDir;
		CVSResource serverDir;
		if (projectRelativeNewDir.isEmpty()) {
			releasedDir = project;
			serverDir = new CVSModule(context, CVSAdapter.FOLDER_REVISION);
		} else {
			releasedDir = project.findMember(projectRelativeNewDir, true);
			serverDir = new CVSResource(context, projectRelativeNewDir, IResource.FOLDER, CVSAdapter.FOLDER_REVISION);
		}
		releaseListener.resourceReleased(releasedDir, serverDir);
		addKnownRemoteDirectory(context.getPathPrefix().lastSegment(), releasedDir.getProjectRelativePath());
	}

	// If we exit here we would leave harmless empty directories created.
	if (monitor.isCanceled())
		throw new OperationCanceledException();

	// rememeber what resources should be release, resources that are released but
	// are identical to the remote revision will not get a message back from cvs server.
	final List resourcesToRelease = new ArrayList(resources);
	// add all parents to the resources list so that the entry iterator includes all directories:
	resources.addAll(parents);

	final boolean[] internalErrorHolder = {false};

	Client client = new Client(this, monitor);
	Request commitRequest = new Commit(project.getFullPath().makeRelative(), releaseComment, context.getTagName(), new CommitEntryIterator(resources, sharingManager, context, onlyIfOutgoing), new ICheckedInListener() {
		public void checkedIn(IPath contextRelativePath, String entryLine) {
			int secondSlashIndex = entryLine.indexOf('/', 1);
			String fileName = entryLine.substring(1, secondSlashIndex);
			IPath filePath = contextRelativePath.append(fileName);
			String revision = entryLine.substring(secondSlashIndex + 1, entryLine.indexOf('/', secondSlashIndex + 1));
			CVSResource serverFile = new CVSResource(context, filePath, IResource.FILE, revision);
			IResource releasedFile = project.findMember(filePath, true);
			try {
				releaseListener.resourceReleased(releasedFile, serverFile);
				resourcesToRelease.remove(releasedFile);
			} catch (CoreException ex) {
				CVSPlugin.log(new CVSStatus(ICVSStatus.ERROR, releasedFile.getFullPath(), CVSPlugin.getResourceString("CVSAdapter.errorReleasingFile"), ex));
				internalErrorHolder[0] = true;
			}
		}
		public void removed(IPath repositoryRelativePath) {
			IResource releasedFile = project.findMember(repositoryRelativePath.removeFirstSegments(1), true);
			try {
				releaseListener.resourceReleased(releasedFile, null);
				resourcesToRelease.remove(releasedFile);
			} catch (CoreException ex) {
				CVSPlugin.log(new CVSStatus(ICVSStatus.ERROR, releasedFile.getFullPath(), CVSPlugin.getResourceString("CVSAdapter.errorReleasingDeletion"), ex));
				internalErrorHolder[0] = true;
			}
		}
	});
	try {
		client.execute(commitRequest);
	} finally {
		client.disconnect();
	}

	// update release info for resources released but identical on server so no
	// new revision created on server. this will clear outgoing.
	for (Iterator it = resourcesToRelease.iterator(); it.hasNext();) {
		IResource resource = (IResource) it.next();
		ISyncInfo info = sharingManager.getSyncInfo(resource);
		try {
			// Use the remote handle because a common will not always exist. (e.g. in 
			// the case of an add/add conflict where the files have the same content).
			if( info.getRemote() != null ) {
				IServerResource remoteRevision = ((Handle) info.getRemote()).getServerResource();
				releaseListener.resourceReleased(resource, remoteRevision);
			}
		} catch (CoreException ex) {
			CVSPlugin.log(new CVSStatus(ICVSStatus.ERROR, resource.getFullPath(), CVSPlugin.getResourceString("CVSAdapter.errorReleasingDeletion"), ex));
			internalErrorHolder[0] = true;
		}
	}
	
	if(internalErrorHolder[0]) {
		throw new CVSException(new CVSStatus(ICVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapter.errorReleasing"), null));
	}
}
/**
 * Return true if the exception from the cvs server is the no tag error, and false
 * otherwise.
 */
protected boolean isNoTagException(CVSServerException e) {
	if (!(e.getStatus() instanceof MultiStatus))
		return false;
	IStatus[] children = ((MultiStatus) e.getStatus()).getChildren();
	if (children.length != 1)
		return false;
	if (!children[0].getMessage().startsWith("cvs [server aborted]: no such tag"))
		return false;
	return true;
}
public IServerResourceTree load(final IPath projectLocation, int type, IServerWorkspace workspace, final IPath workspaceRelativePath, final IProgressMonitor monitor) throws CoreException {
			
	final Context context = (Context) workspace;

	// repository-relative path of the resource to load
	final IPath remotePath = context.getPathPrefix().append(workspaceRelativePath);

	// the context of the checkout request
	final IPath updateContextPath;
	final IPath updateTargetPath;
	
	if (type == IResource.FILE) {
		// loading a file - context should be parent of the file
		updateContextPath = remotePath.removeLastSegments(1);
		updateTargetPath = new Path(remotePath.lastSegment());
	} else {
		// loading a folder or a project - context is the remotePath itself
		updateContextPath = remotePath;
		updateTargetPath = Path.EMPTY;
	}

	String tagName = context.getTagName();

	// the tree builder is responsible for building the IServerResourceTree returned by this method
	final CVSResourceTreeBuilder builder = new CVSResourceTreeBuilder((Context) workspace, workspaceRelativePath);
	
	// to collect errors reported by the server:
	final List errors = new ArrayList();

	Client client = new Client(this, monitor);

	// the update listener does the actual work...
	// we have to remove the project name from the local location because the cvs checkout returns project 
	// paths and not project relative. Update returns project relative.
	Request request = new Checkout(updateContextPath, updateTargetPath, tagName, null, null,
							new UpdatedListener(projectLocation.removeLastSegments(1), Path.EMPTY, builder, errors, monitor));
	try {
		client.execute(request);
	} catch(CVSServerException e) {
		// Translate the exception so that users can understand how to resolve the no
		// tag error, other errors should be reported as is to the user.
		if( isNoTagException(e) ) {
			throw new CVSException(CVSPlugin.getResourceString("CVSAdapter.noTagExceptionOnLoad", new Object[] {context.getPathPrefix(), context.getTagName()}), workspaceRelativePath, e);
		} else {
			throw e;
		}
	} finally {
		client.disconnect();
	}

	// the caller (sharing manager) is responsible for refreshing the resource tree

	// If the request was canceled, it is now safe to exit the task.
	if (monitor.isCanceled())
		throw new OperationCanceledException();

	if (errors.size() > 0) {
		MultiStatus multi = new MultiStatus(VCMPlugin.PI_VCM, IVCMStatus.SERVER_ERROR, (IStatus[]) errors.toArray(new IStatus[errors.size()]), CVSPlugin.getResourceString("CVSAdapter.ioLoad"), null);
		throw new CVSException(multi);
	}

	if (builder.getBuiltTree().getResource() == null) {
		// no file was loaded; check if the project/folder exists
		IServerResource serverResource = workspace.fetchResourceAt(workspaceRelativePath, monitor);
		if (serverResource == null) {
			throw new CVSException(ICVSStatus.SERVER_ERROR, updateContextPath, CVSPlugin.getResourceString("CVSAdapter.noSuchResource"), null);
		} else {
			// resource does exist on server, but has no members. If it is a folder,
			// we haven't created it yet, so just create it:
			if (type == IResource.FOLDER) {
				File folder = new File(projectLocation.append(updateContextPath).toOSString());
				folder.mkdir();
			}
			// fix the server resource tree:
			builder.getBuiltTree().resource = (CVSResource) serverResource;
		}
	}

	return builder.getBuiltTree();
}
public IStatus loadShallow(ISubscriber subscriber, IServerWorkspace serverWorkspace, IResource[] resources, ICatchupListener catchupListener, IProgressMonitor monitor) throws CoreException {
	Map projectSlices = computeProjectSlices(resources);

	// collects problems due to incoming folder deletions that cannot be
	// performed because there are still managed resources in them
	List deleteProblems = new ArrayList();

	for (Iterator it = projectSlices.keySet().iterator(); it.hasNext();) {
		if (monitor.isCanceled())
			throw new OperationCanceledException();

		final IProject project = (IProject) it.next();
		SortedSet sortedResources = (SortedSet) projectSlices.get(project);

		// perform deletions:
		Collection toRemove = performDeletions(subscriber, sortedResources, catchupListener, deleteProblems);
		sortedResources.removeAll(toRemove);

		internalLoadShallow(subscriber, serverWorkspace, sortedResources, catchupListener, monitor);
	}

	if(deleteProblems.isEmpty()) {
		return new CVSStatus(ICVSStatus.OK, "OK");
	} else {
		IStatus[] statuses = (IStatus[])deleteProblems.toArray(new IStatus[deleteProblems.size()]);
		return new MultiStatus(CVSPlugin.PI_CVS, ICVSStatus.ERROR, statuses, CVSPlugin.getResourceString("CVSAdapter.cannotPerformIncomingDeletion"), null);
	}
}
public void magicChangeTeamStreamFile(final IFile file, final InputStream contents) throws VCMException {
	magicCommitFile(file, contents, ISyncInfo.CHANGE);
}
public void magicCommitFile(final IFile file, final InputStream contents, final int outgoingChange) throws VCMException {
	try {
		// create a new revision of this file in the repository without eclipse knowing
		// about it.
		final int size;
		final String revision;
		final ByteArrayInputStream output;

		Client client = new Client(this, null);
		ISharingManager sharingManager = VCMPlugin.getProvider().getSharingManager();
		ISyncInfo info = sharingManager.getSyncInfo(file);

		if (info != null && info.getCommon() != null)
			revision = info.getCommon().getVersionName();
		else
			revision = "0";

		if (contents != null) {
			contents.reset();
			ByteArrayOutputStream os = new ByteArrayOutputStream();
			int c;
			while ((c = contents.read()) != -1) {
				os.write(c);
			}
			output = new ByteArrayInputStream(os.toByteArray());
			size = os.size();
		} else {
			size = 0;
			output = null;
		}

		IEntryContentIterator singleFileIterator = new IEntryContentIterator() {
			boolean dirSent = false;
			boolean fileSent = false;

			public IPath nextDirectory() {
				if (dirSent)
					return null;
				dirSent = true;
				return file.getParent().getProjectRelativePath();
			}
			public String nextEntryLine() {
				if (fileSent)
					return null;
				fileSent = true;
				String filename = file.getName();
				String entryLine = "/" + filename + "/";
				switch (outgoingChange) {
					case ISyncInfo.CREATE :
						entryLine += "0";
						break;
					case ISyncInfo.DELETE :
						entryLine += "-";
						// uh-oh: fall-through
					case ISyncInfo.CHANGE :
						entryLine += revision;
						break;
				}
				// treat all files as binary for now
				entryLine += "//-kb/";
				return entryLine;
			}
			public IFileContent nextFileContent() {
				if (output == null)
					return null;
				return new IFileContent() {
					public InputStream getInputStream() {
						return output;
					}
					public long getSize() {
						return size;
					}
				};
			}
		};
		Request commitRequest = new Commit(file.getProject().getFullPath().makeRelative(), "nothing", null, singleFileIterator, new ICheckedInListener() {
			public void checkedIn(IPath contextRelativeDirectory, String entryLine) {
				return;
			}
			public void removed(IPath repositoryRelativePath) {
				return;
			}
		});
		try {
			client.execute(commitRequest);
		} finally {
			client.disconnect();
		}
	} catch (IOException e) {
		throw new VCMException(VCMStatus.ERROR, file.getLocation(), CVSPlugin.getResourceString("CVSAdapter.ioMagic"), e);
	}
}
public void magicCreateTeamStreamFile(IFile file, InputStream contents) throws VCMException {
	magicCommitFile(file, contents, ISyncInfo.CREATE);
}
public void magicCreateTeamStreamFolder(IContainer folder) throws VCMException {
	IPath[] newDirectoryPaths = new IPath[] { folder.getFullPath().makeRelative()};
	IPath[] parentPathArray = new IPath[] { folder.getFullPath().makeRelative(), folder.getParent().getFullPath().makeRelative()};
	Request addRequest = new Add(Path.EMPTY, new DirectoryEntryIterator(parentPathArray), newDirectoryPaths);
	Client client = new Client(this, null);
	try {
		client.execute(addRequest);
	} finally {
		client.disconnect();
	}
}
public void magicDeleteTeamStreamFile(final IFile file) throws VCMException {
	magicCommitFile(file, null, ISyncInfo.DELETE);
}
public void magicDeleteTeamStreamFolder(IFolder folder, IRepositoryLocation location) throws VCMException {
	String repoLocation = location.getLocation();
	String repositoryHost = repoLocation.substring(repoLocation.indexOf("@", 0) + 1, repoLocation.lastIndexOf(":"));
	String userName = repoLocation.substring(repoLocation.indexOf(":", 1) + 1, repoLocation.indexOf('@'));
	String repoRoot = repoLocation.substring(repoLocation.lastIndexOf(":") + 1, repoLocation.length());
	IPath path = new Path(repoRoot).append(folder.getFullPath());
	String cmd1 = new String("rsh " + repositoryHost + " -l " + userName + " rm -rf " + path);
	try {
		Process p = Runtime.getRuntime().exec(cmd1);
		p.waitFor();
	} catch (IOException e) {
		throw new VCMException(VCMStatus.ERROR, CVSPlugin.getResourceString("CVSAdapter.ioExceptionDeleting", new Object[] {folder.getFullPath()}));
	} catch (InterruptedException e) {
		throw new VCMException(VCMStatus.ERROR, CVSPlugin.getResourceString("CVSAdapter.interruptedExceptionDeleting", new Object[] {folder.getFullPath()}));
	}
	return;
}
public void magicGetRoundTripCount(Integer[] count) {
	count[0] = new Integer(roundTripCnt);
}
public void magicInitializeRepository(IRepositoryLocation location) throws VCMException {
	String repoLocation = location.getLocation();
	String repositoryHost = repoLocation.substring(repoLocation.indexOf("@", 0) + 1, repoLocation.lastIndexOf(":"));
	String userName = repoLocation.substring(repoLocation.indexOf(":", 1) + 1, repoLocation.indexOf('@'));
	String repoRoot = repoLocation.substring(repoLocation.lastIndexOf(":") + 1, repoLocation.length());
	String cmd1 = new String("rsh " + repositoryHost + " -l " + userName + " rm -rf " + repoRoot);
	String cmd2 = new String("rsh " + repositoryHost + " -l " + userName + " cvs -d " + repoRoot + " init");
	try {
		Process p = Runtime.getRuntime().exec(cmd1);
		p.waitFor();
		p = Runtime.getRuntime().exec(cmd2);
	} catch (IOException e) {
		//throw new VCMException(VCMStatus.ERROR, "unable to setup remote repository within magic");
	} catch (InterruptedException e) {
		//throw new VCMException(VCMStatus.ERROR, "unable to setup remote repository within magic: remote commands interrupted");
	}
	return;
}
public void magicResetRoundTripCount() {
	roundTripCnt = 0;
}
private void mkdirs(IContainer target, Context context, ICatchupListener catchupListener) throws CoreException {
	if (target.exists())
		return;
	mkdirs(target.getParent(), context, catchupListener);
	if (target.getType() == IResource.FOLDER) {
		((IFolder) target).create(false, true, null);
		IServerResource newServerDir = new CVSResource(context, target.getProjectRelativePath(), IResource.FOLDER, FOLDER_REVISION);
		catchupListener.resourceCaughtup(target, newServerDir);
	} else {
		throw new VCMException(IVCMStatus.ERROR, target.getFullPath(), CVSPlugin.getResourceString("CVSAdapter.createContainers"), null);
	}
}
Collection performDeletions(ISubscriber subscriber, SortedSet resourceSet, ICatchupListener catchupListener, List deleteProblems) throws CoreException {
	// result of this method:
	List resourcesWithIncomingDeletions = new ArrayList();
	// the resources are passed in sorted. We need to iterate
	// in reverse order so that we see children before parents.
	IResource[] resources = (IResource[])resourceSet.toArray(new IResource[0]);
	for(int i=resources.length-1; i>=0; i--) {
		IResource resource = resources[i];
		ISyncInfo syncInfo = subscriber.getSyncInfo(resource);
		if (syncInfo != null && syncInfo.getRemote()==null) {
			// delete case
			resourcesWithIncomingDeletions.add(resource);
			boolean deleteIsOK = true;
			if(resource.getType() == IResource.FOLDER) {
				// if the resource is a folder, check if it still has a managed
				// child (this can happen if there are outgoing changes in a folder
				// which has an incoming deletion)
				// (just a reminder: there are no incoming deletions on projects)
				// we don't care about phantoms here
				IResource[] children = ((IContainer)resource).members();
				for(int j=0; j<children.length; j++) {
					// we use the sharing manager because it is the one that cares about outgoing changes/creations
					if(VCMPlugin.getProvider().getSharingManager().isManaged(children[j])) {
						// don't perform the deletion because there are still managed resources
						deleteIsOK = false;
						deleteProblems.add(new CVSStatus(ICVSStatus.ERROR, children[j].getFullPath(), CVSPlugin.getResourceString("CVSAdapter.doesNotHaveIncomingDeletion", new Object[]{children[j].getFullPath().toString()})));
					}
				}
			}
			// delete ignores non-existing resources, so we can call it without checking for existence
			if(deleteIsOK) {
				resources[i].delete(false, null);
				catchupListener.resourceCaughtup(resource, null);
			}
		} else
			if (syncInfo == null) {
				// XXX this code can probably be removed.
				// it was originally put in because we did not iterate
				// in reverse order which means that some of the resources
				// could have been deleted already because of the deletion
				// of its parent. This code should not be reached anymore.
				resourcesWithIncomingDeletions.add(resources[i]);
			}
	}
	return resourcesWithIncomingDeletions;
}
public static List readCvsIgnoreFile(InputStream in) throws IOException {
	BufferedReader input = new BufferedReader(new InputStreamReader(in));
	String line;
	List entries = new ArrayList(11);
	try {
		while ((line = input.readLine()) != null) {
			entries.add(line);
		}
	} finally {
		input.close();
	}
	return entries;
}
public IServerResource readServerResource(IResource ignoredByCVS, IServerWorkspace serverWorkspace, IPath projectRelativePath, int resourceType, DataInputStream input) throws IOException {
	Context context = (Context) serverWorkspace;
	String revision = input.readUTF();
	if (resourceType == IResource.PROJECT)
		return new CVSModule(context, revision);
	else
		return new CVSResource(context, projectRelativePath, resourceType, revision);
}
public void release(IChangeSet changeSet, boolean onlyIfOutgoing, final IReleaseListener releaseListener, IProgressMonitor monitor) throws CoreException {
	
	Map projectSlices = computeProjectSlices(changeSet.getResources());
	for (Iterator it = projectSlices.keySet().iterator(); it.hasNext();) {
		if (monitor.isCanceled())
			throw new OperationCanceledException();

		final IProject project = (IProject) it.next();
		SortedSet resources = (SortedSet) projectSlices.get(project);

		ITeamStream ts = VCMPlugin.getProvider().getSharingManager().getSharing(project);
		IServerWorkspace serverWorkspace = getTeamStreamProject(ts.getName(), project.getName());
		internalRelease(serverWorkspace, resources, onlyIfOutgoing, changeSet.getComment(), releaseListener, monitor);
	}
}
private void removeKnownRemoteDirectory(String projectName, IPath path) {
	SortedSet remoteDirs = (SortedSet) knownRemoteDirectories.get(projectName);
	if (remoteDirs != null)
		remoteDirs.remove(path);
}
private boolean sameBranch(CVSResource r1, CVSResource r2) {
	String branch1 = r1.stateId.substring(0, r1.stateId.lastIndexOf('.'));
	String branch2 = r2.stateId.substring(0, r2.stateId.lastIndexOf('.'));
	return branch1.equals(branch2);
}
public IStatus setIgnored(IResource resource, boolean value) throws CoreException {

	try {
		// you can't ignore projects!
		if (resource.getType() != IResource.PROJECT) {
			IContainer parentResource = resource.getParent();
			IFile cvsIgnoreFile = (IFile) parentResource.getFile(new Path(".cvsignore"));
			addCvsIgnoreEntry(cvsIgnoreFile, resource);
		}
		return new CVSStatus(ICVSStatus.OK, CVSPlugin.getResourceString("ok"));

	} catch (IOException ex) {
		return new CVSStatus(ICVSStatus.ERROR, resource.getFullPath(), CVSPlugin.getResourceString("CVSAdapter.cvsignoreEdit"), ex);
	}
}
public void shutdown() throws CVSException {
	CVSPlugin.getInstance().removeAllTeamStreams(getRepositoryRoot());
}
static boolean validatePath(String path) {
	return validatePath(new Path(path));
}
static boolean validatePath(IPath path) {
	/* path must not have a device */
	if (path.getDevice() != null)
		return false;

	/* path must not be the root path */
	if (path.isRoot())
		return false;
	return true;
}
public IStatus validateProjectVersionName(String name) {
	return validateTagName(name);
}
public IStatus validateTagName(String name) {
	if (name == null)
		return new CVSStatus(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapter.nullName"));
	if (name.equals(""))
		return new CVSStatus(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapter.emptyName"));
	if (!Character. isLetter(name.charAt(0)))
		return new CVSStatus(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapter.beginName"));
	
	for (int i = 0; i < name.length(); i++) {
		char c = name.charAt(i);
		if ( Character.isSpaceChar(c) || c == '$' || c == ',' || c == '.' || c == ':' || c == ';' || c == '@' )
			return new CVSStatus(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapter.badCharName"));
	}
	return new CVSStatus(ICVSStatus.OK, CVSPlugin.getResourceString("ok"));
}
public IStatus validateTeamStreamName(String name) {
	return validateTagName(name);
}
public void writeServerResource(IServerResource target, DataOutputStream output) throws IOException {
	output.writeUTF(((StateId) target.getStateId()).getId());
}
}
