package org.eclipse.vcm.internal.core.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.runtime.*;
import org.eclipse.vcm.internal.core.adapter.*;
import org.eclipse.vcm.internal.core.base.IVCMProvider;
import java.io.*;
import java.io.File;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.CoreException;

import org.eclipse.vcm.internal.core.ccvs.client.CVSException;
import java.util.*;
import java.text.MessageFormat;

/**
 * <code>CVSPlugin</code> is the plug-in runtime class
 * for the portion of the VCM system that is specific to CVS.
 */
public class CVSPlugin extends Plugin {
	/**
	 * The singleton plugin instance.
	 */
	private static CVSPlugin instance;
	
	/**
	 * The id of the CVS plug-in
	 */
	public static final String PI_CVS = "org.eclipse.vcm.core.cvs";
	/**
	 * The id of the authenticator extension point
	 */
	public static final String PT_AUTHENTICATOR = "authenticator";
	/**
	 * The id of the connectionmethods extension point
	 */
	public static final String PT_CONNECTIONMETHODS = "connectionmethods";
	/**
	 * This map is used by the adapter instances to remember team stream names.
	 * keys: String (repository location string)
	 * values: ArrayList of String (team stream names)
	 */
	private Map teamStreamNames = new HashMap(11);
	private static final String TEAMSTREAMS_FILE = ".teamstream-names";
	
	private static ResourceBundle resourceBundle;	

/** Constructs a plug-in runtime class for the given plug-in descriptor.
 */
public CVSPlugin(IPluginDescriptor pluginDescriptor) {
	super(pluginDescriptor);
	instance = this;		
	try {
		resourceBundle = ResourceBundle.getBundle("org.eclipse.vcm.internal.core.ccvs.CVSPluginResources");
	} catch (MissingResourceException ex) {
		log(new CVSStatus(ICVSStatus.WARNING, null, "resource bundle missing for CVS plugin", ex));
		resourceBundle = null;
	}		
}
void addTeamStream(String repositoryLocation, String teamStreamName) throws CVSException {
	List names = (List) teamStreamNames.get(repositoryLocation);
	if (names == null)
		names = new ArrayList(5);
	names.add(teamStreamName);
	teamStreamNames.put(repositoryLocation, names);
	writeTeamStreamNames();
}
/**
 * Return the CVS plugin instance
 */
public static CVSPlugin getInstance() {
	return instance;
}
/**
 * Gets the CVS resource bundle
 */
public static ResourceBundle getResourceBundle() {
	return resourceBundle;
}
/**
 * Gets a string from the CVSPlugin resource bundle.
 * We don't want to crash because of a missing String.
 * Returns the key if not found.
 */
public static String getResourceString(String key) {
	try {
		return resourceBundle.getString(key);
	} catch (MissingResourceException e) {
		return key;
	} catch (NullPointerException e) {
		return "!" + key + "!";
	}			
}
/**
 * Gets a string from the resource bundle and binds it
 * with the given arguments. If the key is not found,
 * return the key.
 */
public static String getResourceString(String key, Object[] args) {
	try {
		return MessageFormat.format(getResourceString(key), args);
	} catch (MissingResourceException e) {
		return key;
	} catch (NullPointerException e) {
		return "!" + key + "!";
	}
}
String[] getTeamStreamNames(String repositoryLocation) {
	List names = (List) teamStreamNames.get(repositoryLocation);
	if (names == null)
		names = new ArrayList(1);
	return (String[]) names.toArray(new String[names.size()]);
}
/**
 * Convenience method for logging statuses to the plugin log
 */
public static void log(IStatus status) {
	instance.getLog().log(status);
}
private void readTeamStreamNames() throws CVSException {
	IPath pluginStateLocation = getStateLocation();
	File tempFile = pluginStateLocation.append(TEAMSTREAMS_FILE + ".tmp").toFile();
	File stateFile = pluginStateLocation.append(TEAMSTREAMS_FILE).toFile();
	if (!stateFile.exists()) {
		if (!tempFile.exists())
			return;
		boolean renamed = tempFile.renameTo(stateFile);
		if (!renamed)
			throw new CVSException(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSPlugin.renaming", new Object[] {tempFile.getAbsolutePath()}), null);
	}
	try {
		DataInputStream dis = new DataInputStream(new FileInputStream(stateFile));
		int repositoryCount = dis.readInt();
		for (int i = 0; i < repositoryCount; i++) {
			String repositoryLocation = dis.readUTF();
			int nameCount = dis.readInt();
			List names = new ArrayList(nameCount);
			for (int j = 0; j < nameCount; j++) {
				String name = dis.readUTF();
				names.add(name);
			}
			teamStreamNames.put(repositoryLocation, names);
		}
	} catch (IOException ex) {
		throw new CVSException(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSPlugin.reading", new Object[] {stateFile.getAbsolutePath()}), ex);
	}
}
void removeAllTeamStreams(String repositoryLocation) throws CVSException {
	teamStreamNames.remove(repositoryLocation);
	writeTeamStreamNames();
}
void removeTeamStream(String repositoryLocation, String teamStreamName) throws CVSException {
	List names = (List) teamStreamNames.get(repositoryLocation);
	if (names == null)
		return;
	names.remove(teamStreamName);
	teamStreamNames.put(repositoryLocation, names);
	writeTeamStreamNames();
}
public void shutdown() throws CoreException {
}
public void startup() throws CoreException {
	readTeamStreamNames();
}
private void writeTeamStreamNames() throws CVSException {
	// persist team stream names
	IPath pluginStateLocation = getStateLocation();
	File tempFile = pluginStateLocation.append(TEAMSTREAMS_FILE + ".tmp").toFile();
	File stateFile = pluginStateLocation.append(TEAMSTREAMS_FILE).toFile();
	try {
		DataOutputStream dos = new DataOutputStream(new FileOutputStream(tempFile));
		dos.writeInt(teamStreamNames.size());
		for (Iterator it = teamStreamNames.keySet().iterator(); it.hasNext();) {
			String repositoryLocation = (String) it.next();
			dos.writeUTF(repositoryLocation);
			Collection names = (Collection) teamStreamNames.get(repositoryLocation);
			dos.writeInt(names.size());
			for (Iterator it2 = names.iterator(); it2.hasNext();) {
				String name = (String) it2.next();
				dos.writeUTF(name);
			}
		}
		dos.close();
		if (stateFile.exists())
			stateFile.delete();
		boolean renamed = tempFile.renameTo(stateFile);
		if (!renamed)
			throw new CVSException(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSPlugin.renaming", new Object[] {tempFile.getAbsolutePath()}), null);
	} catch (IOException ex) {
		throw new CVSException(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSPlugin.saving", new Object[] {stateFile.getAbsolutePath()}), ex);
	}
}
}
