package org.eclipse.vcm.internal.core.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.*;
import org.eclipse.core.runtime.*;
import java.util.*;
import org.eclipse.core.resources.*;
import org.eclipse.vcm.internal.core.ccvs.requests.*;
import org.eclipse.vcm.internal.core.base.*;

public class CommitEntryIterator implements IEntryContentIterator {
	ISharingManager sharingManager;
	Iterator resourceIterator;
	IResource currentResource;
	boolean entrySent = false;
	boolean onlyIfOutgoing = true;
	Context context;

public CommitEntryIterator(SortedSet resources, ISharingManager sharingManager, Context context, boolean onlyIfOutgoing) {
	this.sharingManager = sharingManager;
	this.resourceIterator = resources.iterator();
	this.context = context;
	this.onlyIfOutgoing = onlyIfOutgoing;
}
public IPath nextDirectory() {
	if (currentResource == null || currentResource.getType() == IResource.FILE) {
		do {
			if (!resourceIterator.hasNext())
				return null;
			currentResource = (IResource) resourceIterator.next();
		} while (currentResource.getType() == IResource.FILE);
	}
	IPath result = currentResource.getProjectRelativePath();
	currentResource = null;
	return result;
}
public String nextEntryLine() {
	if (currentResource == null) {
		if (!resourceIterator.hasNext())
			return null;
		currentResource = (IResource) resourceIterator.next();
		entrySent = false;
	}
	
	
	if (currentResource.getType() == IResource.FILE && !entrySent) {
		ISyncInfo syncInfo = sharingManager.getSyncInfo(currentResource);
		
		// Depending on the type of release, the common or remote revision
		// numbers will be sent to the repository. It's important that we 
		// never send an empty revision number to the server because the cvs
		// server will crash! Thus, if we are overiding a conflict and releasing
		// we must ensure that if the remote does not exist (change/delete) we
		// send '0' as the revision number.
		IResourceEdition remoteResource = syncInfo.getRemote();
		IResourceEdition commonResource = syncInfo.getCommon();
		String commonRevisionNum = null;
		String remoteRevisionNum = null;
		if( remoteResource != null ) {
			remoteRevisionNum = ((StateId) remoteResource.getStateId()).getId();
		} else {
			remoteRevisionNum = "0";
		}
		if( commonResource != null ) {
			commonRevisionNum = ((StateId) commonResource.getStateId()).getId();
		} else {
			commonRevisionNum = "0";
		}
		
		String filename = currentResource.getName();
		String entryLine = "/" + filename + "/";
		switch (syncInfo.getOutgoingChange()) {
			case ISyncInfo.CREATE :
				if( onlyIfOutgoing || remoteResource == null) {
					entryLine += "0";
				} else {
					entryLine += remoteRevisionNum;
				}		
				break;
			case ISyncInfo.DELETE :
				entryLine += "-";
				// uh-oh: fall-through
			case ISyncInfo.CHANGE :				
				// uh-oh: fall-through
			default :
				// To overide a conflict and release anyway, we tell the server that
				// we have the remote resource. Otherwise, we are not overiding a
				// conflict and wish to release based on our common.
				// We must handle the case were the outgoing sync is ignored. If
				// the user choses to release regardless of the sync info. In this
				// case we behave exactly as if it was an outgoing change.
				if( onlyIfOutgoing ) {
					entryLine += commonRevisionNum;
				} else {
					entryLine += remoteRevisionNum;
				}				
		}
		// treat all files as binary for now
		entryLine += "//-kb/";

		// Send tag name
		String tagName = context.getTagName();
		if (!tagName.equals("HEAD")) {
			// append 'T' for Tag.
			entryLine += "T" + tagName;
		}

		entrySent = true;
		return entryLine;
	} else {
		return null;
	}
}
public IFileContent nextFileContent() {
	if (currentResource == null || currentResource.getType() != IResource.FILE) {
		currentResource = null;
		return null;
	}
	ISyncInfo syncInfo = sharingManager.getSyncInfo(currentResource);
	if (syncInfo.getOutgoingChange() == ISyncInfo.DELETE) {
		currentResource = null;
		return null;
	} else {
		final File file = new File(currentResource.getLocation().toOSString());
		currentResource = null;
		return new IFileContent() {
			public long getSize() {
				return file.length();
			}
			public InputStream getInputStream() throws IOException {
				return new FileInputStream(file);
			}
		};
	}
}
}
