package org.eclipse.vcm.internal.core.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.resources.IResource;
import org.eclipse.vcm.internal.core.base.IStateId;
import org.eclipse.vcm.internal.core.adapter.*;
import org.eclipse.vcm.internal.core.VCMException;
import org.eclipse.vcm.internal.core.ccvs.requests.*;
import org.eclipse.vcm.internal.core.ccvs.client.*;
import java.io.*;
import java.util.*;

public class Context implements IServerWorkspace {
	String tagName;
	CVSAdapter adapter;
	IPath pathPrefix;
/**
 * The "path prefix" is a path with one segment. It is a path rather than a string because it
 * is mostly used together with a project relative path to construct a full path.
 */
public Context(CVSAdapter adapter, IPath pathPrefix, String tagName) {
	this.adapter = adapter;
	this.pathPrefix = pathPrefix;
	this.tagName = tagName;
}
public void delete(IPath path) {
}
public boolean equals(Object other) {
	if (!(other instanceof Context))
		return false;
	Context otherContext = (Context) other;
	return Util.equals(getRepositoryRoot(), otherContext.getRepositoryRoot()) && Util.equals(pathPrefix, otherContext.pathPrefix) && Util.equals(tagName, otherContext.tagName);
}
public String[] fetchMemberNames(IPath path, final IProgressMonitor monitor) throws VCMException {
	String[][] filesAndDirs = adapter.internalFetchMemberNames(getPathPrefix().append(path), getTagName(), monitor);
	List result = new ArrayList();
	result.addAll(Arrays.asList(filesAndDirs[0]));
	result.addAll(Arrays.asList(filesAndDirs[1]));
	return (String[]) result.toArray(new String[result.size()]);
}
public IServerResource[] fetchMembers(final IPath path, final IProgressMonitor monitor) throws VCMException {
	Client client = new Client(adapter, monitor);
	try {
		return internalFetchMembers(client, path, monitor);
	} finally {
		client.disconnect();
	}
}
public IServerResource fetchResourceAt(IPath path, IProgressMonitor monitor) throws VCMException {
	String revision = null;
	if (path.isEmpty()) {
		String[] moduleNames;
		List topLevelModules = Arrays.asList(adapter.internalFetchMemberNames(Path.EMPTY, "HEAD", monitor)[0]);
		List result = new ArrayList();
		result.addAll(topLevelModules);
		result.remove("CVSROOT");
		moduleNames = (String[]) result.toArray(new String[result.size()]);
		if (Util.arrayIncludes(moduleNames, getPathPrefix().lastSegment())) {
			return new CVSModule(this, CVSAdapter.FOLDER_REVISION);
		} else {
			return null;
		}
	}
	final List remoteRevs = new ArrayList(11);
	Client client = new Client(adapter, monitor);
	Request request = new Status(new IPath[] { getPathPrefix().append(path.removeLastSegments(1))}, new IPath[] { getPathPrefix().append(path)}, new String[] { "-l" }, getTagName(), null, new IStatusListener() {
		public void fileStatus(IPath path, String remoteRevision) {
			remoteRevs.add(remoteRevision);
		}
	});
	try {
		client.execute(request);
	} catch (CVSServerException e) {
		if (remoteRevs.isEmpty())
			throw e;
	}
	client.disconnect();
	if (!remoteRevs.isEmpty())
		revision = (String) remoteRevs.get(0);

	// check if resource exists:
	if (revision == null)
		return null;
	// check type (CVSAdapter.FOLDER_REVISION is returned as the revision number for all folders)
	int type = IResource.FILE;
	if (revision.equals(CVSAdapter.FOLDER_REVISION))
		type = IResource.FOLDER;
	return new CVSResource(this, path, type, revision);
}
IPath getPathPrefix(){
	return pathPrefix;
}
/**
 * Return the project version name of this workspace. If the
 * workspace is a team stream the result is unspecified.
 */
public String getProjectVersionName() {
	return getTagName();
}
String getRepositoryRoot() {
	return adapter.getRepositoryRoot();
}
String getTagName() {
	return tagName;
}
public int hashCode() {
	return Util.hashCode(getRepositoryRoot()) + Util.hashCode(pathPrefix) + Util.hashCode(tagName);
}
/**
 * Returns handles to the remote members at the give remote path. This operation can be run within one 
 * cvs connection. The caller is responsible for connecting and disconnecting the client.
 * 
 * This operation is not included in the adapter API, and is only intended for use within
 * the cvs adapter.
 */
public IServerResource[] internalFetchMembers(Client client, final IPath path, final IProgressMonitor monitor) throws VCMException {
	
	final String[][] fileAndDirMembers = adapter.internalFetchMemberNames(client, getPathPrefix().append(path), getTagName(), monitor);

	// add directory members to result
	final List result = new ArrayList();
	for (int i = 0; i < fileAndDirMembers[0].length; i++) {
		result.add(new CVSResource(this, path.append(fileAndDirMembers[0][i]), IResource.FOLDER, CVSAdapter.FOLDER_REVISION));
	}

	List realFilenames = Arrays.asList(fileAndDirMembers[1]);

	// get revisions of file members
	IPath[] filePaths = new IPath[realFilenames.size()];
	for (int i = 0; i < filePaths.length; i++) {
		String fileName = (String) realFilenames.get(i);
		filePaths[i] = getPathPrefix().append(path).append(fileName);
	}

	if (filePaths.length > 0) {
		Request request = new Status(new IPath[] { getPathPrefix().append(path)}, filePaths, new String[] { "-l" }, getTagName(), null, new IStatusListener() {
			public void fileStatus(IPath filePath, String revision) {
				result.add(new CVSResource(Context.this, path.append(filePath.lastSegment()), IResource.FILE, revision));
				monitor.subTask(filePath.lastSegment());
				monitor.worked(1);
			}
		});
		client.execute(request);
	}
	return (CVSResource[]) result.toArray(new CVSResource[result.size()]);
}
/** Returns all projects for the given tag */
public String[] internalFetchModulesForTag(String projectName, final String tagName, IProgressMonitor monitor) throws VCMException {
	Client client = new Client(adapter, monitor);
	final List result = new ArrayList();
	Request request = new History(projectName, null, new IHistoryListener() {
		public void historyLine(String module, String tag, String author, Date creationDate) {
			if (tagName.equals(tag))
				result.add(module);
		}
	});
	client.execute(request);
	client.disconnect();
	return (String[]) result.toArray(new String[0]);
}
}
