package org.eclipse.vcm.internal.core.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.vcm.internal.core.Policy;
import org.eclipse.vcm.internal.core.ccvs.requests.IUpdatedListener;
import org.eclipse.vcm.internal.core.ccvs.utils.StreamUtil;

/**
 * Listener for the "Updated" response from a CVS server. This class
 * is responsible for loading files.
 */
public class UpdatedListener implements IUpdatedListener {
	
	IPath projectLocation;
	IPath updateContextPath;
	/** may be null */
	CVSResourceTreeBuilder builder;
	List errors;
	IProgressMonitor monitor;
	
	UpdatedListener(IPath projectLocation, IPath updateContextPath, CVSResourceTreeBuilder builder, List errors, IProgressMonitor monitor) {
		this.projectLocation = projectLocation;
		this.updateContextPath = updateContextPath;
		this.builder = builder;
		this.errors = errors;
		this.monitor = monitor;
	}
	/**
	 * @see IUpdatedListener#fileInformation(IPath, String, InputStream, int)
	 */
	// this is called for every file coming from the server. The path arg does not include the file name,
	// it is part of the entry line.
	public void fileInformation(IPath updateContextRelativePath, String entryLine, InputStream in, int size) {
			int secondSlashIndex = entryLine.indexOf('/', 1);
			String fileName = entryLine.substring(1, secondSlashIndex);
			String revision = entryLine.substring(secondSlashIndex + 1, entryLine.indexOf('/', secondSlashIndex + 1));

			IPath projectRelativePath = updateContextPath.removeLastSegments(1).append(updateContextRelativePath).append(fileName);
			IPath localLocation = projectLocation.append(projectRelativePath);

			if( !CVSAdapter.validatePath(updateContextRelativePath) || !CVSAdapter.validatePath(fileName) ) {
				Util.logError(CVSPlugin.getResourceString("CVSAdapter.badPath", new Object[] {updateContextRelativePath.append(fileName).toOSString()}), null);
				return;
			}
						
			String progressTitle = projectRelativePath.toString();
			monitor.subTask(progressTitle);

			// Make dirs if needed
			File file = new File(localLocation.toOSString());
			File parent = file.getParentFile();
			parent.mkdirs();

			// write the file.
			OutputStream out = null;
			try {
				out = new FileOutputStream(file);
				StreamUtil.transferWithProgress(in, out, size, Policy.subMonitorFor(monitor, size), progressTitle);
			} catch (IOException ex) {
				errors.add(new CVSStatus(ICVSStatus.ERROR, localLocation, CVSPlugin.getResourceString("CVSAdapter.ioError"), ex));
			} finally {
				try {
					if(out!=null)
						out.close();
				} catch (IOException ex) {
					errors.add(new CVSStatus(ICVSStatus.ERROR, localLocation, CVSPlugin.getResourceString("CVSAdapter.ioError"), ex));
				}
			}

			if (builder!=null) {
				// add a node to the server resource tree being built.
				builder.fileStatus(updateContextRelativePath.append(fileName).removeFirstSegments(1), revision);
			}

			monitor.worked(1);
	}
}
