package org.eclipse.vcm.internal.core.ccvs.client;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.vcm.internal.core.IVCMStatus;
import org.eclipse.vcm.internal.core.Policy;
import org.eclipse.vcm.internal.core.VCMException;
import org.eclipse.vcm.internal.core.base.VCMPlugin;
import org.eclipse.vcm.internal.core.ccvs.CVSAdapter;
import org.eclipse.vcm.internal.core.ccvs.CVSPlugin;
import org.eclipse.vcm.internal.core.ccvs.CVSStatus;
import org.eclipse.vcm.internal.core.ccvs.ICVSStatus;
import org.eclipse.vcm.internal.core.ccvs.IUserAuthenticator;
import org.eclipse.vcm.internal.core.ccvs.IUserInfo;
import org.eclipse.vcm.internal.core.ccvs.requests.IFileContent;
import org.eclipse.vcm.internal.core.ccvs.utils.PathUtil;
import org.eclipse.vcm.internal.core.ccvs.utils.StreamUtil;

/**
 * An generic cvs client that can execute a request and can handle
 * responses. Requests are execute by passing a <code>Request</code>
 * to the <code>execute</code> method. Server responses are handled
 * by registering a response handler either via the client directly
 * or by using the <code>IRequestContext</code> interface.
 *
 * After the client has established a connection to a server it
 * uses the client / server negotiation protcoll to tell the server
 * which response can be handled by the client. Additionally the
 * server tells the client which requests can the handle by the
 * server. The list of responses that can be handled by the client
 * is determined by the registered handlers.
 * <p>
 * Although not documented the client must have a handler for at
 * least the following responses: "ok", "error", "Checked-in",
 * "Updated", "Merged", "Removed", "M" text and "E" text.
 * <p.
 * The client installs default hanlder for all must responses as 
 * long as there isn't a user specific handler for the response. 
 * The default handlers generate an <code>Assert</code> when it
 * gets activated.
 * <p>
 * Subclasses should override <code>installResponseHandlers</code> to 
 * install their own response handlers
 */
public class Client {

	private Connection fConnection;
	private IRequestContext fRequestContext;
	private IResponseContext fResponseContext;
	private Hashtable fHandlers;
	private Vector fErrors;
	private String fValidRequests;
	private IProgressMonitor fProgressMonitor;
	private IProgressMonitor fNullProgressMonitor;
	private Hashtable fDataPool;
	private WorkingCopyFileUpdater fAdminFileUpdater;
	private String fLocalLocation;
	
	private static final String ok = "ok";
	private static final String error = "error";
	private static final int error_length = error.length() + 1;	
	
	private static final String CVSROOT = "CVSROOT";
	private static final char BLANK_DELIMITER = ' ';
	
	/** Implements the IResponseContext using the client API */
	private class ClientResponseContext implements IResponseContext {
		public String getRootDirectory() throws CVSException {
			return fConnection.getRootDirectory();
		}

		public InputStream getResponseStream() throws CVSException {
			return fConnection.getResponseStream();
		}

		public String readLine() throws CVSException {
			return fConnection.readLine();
		}

		public String readToken() throws CVSException {
			return fConnection.readToken();
		}

		public IProgressMonitor getProgressMonitor() {
			return Client.this.getProgressMonitor();
		}

		public void remember(Object key, Object value) {
			fDataPool.put(key, value);
		}
		public Object get(Object key) {
			return fDataPool.get(key);
		}
		public void addError(String error) {
			Client.this.addError(error);
		}
	}
	
/** Implements the IRequestContext using the client API */
private class ClientRequestContext implements IRequestContext, IRequestNames {
	public IProgressMonitor getProgressMonitor() {
		return Client.this.getProgressMonitor();
	}

	public String getRootDirectory() throws CVSException {
		return fConnection.getRootDirectory();
	}

	public ResponseHandler setResponseHandler(ResponseHandler handler) {
		return Client.this.setResponseHandler(handler);
	}

	public ResponseHandler removeResponseHandler(String key) {
		return Client.this.removeResponseHandler(key);
	}

	public void sendArgument(String arg) throws CVSException {
		fConnection.writeLine(Argument, arg);
	}

	public void sendIsModified(String file) throws CVSException {
		fConnection.writeLine(is_modified, file);
	}
	/**
	 * The Directory request is sent as:
	 * 		Directory localdir
	 * 		repository_root/remotedir
	 * 
	 * [Note: A CVS repository root can end with a trailing slash. The CVS server
	 * expects that the repository root sent contain this extra slash. Including
	 * the foward slash in addition to the absolute remote path makes for a string
	 * containing two consecutive slashes (e.g. /home/cvs/repo//projecta/a.txt).
	 * This is valid in the CVS protocol.]
	 */
	public void sendDirectory(IPath local, IPath remote) throws CVSException {		
		fConnection.writeLine(Directory, PathUtil.toString(local.makeRelative()));
		fConnection.writeLine(getRootDirectory() + PathUtil.toString(remote.makeAbsolute()));
	}

	public void sendEntry(String entryLine) throws CVSException {
		fConnection.writeLine(Entry, entryLine);
	}

	public void sendGlobalOption(String option) throws CVSException {
		fConnection.writeLine(Global_option, option);
	}

	public void sendUnchanged(String filename) throws CVSException {
		fConnection.writeLine(Unchanged, filename);
	}

	public void sendSticky(String tag) throws CVSException {
		fConnection.writeLine(Sticky, tag);
	}

public void sendModified(String filename, IFileContent fileContent) throws CVSException {
	IProgressMonitor monitor = Policy.monitorFor(getProgressMonitor());
	fConnection.writeLine(Modified, filename);
	fConnection.writeLine("u=rw,g=rw,o=r");
	long size = fileContent.getSize();
	fConnection.writeLine("" + size);
	try {
		String progressTitle = CVSPlugin.getResourceString("Client.releasing", new Object[] {filename});
		monitor.subTask(progressTitle);
		InputStream is = fileContent.getInputStream();
		StreamUtil.transferWithProgress(is, fConnection.getRequestStream(), size, monitor, progressTitle);
		is.close();
	} catch (FileNotFoundException ex) {
		throw new CVSFileException(CVSPlugin.getResourceString("Client.fileNotFound", new Object[] {filename}), new Path(filename));
	} catch (IOException ex) {
		throw new CVSFileException(new Path(filename), ex);
	}
}

	public String expandModules(String name) throws CVSException {
		if (!isValidRequest(expand_modules))
			return null;
		sendArgument(name);
		sendDirectory(Path.EMPTY, new Path(getRootDirectory()));
		fConnection.writeLine(expand_modules);
		return (String) processResponses(ModuleExpansionHandler.NAME);
	}
}
	
	private static class UserInfo implements IUserInfo {
		private String username;
		private String password;
		public void setUsername(String username) {
			this.username = username;
		}
		public void setPassword(String password) {
			this.password = password;
		}
	}

/**
 * Creates a new server for the given cvs root. Password
 * <code>password</code> is used to log into the server.
 */
public Client(CVSAdapter adapter, IProgressMonitor monitor) throws CVSException {
	IUserAuthenticator authenticator = adapter.getAuthenticator();
	if (authenticator == null) {
		throw new CVSAuthenticationException(adapter.getLocation().getLocation(), CVSPlugin.getResourceString("Client.noAuthenticator"));
	}
	String message = null;
	boolean retry = false;
	while (true) {
		UserInfo userInfo = new UserInfo();
		try {
			IStatus status = authenticator.authenticateUser(adapter.getLocation(), userInfo, retry, message);
			if (!status.isOK()) {
				throw new CVSAuthenticationException(status);
			}
		} catch (CVSException e) {
			throw e;
		} catch (VCMException e) {
			throw new CVSException(e.getStatus());
		}
		try {
			fConnection = Connection.create(adapter.getLocation().getLocation(), userInfo.password);
			fConnection.open();
			
			// keep track of the number of connections made to the server.
			adapter.incrementRoundTripCount();
			
			// if we get here, authentication did succeed. Break out of the loop...
			break;
		} catch (CVSAuthenticationException ex) {
			retry = true;
			message = ex.getMessage();
		}
	}
	initialize();
	setProgressMonitor(monitor);
}
	//---- Error handling -----------------------------------------------------------
	
	/**
	 * Add a server error message to the list of server errors.
	 */
	public void addError(String error) {
		fErrors.addElement(error);	
	}
	/**
	 * Disconnects the client from the server
	 */
	public void disconnect() throws CVSException {
		if (fConnection != null)
			fConnection.close();
	}
	/**
	 * Executes the given request.
	 */
	public void execute(Request request) throws CVSException {
		if( ! isValidRequest(request.getName()) ) {
			throw new CVSException(CVSPlugin.getResourceString("Client.requestNotSupported",
															new Object[] {request.getName(), fConnection.getCVSRoot()}));
		}
		IProgressMonitor monitor = getProgressMonitor();
		try {
			monitor.beginTask(VCMPlugin.getResourceString("Handle.server"), 100);
			if (monitor.isCanceled())
				throw new OperationCanceledException();

			request.setUp(fRequestContext);

			// Guess that set up contributes 20% of work.
			monitor.worked(20);
			if (monitor.isCanceled())
				throw new OperationCanceledException();

			fConnection.writeLine(request.getName());
			try {
				// Processing responses contributes 70% of work.
				processResponses();
			} catch (CVSServerException e) {
				request.finished(fRequestContext, false);
				throw e;
			}
			// Finished adds last 10% of work.
			request.finished(fRequestContext, true);
			monitor.worked(10);
		} finally {
			monitor.done();
		}
	}
	/**
	 * Checks, whether there is a response handler under the
	 * given key.
	 */
	protected boolean existsResponseHandler(String key) {
		return fHandlers.get(key) != null;
	}
	/**
	 * Returns the list of server errors.
	 */
	private Vector getErrors() {
		return fErrors;
	}
	private IProgressMonitor getProgressMonitor() {
		if (fProgressMonitor != null)
			return fProgressMonitor;
		if (fNullProgressMonitor == null)
			fNullProgressMonitor = new NullProgressMonitor();
		return fNullProgressMonitor;
	}
	/**
	 * Intializes the client.
	 */
	private void initialize() throws CVSException {
		fRequestContext= new ClientRequestContext();
		fResponseContext= new ClientResponseContext();
		fHandlers= new Hashtable(25);
		installResponseHandlers();
		installDefaultResponseHandlers();
		fDataPool= new Hashtable(10);
		fErrors= new Vector(5);
		
		fAdminFileUpdater = new WorkingCopyFileUpdater(fConnection.getCVSRoot(),
													   fLocalLocation);
		
		// Tell the server out response handlers.
		fConnection.writeLine(IRequestNames.Valid_responses, makeResponseList());
		
		// Get all valid requests from the server
		fConnection.writeLine(IRequestNames.valid_requests);
		fValidRequests= (String)processResponses(ValidRequestsHandler.NAME);
		
		// Set the root.
		if (isValidRequest(IRequestNames.Root))
			fConnection.writeLine(IRequestNames.Root, fConnection.getRootDirectory());
	}
	/**
	 * Installs default handlers for the must responses if thers isn't 
	 * any user defined response handler.
	 */
	private void installDefaultResponseHandlers() {
		if (!existsResponseHandler(MessageHandler.NAME))
			setResponseHandler(new MessageHandler());
			
		if (!existsResponseHandler(ErrorMessageHandler.NAME))	
			setResponseHandler(new ErrorMessageHandler());
			
		if (!existsResponseHandler(ValidRequestsHandler.NAME))	
			setResponseHandler(new ValidRequestsHandler());
			
		if (!existsResponseHandler(ModuleExpansionHandler.NAME))			
			setResponseHandler(new ModuleExpansionHandler());
		
		final String MERGED= "Merged";
		if (!existsResponseHandler(MERGED))
			setResponseHandler(new UnsupportedHandler() {
				public String getName() {
					return MERGED;
				}
			});

		final String CHECKED_IN= "Checked-in";
		if (!existsResponseHandler(CHECKED_IN))
			setResponseHandler(new UnsupportedHandler() {
				public String getName() {
					return CHECKED_IN;
				}
			});
			
		final String UPDATED= "Updated";
		if (!existsResponseHandler(UPDATED))
			setResponseHandler(new UnsupportedHandler() {
				public String getName() {
					return UPDATED;
				}
			});
			
		final String REMOVED= "Removed";
		if (!existsResponseHandler(REMOVED))
			setResponseHandler(new UnsupportedHandler() {
				public String getName() {
					return REMOVED;
				}
			});
	}
	/**
	 * Install customized handlers. Subclasses should reimplement this
	 * method to install special response handlers.
	 */
	protected void installResponseHandlers() {
	}
	/**
	 * Is the client connected.
	 */
	public boolean isConnected() {
		return fConnection.isEstablished();
	}
	//---- Basic Request API --------------------------------------------------------
	
	/**
	 * Is the given request a valid server request.
	 */
	public boolean isValidRequest(String requestName) {
		if (fValidRequests == null)
			return false;
		return fValidRequests.indexOf(requestName) != -1;
	}
	/**
	 * Computes the response list form the list of install response handlers.
	 */
	private String makeResponseList() {
		// Init are "must handlers" which aren't currently implemented.
		StringBuffer result= new StringBuffer("ok error");		
		Enumeration elements= fHandlers.elements();
		while (elements.hasMoreElements()) {
			ResponseHandler handler= (ResponseHandler)elements.nextElement();
			result.append(' ');
			result.append(handler.getName());
		}
		return result.toString();
	}
	/**
	 * Runs the response event loop.
	 */
	private void processResponses() throws CVSException {
		processResponses(null);
	}
/**
 * Runs the response event loop. The result from the response
 * <code>responseName</code> is returned by this method. If there
 * wasn't any response for the name then <code>null</code> is returned.
 */
private Object processResponses(String responseName) throws CVSException {
	Hashtable involvedHandlers = new Hashtable(10);
	boolean success = false;
	fConnection.flush();
	Object result = null;
	IProgressMonitor monitor = getProgressMonitor();
	int work = 70; // Processing responses contribtes 70% of total work.
	resetErrors();
	try {
		while (true) {
			String response = fConnection.readToken();
			
			// Don't really know how much work we have to do, so count up to max and wait there.
			if (work-- > 0)
				monitor.worked(1);
			if (monitor.isCanceled())
				throw new OperationCanceledException();
				
			if (response.startsWith(ok)) {
				// We should report errors that occured even if the request was ok. The error
				// handlers must ensure that they filter out expected error messages sent by the
				// server.
				if(!fErrors.isEmpty()) {
					MultiStatus status = new MultiStatus(CVSPlugin.PI_CVS, IVCMStatus.SERVER_ERROR, CVSPlugin.getResourceString("Client.problemsReported"), null);
					for (Iterator it = fErrors.iterator(); it.hasNext();) {
						status.add(new CVSStatus(ICVSStatus.SERVER_ERROR, (String) it.next()));
					}
					throw new CVSServerException(status);
				}
				success = true;
				break;
			} else
				if (error.equals(response)) {
					String message = response;
					// the error tag can be followed by some error message too
					if (fConnection.getLastUsedDelimiterToken() == BLANK_DELIMITER)
						message = fConnection.readLine();
					if(message.equals("") || message.equals(" ")) message = CVSPlugin.getResourceString("Client.problemsReported");
					MultiStatus status = new MultiStatus(CVSPlugin.PI_CVS, IVCMStatus.SERVER_ERROR, message, null);
					for (Iterator it = fErrors.iterator(); it.hasNext();) {
						status.add(new CVSStatus(ICVSStatus.SERVER_ERROR, (String) it.next()));
					}
					throw new CVSServerException(status);
				} else {
					ResponseHandler handler = (ResponseHandler) fHandlers.get(response);
					if (handler != null) {
						involvedHandlers.put(handler, handler);
						Object o = handler.handle(fResponseContext);
						if (handler.getName().equals(responseName)){
							result = o;
						}
					} else
						if (fConnection.closed) {
							MultiStatus status = new MultiStatus(CVSPlugin.PI_CVS, IVCMStatus.SERVER_ERROR, CVSPlugin.getResourceString("Client.closedConnection"), null);
							for (Iterator it = fErrors.iterator(); it.hasNext();) {
								status.add(new CVSStatus(ICVSStatus.SERVER_ERROR, (String) it.next()));
							}
							throw new CVSServerException(status);
						} else {
							// read rest of line.
							if (fConnection.getLastUsedDelimiterToken() == BLANK_DELIMITER)
								fConnection.readLine();
							// write to log ???
						}
				}
		}
	} finally {
		Enumeration e = involvedHandlers.keys();
		while (e.hasMoreElements()) {
			((ResponseHandler) e.nextElement()).finished(fResponseContext, success);
		}
		fDataPool.clear();
		// Show the final amount of work completed (if any).
		monitor.worked(work);
	}
	return result;
}
	/**
	 * Remove the response handler managed under the given key.
	 */
	protected ResponseHandler removeResponseHandler(String key) {
		return (ResponseHandler)fHandlers.remove(key);
	}
	/**
	 * Resets the list of server error.
	 */	
	private void resetErrors() {
		fErrors.clear();
	}
	/**
	 * Resets the progress monitor to the <code>NullProgressMonitor</code>
	 */
	public void resetProgressMonitor() {
		fProgressMonitor= null;
	}
	//---- Progress Monitor --------------------------------------------------------
	
	/**
	 * Sets the progress monitor to be used in requests and responses.
	 */
	public void setProgressMonitor(IProgressMonitor pm) {
		fProgressMonitor= pm;
	}
	//---- Response handling --------------------------------------------------------

	/**
	 * Installs the given response handler. Any previously install
	 * reponse handler is returned.
	 */
	protected ResponseHandler setResponseHandler(ResponseHandler handler) {
		return (ResponseHandler)fHandlers.put(handler.getName(), handler);
	}
}
