package org.eclipse.vcm.internal.core.ccvs.client;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.eclipse.vcm.internal.core.Assert;
import org.eclipse.vcm.internal.core.ccvs.CVSAdapterFactory;
import org.eclipse.vcm.internal.core.ccvs.CVSPlugin;
import org.eclipse.vcm.internal.core.ccvs.base.IConnectionMethod;
import org.eclipse.vcm.internal.core.ccvs.base.IServerConnection;

/**
 * A connection to talk to a cvs server. The life cycle of a connection is
 * as follows:
 * <ul>
 *	<li> Connection.create: creates a new connection object. The right
 *	     connection is determined from the cvsroot.
 *	<li> open: opens a connection.
 *	<li> send a request: use write* method or use the request stream directly.
 *	     <code>GetRequestStream</code> returns an output stream to directly
 *	     talk to the server.
 *	<li> read responses: use read* methods or use the response stream directly.
 *	     <code>GetResponseStream</code> returns an input stream to directly
 *	     read output from the server.
 *	<li> close: closes the connection. A closed connection can be reopened by
 *	     calling open again.
 * </ul>
 */
public class Connection {
	
	private static final boolean DEBUG= System.getProperty("cvsclient.debug")!=null;
	
	public static final byte NEWLINE= 0xA;
	
	protected static final String LOCAL= ":local:";
	protected static final int    LOCAL_LENGTH= LOCAL.length();
	
	protected static final String PSERVER= ":pserver:";
	protected static final int    PSERVER_LENGTH= PSERVER.length();
	
	protected static final String EXT= ":ext:";
	protected static final int    EXT_LENGTH= EXT.length();
	
	private IServerConnection serverConnection;
	
	private String fCVSRoot;
	private String fCVSRootDirectory;
	private boolean fIsEstablished;
	private BufferedInputStream fResponseStream;
	private char fLastUsedTokenDelimiter;

	boolean closed = false;

	protected Connection(String cvsroot, IServerConnection serverConnection) {
		fCVSRoot = cvsroot;
		this.serverConnection = serverConnection;
		Assert.isNotNull(fCVSRoot);
	}
	private static byte[] append(byte[] buffer, int index, byte b) {
		if (index >= buffer.length) {
			byte[] newBuffer= new byte[index * 2];
			System.arraycopy(buffer, 0, newBuffer, 0, buffer.length);
			buffer= newBuffer;
		}
		buffer[index]= b;
		return buffer;
	}
	/**
	 * Closes the connection.
	 */
	public void close() throws CVSException {
		if (!isEstablished())
			return;
		try {
			readPendingInput();
			serverConnection.close();
		} catch (IOException ex) {
			throw new CVSCommunicationException(CVSPlugin.getResourceString("Connection.cannotClose"), ex);
		} finally {
			fResponseStream= null;
			fIsEstablished= false;
		}
	}
	/**
	 * Creates a new connection for the given location. Password can
	 * be <code>null</code> indicating that no password is required
	 * or that a default password should be used.
	 */
	public static Connection create(String cvsroot, String password) throws CVSException {
		
		String method = cvsroot.substring(1, cvsroot.indexOf(':', 1));
		
		IConnectionMethod[] methods = CVSAdapterFactory.getPluggedInConnectionMethods();
		for(int i=0; i<methods.length; i++) {
			String name = methods[i].getName();
			if(name.equals(method))
				return new Connection(cvsroot, methods[i].createConnection(cvsroot, password));
		}
			
			return null;	
	}
	/**
	 * Flushes the request stream.
	 */
	public void flush() throws CVSException {
		if (!isEstablished())
			return;
		try {
			getRequestStream().flush();	
		} catch(IOException e) {
			throw new CVSCommunicationException(e);
		}
	}
	//---- CVS root management -------------------------------------------------------
	
	/**
	 * Returns the CVS root.
	 */ 
	public String getCVSRoot() {
		return fCVSRoot;
	}
	/**
	 * Returns the last delimiter character used to read a token.
	 */
	public char getLastUsedDelimiterToken() {
		return fLastUsedTokenDelimiter;
	}
	/**
	 * Returns the <code>OutputStream</code> used to send requests
	 * to the server.
	 */
	public OutputStream getRequestStream() throws CVSException {
		if (!isEstablished())
			return null;
		return serverConnection.getOutputStream();
	}
	/**
	 * Returns the <code>InputStream</code> used to read responses from
	 * the server.
	 */
	public InputStream getResponseStream() throws CVSException {
		if (!isEstablished())
			return null;
		if (fResponseStream == null)
			fResponseStream= new BufferedInputStream(serverConnection.getInputStream());
		return fResponseStream;	
	}
	public String getRootDirectory() throws CVSException {
		if (fCVSRootDirectory == null)
			fCVSRootDirectory = getRootDirectory(fCVSRoot);
		return fCVSRootDirectory;
	}
	/**
	 * Returns the root directory part of the cvs root.
	 */
	public static String getRootDirectory(String cvsroot) throws CVSException {
		int lastPos= cvsroot.length() - 1;
		int p1= cvsroot.indexOf(':');
		
		p1++;
		int p2= cvsroot.indexOf(':', p1);
		if (p2 == -1) {
			if (p1 == lastPos)
				throw new CVSException(CVSPlugin.getResourceString("Connection.malformedCvsRoot", new Object[] {cvsroot}));
			else
				return cvsroot.substring(p1);
		}
		
		int p3= cvsroot.indexOf(':', ++p2);
		if (p3 == -1 || p3 == lastPos)
			throw new CVSException(CVSPlugin.getResourceString("Connection.malformedCvsRoot", new Object[] {cvsroot}));
			
		return cvsroot.substring(++p3).replace('\\', '/');
	}
	/**
	 * Returns <code>true</code> if the connection is established;
	 * otherwise <code>false</code>.
	 */
	public boolean isEstablished() {
		return fIsEstablished;
	}
	//--- Helper to read strings from server -----------------------------------------
	
	/**
	 * Is input available in the response stream.
	 */
	public boolean isInputAvailable() {
		if (!isEstablished())
			return false;
		try {
			return getResponseStream().available() != 0;
		} catch (CVSException e) {
			return false;
		} catch (IOException e) {
			return false;
		}	
	}
	/**
	 * Creates a blank separated string from the given string array.
	 */
	private String makeString(String[] s) {
		StringBuffer buffer= new StringBuffer();
		int length= s.length;
		for (int i= 0; i < s.length; i++) {
			if (i != 0)
				buffer.append(' ');
			buffer.append(s[i]);
		}
		return buffer.toString();
	}
	/**
	 * Opens the connection.
	 */	
	public void open() throws CVSException {
		if (isEstablished())
			return;
		serverConnection.open();
		fIsEstablished= true;
	}
	/**
	 * Reads a line from the response stream.
	 */
	public String readLine() throws CVSException {
		return readLineOrUntil(-1);
	}
static String readLine(InputStream in) throws IOException {
	byte[] buffer = new byte[256];
	int index = 0;
	int r;
	while ((r = in.read()) != -1) {
		if (r == NEWLINE)
			break;
		buffer = append(buffer, index++, (byte) r);
	}
	String result = new String(buffer, 0, index);
	if (DEBUG)
		System.out.println(result);
	return result;
}
/**
 * Low level method to read a token.
 */
private String readLineOrUntil(int end) throws CVSException {
	if (!isEstablished())
		throw new CVSCommunicationException(CVSPlugin.getResourceString("Connection.readUnestablishedConnection"));
	byte[] buffer = new byte[256];
	InputStream in = getResponseStream();
	int index = 0;
	int r;
	try {
		while ((r = in.read()) != -1) {
			if (r == NEWLINE || (end != -1 && r == end))
				break;
			buffer = append(buffer, index++, (byte) r);
		}
		switch (r) {
			case -1 :
				closed = true;
			case NEWLINE :
				fLastUsedTokenDelimiter = '\n';
				break;
			default :
				fLastUsedTokenDelimiter = (char) r;
		}
		String result = new String(buffer, 0, index);
		if (DEBUG)
			System.out.print(result + fLastUsedTokenDelimiter);
		return result;
	} catch (IOException e) {
		throw new CVSCommunicationException(e);
	}
}
	/**
	 * Reads any pending input from the response stream so that
	 * the stream can savely be closed.
	 */
	protected void readPendingInput() throws CVSException {
		byte[] buffer= new byte[2048];
		InputStream in= getResponseStream();
		OutputStream out= getRequestStream();
		try {
			while (true) {
				int available = in.available();
				if (available < 1) break;
				if (available > buffer.length) available = buffer.length;
				if (in.read(buffer, 0, available) < 1) break;
			}
			out.flush();
			while (true) {
				int available = in.available();
				if (available < 1) break;
				if (available > buffer.length) available = buffer.length;
				if (in.read(buffer, 0, available) < 1) break;
			}
		} catch (IOException e) {
			throw new CVSCommunicationException(e);
		}	
	}
	/**
	 * Reads a token from the response stream.
	 */
	public String readToken() throws CVSException {
		return readLineOrUntil(' ');
	}
	/**
	 * Sends the given array of strings to the server. The array's strings
	 * are concatenated using a blank.
	 */
	public void write(String[] a) throws CVSException {
		write(makeString(a), false);
	}
	//---- Helper to send strings to the server ----------------------------
	
	/**
	 * Sends the given string to the server.
	 */
	public void write(String s) throws CVSException {
		write(s, false);
	}
	/**
	 * Sends the given two strings separated by a blank to the
	 * server.
	 */
	public void write(String s1, String s2) throws CVSException {
		write(s1 + ' ' + s2, false);
	}
	/**
	 * Low level method to write a string to the server. All write* methods are
	 * funneled through this method.
	 */
	void write(String s, boolean newline) throws CVSException {
		if (!isEstablished())
			throw new CVSCommunicationException(CVSPlugin.getResourceString("Connection.writeUnestablishedConnection"));
			
		if (DEBUG)
			System.out.print(s + (newline ? "\n" : ""));
	
		try {
			OutputStream out= getRequestStream();
			out.write(s.getBytes());
			if (newline)
				out.write(NEWLINE);
			
		} catch (IOException e) {
			throw new CVSCommunicationException(e);
		}
	}
	/**
	 * Sends the given array of strings to the server. The array's strings
	 * are concatenated using a blank. Additionally a newline is sent.
	 */
	public void writeLine(String[] a) throws CVSException {
		write(makeString(a), true);
	}
	/**
	 * Sends the given string and a newline to the server. 
	 */
	public void writeLine(String s) throws CVSException {
		write(s, true);
	}
	/**
	 * Sends the given two strings separated by a blank to the
	 * server. Additionally a newline is sent.
	 */
	public void writeLine(String s1, String s2) throws CVSException {
		write(s1 + ' ' + s2, true);
	}
}
