package org.eclipse.vcm.internal.ui.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.vcm.internal.core.VCMStatus;
import org.eclipse.vcm.internal.core.base.IRepositoryLocation;
import org.eclipse.vcm.internal.core.ccvs.IUserAuthenticator;
import org.eclipse.vcm.internal.core.ccvs.IUserInfo;
import org.eclipse.vcm.internal.core.ccvs.client.CVSException;
import org.eclipse.vcm.internal.core.VCMException;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

/**
 * An authenticator that prompts the user for authentication
 * info, and stores the results in the Platform's authentication
 * keyring.
 */
public class WorkbenchUserAuthenticator implements IUserAuthenticator {
	private static final String REPOSITORY_TYPE_CVS = "CVS";
	public static final String INFO_PASSWORD = "org.eclipse.ui.vcm.password";
	public static final String AUTH_SCHEME = "WhatsAnAuthScheme?";
	public static final URL FAKE_URL;
	
	public static final String INFO_USERNAME = "org.eclipse.ui.vcm.username";
	static {
		URL temp = null;
		try {
			temp = new URL("http://org.eclipse.ui.vcm");
		} catch (MalformedURLException e) {
		}
		FAKE_URL = temp;
	} 
/**
 * WorkbenchUserAuthenticator constructor comment.
 */
public WorkbenchUserAuthenticator() {
	super();
}
/**
 * @see IUserAuthenticator#authenticateUser
 */
public IStatus authenticateUser(final IRepositoryLocation location, final IUserInfo info, final boolean retry, final String message) throws VCMException {
	//make sure we're dealing with a CVS repository
	if (!location.getType().equals(REPOSITORY_TYPE_CVS)) {
		throw new CVSException(WorkbenchCVSPlugin.getResourceString("WorkbenchUserAuthenticator.notCVS"));
	}

	//first check the platform map
	if (!retry) {
		Map map = Platform.getAuthorizationInfo(FAKE_URL, location.getLocation(), AUTH_SCHEME);
		if (map != null) {
			String username = (String) map.get(INFO_USERNAME);
			String password = (String) map.get(INFO_PASSWORD);
			if (password != null) {
				info.setUsername(username);
				info.setPassword(password);
				return new VCMStatus(VCMStatus.OK, WorkbenchCVSPlugin.getResourceString("ok"));
			}
		}
	}

	//ask the user for a password
	final String[] result = new String[2];
	Display display = Display.getCurrent();
	if (display != null) {
		promptForPassword(location, message, result);
	} else {
		//sync exec in default thread
		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				promptForPassword(location, message, result);
			}
		});
	}

	if (result[0] == null) {
		return new VCMStatus(VCMStatus.INFO, WorkbenchCVSPlugin.getResourceString("WorkbenchUserAuthenticator.cancelled"));
	}

	//put the password into the Platform map
	Map map = Platform.getAuthorizationInfo(FAKE_URL, location.getLocation(), AUTH_SCHEME);
	if (map == null) {
		map = new java.util.HashMap(10);
	}
	map.put(INFO_USERNAME, result[0]);
	map.put(INFO_PASSWORD, result[1]);
	try {
		Platform.addAuthorizationInfo(FAKE_URL, location.getLocation(), AUTH_SCHEME, map);
	} catch (CoreException e) {
		WorkbenchCVSPlugin.log(e.getStatus());
		throw new CVSException(e.getStatus());
	}
	info.setUsername(result[0]);
	info.setPassword(result[1]);
	return new VCMStatus(VCMStatus.OK, WorkbenchCVSPlugin.getResourceString("ok"));
}
/**
 * Returns the CVS username for the given repository location.
 */
private String extractUsername(IRepositoryLocation repLocation) {
	String location = repLocation.getLocation();
	int start = location.indexOf(":", 1) + 1;
	int end = location.indexOf('@');
	return location.substring(start, end);
}
/**
 * Asks the user to enter a password. Places the
 * results in the supplied string[].  result[0] must
 * contain the username, result[1] must contain the password.
 * If the user canceled, both values must be zero.
 */
private void promptForPassword(IRepositoryLocation location, String message, String[] result) {
	Shell shell = new Shell();
	
	String username = extractUsername(location);
	UserValidationDialog dialog = new UserValidationDialog(shell, location.getLocation(), username, message);
	dialog.setUsernameMutable(false);
	dialog.open();

	shell.dispose();
	result[0] = dialog.getUsername();
	result[1] = dialog.getPassword();
}
}
