package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.IOException;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.scripting.internal.IXGRDataModel;
import org.eclipse.scripting.internal.IXGRElement;
import org.eclipse.scripting.internal.ScriptManager;
import org.eclipse.scripting.internal.ScriptObjectDescriptor;
import org.eclipse.scripting.internal.ScriptToolkitUI;
import org.eclipse.scripting.internal.uirenderer.XGR;
import org.xml.sax.SAXException;

/**
 * This class renders a user interface given a user interface definition
 * XHTML file, and an optional data definition XML file.  Wizard and preference dialogs are
 * the only supported user interface types.
 * The process flow is as follows&colon.
 * <ul>
 * <li>Create a Renderer instance</li>
 * <li>Give the renderer any objects whose methods are to be called when
 * an event occurs</li>
 * <li>
 * Give the renderer the name of a user interface definition file in the
 * form of either a workbench IFile, or a file system filename string, and the type of interface to render,
 * which may be one of the following&colon.
 * <ul>
 * <li>WIZARD</li>
 * <li>PREFERENCES</li>
 * </ul>
 * </li>
 * <li>Obtain and optionally store the IToolkitUI object from the renderer using the getToolkit method</li>
 * <li>Optionally obtain an IDataModel object from the IToolkitUI by calling one of the loadDataModel methods</li>
 * <li>Optionally associate the model with the renderer by using the IToolkitUI function
 * setUIDataModel</li>
 * </ul>
 * <p>
 * This class is primarily intended for use 
 * by developers who are exploiting the UI-rendering support in
 * this package as part of their workbench extension.
 * </p>
 * <p>
 * This class is not intended to be subclassed by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see IToolkitUI
 * @see ScriptAdapterEventHandler
 */
public class Renderer {
	public static final String WIZARD = "wizard";
	public static final String PREFERENCES = "preferences";

	private IFile filePresentation = null;
	private IXGRDataModel dataModel = null;
	private ScriptToolkitUI scriptToolkitUI = null;
	private String strPresentationFilename = null;
	private String strPresentationType = null;
	private String strTranslationsFilename = null;
	private Vector vectorObjectDescriptors = new Vector();
	private XGR xgr = null;
/**
 * Constructs a user interface renderer.
 */
public Renderer() {
	// Create a new script manager
	//----------------------------
	ScriptManager scriptManager = new ScriptManager();

	// Tell the script manager that we are running Java language scripts
	//------------------------------------------------------------------
	scriptManager.setJavaMode(true);

	// Create a renderer
	//------------------
	this.xgr = new XGR();

	// Give the script manager to the GUI renderer
	//--------------------------------------------
	this.xgr.setScriptManager(scriptManager);

	// Create a toolkit object
	//------------------------
	this.scriptToolkitUI = new ScriptToolkitUI();

	scriptManager.setToolkit(this.scriptToolkitUI);

	// Give the GUI renderer to the toolkit
	//-------------------------------------
	this.scriptToolkitUI.setGuiRenderer(this.xgr);
}
/**
 * Registers an object with the renderer.  This allows a script to call
 * methods on the given object by referencing the object by its given name.
 * For example, if an object is given the name myObject, and the object
 * has a method myMethod, then the method myMethod can be called when, for
 * example, an onreset user interface event occurs by specifying
 * myObject.myMethod() in the onload definition.
 * <code>
 * form id="page1" title="mytitle" onreset="myObject.myMethod()" action=""
 * </code>
 *
 * @param objectName object name
 * @param object object instance
 */
public void addObject(String strName, Object object) {
	this.vectorObjectDescriptors.addElement(
		new ScriptObjectDescriptor(strName, object));
}
/**
 * Returns an IToolkitUI object.
 *
 * @return script ui toolkit object
 */
public IToolkitUI getToolkit() {
	return this.scriptToolkitUI;
}
/**
 * Returns the widget that has the given user interface element identifier.
 *
 * @param userInterfaceElementID user interface element identifier
 * @return widget
 */
public Object getWidget( String userInterfaceElementID )
{
	return this.xgr.getWidget( userInterfaceElementID );
}
/**
 * Initializes and displays the user interface as defined in the
 * previously specified presentation file.
 *
 * @param bVisible indicates whether to make the rendered
 *		user interface visible
 * @exception IOException
 * @exception SAXException
 * @exception CoreException
 */
public void render(boolean bVisible) throws IOException, SAXException, CoreException {
	
	// Set the presentation type
	//--------------------------
	if (this.strPresentationType != null) {

		int iPresentationType = XGR.WIZARD;

		if (this.strPresentationType.equals("wizard") == true) {
			iPresentationType = XGR.WIZARD;
		}
		else if (this.strPresentationType.equals("preferences") == true) {
			iPresentationType = XGR.PREFERENCES;
		}
		else if (this.strPresentationType.equals("panel") == true) {
			iPresentationType = XGR.PANEL;
		}

		this.xgr.setPresentationType(iPresentationType);
	}

	// Set the xhtml filename
	//-----------------------
	if (this.strPresentationFilename != null) {
		this.xgr.setPresentationFilename(this.strPresentationFilename);
	}
	else if (this.filePresentation != null) {
		this.xgr.setPresentationFile(this.filePresentation);
	}

	// Set the translatable string properties filename
	//------------------------------------------------
	if (this.strTranslationsFilename != null) {
		this.xgr.setTranslationsFilename(this.strTranslationsFilename);
	}

	// Set the user's objects
	//-----------------------
	this.xgr.setJavaObjectDescriptors(this.vectorObjectDescriptors);

	// Present the GUI
	//----------------
	this.xgr.execute((IXGRElement) null, bVisible);
}
/**
 * Tells the renderer what user interface to render by
 * specifying a workbench XHTML file, and how to render it by
 * specifying the rendering type.  The possible rendering types are&colon.
 * <ul>
 * <li>WIZARD</li>
 * <li>PREFERENCES</li>
 * </ul>
 * @param presentationWorkbenchFile file containing user
 *		interface definition
 * @param presentationRenderingType  rendering type, specified as
 *		<code>WIZARD</code> or <code>PREFERENCES</code>
 */
public void setPresentationFile(IFile filePresentation, String presentationType) {
	this.filePresentation = filePresentation;
	this.strPresentationType = presentationType;
}
/**
 * Tells the renderer what user interface to render by
 * specifying a file system XHTML file, or a URL to an XHTML input stream,
 * and how to render it by
 * specifying the rendering type.  The possible rendering types are&colon.
 * <ul>
 * <li>WIZARD</li>
 * <li>PREFERENCES</li>
 * </ul>
 * @param presentationFilename file name, or URL to an input stream to a user 
 *		interface definition
 * @parem presentationType rendering type, specified as
 *		<code>WIZARD</code> or <code>PREFERENCES</code>
 */
public void setPresentationFilename(String presentationFilename, String presentationType) {
	this.strPresentationFilename = presentationFilename;
	this.strPresentationType = presentationType;
}
/**
 * Sets the filename, or URL to an input stream that contains translatable strings. 
 * This filename has to have a file extension of ".properties".  Any caption strings
 * that are found in the user interface definition file that are
 * prefixed with a % character are assumed to be keys into the
 * property file's table.  If the key is not found, the key without the
 * % prefix is used.
 *
 * @param filename resource properties name, or URL
 */
public void setTranslatablePropertiesFilename(String filename) {
	this.strTranslationsFilename = filename;
}
}
