package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

/**
 * This class is a script adapter for workbench resource editor.
 * <p>
 * This class is primarily intended for use 
 * by developers who are exploiting the UI-rendering support in
 * this package as part of their workbench extension.
 * </p>
 * <p>
 * This class is intended to be subclassed by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see IScriptAdapter
 * @see IToolkitUI
 * @see Renderer
 */
public class ScriptAdapterWorkbenchResourceEditor extends EditorPart implements IScriptAdapter {

	private IConfigurationElement configElement = null;
	private Object objData = null;
	private ScriptAdapterWorkbenchResourceEditorContent content = null;
	private String strName = null;
	private String strPresentationFilename = null;

/**
 * Constructs an instance of ScriptAdapterWorkbenchResourceEditor.
 */
public ScriptAdapterWorkbenchResourceEditor()
{
	super();
}
/**
 * Initializes and displays the user interface.
 *
 * @param compositeParent parent container
 */
public void createPartControl(Composite compositeParent) {

	// Create the editor content
	//--------------------------
	this.content = new ScriptAdapterWorkbenchResourceEditorContent(compositeParent);
	this.content.setInitializationData(this.configElement, this.strName, this.objData);
	this.content.setInput(getEditorInput());
	this.content.run();
}
/**
 * Saves the contents of this editor.
 *
 * @param monitor progress monitor
 */
public void doSave(IProgressMonitor monitor) {
	this.content.doSave();
}
/**
 * Saves the contents of the target.
 */
public void doSaveAs() {
	this.content.doSaveAs();
	return;
}
/**
 * Instructs the editor to go to the specified marker. The default implementation
 * of this method is a <code>noop</code>.
 *
 * @param marker to go to
 */
public void gotoMarker(IMarker marker) {
}
/**
 * Initializes the editor part with a site and input.
 * <p>
 * Subclasses of <code>EditorPart</code> must implement this method.  Within
 * the implementation subclasses should verify that the input type is acceptable
 * and then save the site and input.
 *
 * @param site editor site
 * @param input editor input
 * @exception PartInitException
 */
public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		// Check input
	//------------	
	if (!(input instanceof IFileEditorInput))
		throw new PartInitException("Invalid Input: " + input + ".  Input must implement IFileEditorInput");

	// Save input
	//-----------
	setSite(site);
	setInput(input);
}
/**
 * Returns whether the contents of this editor have changed since the last save
 * operation.  The default implementation always returns false.
 *
 * @return <code>false</code>.
 */
public boolean isDirty() {
	return false;
}
/**
 * Returns whether the "save as" operation is supported by this editor.  The default 
 * implementation always returns false.
 *
 * @return <code>false</code>.
 */
public boolean isSaveAsAllowed() {
	return false;
}
/**
 * Returns <code>true</code> if the contents of the target have changed since
 * the last save.  The default implementation always returns false.
 *
 * @return <code>false</code>.
 */
public boolean isSaveNeeded() {
	return false;
}
/**
 * Asks the part to take focus within the desktop.  The default implementation
 * of this method is a <code>noop</code>.
 */
public void setFocus() {
}
/**
 * This method is called on a newly constructed extension, passing 
 * it its relevant configuration information.
 *
 * @param config the configuration element used to trigger this execution. 
 *		It can be queried by the executable extension for specific
 *		configuration properties
 * @param propertyName the name of an attribute of the configuration element
 *		used on the <code>createExecutableExtension(String)</code> call. This
 *		argument can be used in the cases where a single configuration element
 *		is used to define multiple executable extensions.
 * @param data adapter data in the form of a <code>String</code>, 
 *		a <code>Hashtable</code>, or <code>null</code>.
 */
public void setInitializationData(
	IConfigurationElement configElement,
	String strName,
	Object objData) {

	super.setInitializationData(configElement, strName, objData);

	this.configElement = configElement;
	this.strName = strName;
	this.objData = objData;
}
/**
 * Associates event handler with the adaptor
 *
 * @param handler script adaptor event handler
 */
public void setScriptAdapterEventHandler( ScriptAdapterEventHandler handler ) {

	this.content.setScriptAdapterEventHandler( handler );
}
}
