package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.scripting.internal.ScriptRunnerExtended;
import org.eclipse.scripting.internal.uirenderer.XGRElementUISwtCompositePanel;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IFileEditorInput;

/**
 * This class is a script adapter for resource editor content viewer.
 * <p>
 * This class is primarily intended for use 
 * by developers who are exploiting the UI-rendering support in
 * this package as part of their workbench extension.
 * </p>
 * <p>
 * This class is intended to be subclassed by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see IScriptAdapter
 * @see IToolkitUI
 * @see Renderer
 */
public class ScriptAdapterWorkbenchResourceEditorContent extends ContentViewer implements IScriptAdapter {
	private ScriptRunnerExtended scriptRunner = null;
	private XGRElementUISwtCompositePanel xgrComposite = null;
	private IResource[] resourceSelectionArray = null;
/**
 * Constructs an instance of ScriptAdapterWorkbenchResourceEditorContent.
 */
public ScriptAdapterWorkbenchResourceEditorContent( Composite compositeParent ) {
	super();

	createControl( compositeParent );
}
/**
 * Creates editor content.
 *
 * @param compositeParent parent container
 * @return editor content
 */
protected Control createControl( Composite compositeParent ) {

	if( this.xgrComposite == null ) {
		
		this.xgrComposite = new XGRElementUISwtCompositePanel( compositeParent );
	}

	return (Composite)this.xgrComposite.getWidget();
}
/**
 * Saves the contents of this editor.
 */
public void doSave() {
	this.xgrComposite.doSave();
}
/**
 * Saves the contents of this editor to the selected resource.
 */
public void doSaveAs() {
	
	if (this.resourceSelectionArray != null && this.resourceSelectionArray.length > 0) {
		this.xgrComposite.doSaveAs(this.resourceSelectionArray[0]);
	}

	return;
}
/**
 * Returns the widget that contains the editor content.
 * @return content control
 */
public Control getControl() {
	
	return (Control)this.xgrComposite.getWidget();
}
/**
 * Returns an empty editor content selection.
 *
 * @return content selection
 */
public ISelection getSelection() {
	return StructuredSelection.EMPTY;
}
/**
 * Refreshes this viewer completely, getting new contents
 * for the input from the content provider.  The default implementation 
 * is a <code>noop</code>.
 */
public void refresh() {
}
/**
 * Presents the user interface.  The presentation type is ensured to be a panel.
 */
public void run() {
	
	if( this.scriptRunner != null ) {
		
		// Override the presentation type
		//-------------------------------
		this.scriptRunner.initialize( this.resourceSelectionArray );
		this.scriptRunner.setPresentationType( ScriptRunner.PANEL );
		this.scriptRunner.run( this.xgrComposite );
	}
}
/**
 * This method is called on a newly constructed extension, passing 
 * it its relevant configuration information.
 *
 * @param config the configuration element used to trigger this execution. 
 *		It can be queried by the executable extension for specific
 *		configuration properties
 * @param propertyName the name of an attribute of the configuration element
 *		used on the <code>createExecutableExtension(String)</code> call. This
 *		argument can be used in the cases where a single configuration element
 *		is used to define multiple executable extensions.
 * @param data adapter data in the form of a <code>String</code>, 
 *		a <code>Hashtable</code>, or <code>null</code>.
 */
public void setInitializationData(
	IConfigurationElement configElement,
	String strName,
	Object objData) {

	this.scriptRunner = new ScriptRunnerExtended(configElement, objData);
}
/**
 * Sets or clears the input for this viewer.  The input passed as a selection to scripts.
 *
 * @param object is the input of this viewer, or <code>null</code> if none
 */
public void setInput(Object object) {

	if (object instanceof IFileEditorInput) {
		this.resourceSelectionArray = new IFile[1];
		this.resourceSelectionArray[0] = ((IFileEditorInput) object).getFile();
	}

	else if (object instanceof IProject) {
		this.resourceSelectionArray = new IProject[1];
		this.resourceSelectionArray[0] = (IProject) object;
	}

	else if (object instanceof IResource) {
		this.resourceSelectionArray = new IResource[1];
		this.resourceSelectionArray[0] = (IResource) object;
	}

	else {
		this.resourceSelectionArray = new IResource[0];
	}
}
/**
 * Associates an event handler with the adaptor
 *
 * @param handler script adaptor event handler
 */
public void setScriptAdapterEventHandler( ScriptAdapterEventHandler handler ) {

	this.scriptRunner.setScriptAdapterEventHandler( handler );
}
/**
 * Sets a new selection for this viewer and optionally makes it visible.  The
 * default implementation of this method is a <code>noop</code>.
 *
 * @param selection the new selection
 * @param reveal <code>true</code> if the selection is to be made
 *   visible, and <code>false</code> otherwise
 */
public void setSelection(ISelection selection, boolean reveal) {
}
}
