package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.util.Vector;

/**
 * This class may be used as an intermediary buffer between a
 * programmer's string/object pairs, and the user interface renderer.
 * It is up to the developer to ensure that the objects are not null, and the
 * number of objects is equal to the number of strings.
 * This class is typically used with a user interface element that
 * is able to display a list of strings, such as a list box, or combination box.
 * <p>
 * This class is primarily intended for use 
 * by workbench users writing script code, or
 * by developers who are exploiting the UI-rendering support in
 * this package as part of their workbench extension.
 * </p>
 * <p>
 * This class is not intended to be subclassed by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 */
public class StringDataMultiple implements IStringData {
	private Vector _vectorStrings = null;
	private Vector _vectorObjects = null;
/**
 * This is the default constructor.
 */
public StringDataMultiple() {
	_vectorObjects = new Vector();
	_vectorStrings = new Vector();
}
/**
 * This constructs a multiple value buffer.  The parameters are the selectable
 * strings, and their corresponding objects.  Each string/object pair
 * becomes a member of a list of string/object pairs.  It is up to the
 * user of this class to ensure that the lengths of the two arrays are equal.
 *
 * @param strings
 * @param objects
 */
public StringDataMultiple(String[] strings, Object[] objects) {
	_vectorStrings = convertArrayToVector(strings);
	_vectorObjects = convertArrayToVector(objects);
}
/**
 * This method adds a string/object pair to the list
 * of string/object pairs.
 *
 * @param string 
 * @param object
 */
public void add(String string, Object object) {
	_vectorStrings.addElement(string);
	_vectorObjects.addElement(object);
}
/**
 * Converts an object array to a vector
 *
 * @param objaInput array
 * @return vector
 */
private Vector convertArrayToVector(Object[] objaInput) {
	Vector vectorOutput = new Vector(objaInput.length);
	for (int i = 0; i < objaInput.length; ++i) {
		vectorOutput.addElement(objaInput[i]);
	}
	return vectorOutput;
}
/**
 * Converts a vector of strings to a string array
 *
 * @param vectorInput
 * @return array
 */
private Object[] convertVectorToObjectArray(Vector vectorInput) {
	Object[] objaOutput = new Object[vectorInput.size()];
	for (int i = 0; i < vectorInput.size(); ++i) {
		objaOutput[i] = (Object) vectorInput.elementAt(i);
	}
	return objaOutput;
}
/**
 * Converts a vector of strings to a string array
 *
 * @param vectorInput
 * @return string array
 */
private String[] convertVectorToStringArray(Vector vectorInput) {
	String[] straOutput = new String[vectorInput.size()];
	for (int i = 0; i < vectorInput.size(); ++i) {
		straOutput[i] = (String) vectorInput.elementAt(i);
	}
	return straOutput;
}
/**
 * Returns an array of all the objects of the string/object pairs.
 *
 * @return array of defined objects
 */
public Object[] getObjects() {
	return convertVectorToObjectArray(_vectorObjects);
}
/**
 * Returns an array of all the string values of the string/object pairs.
 *
 * @return an array of defined string
 */
public String[] getStrings() {
	return convertVectorToStringArray(_vectorStrings);
}
/**
 * This method removes the string and associated
 * object from the list.  If
 * both parameters are not null, then both the String and the Object
 * are removed only if they have the same index in the string/object pair list.
 * If one parameter is null, then the non-null
 * parameter and corresponding partner is removed from the list.
 *
 * @param string
 * @param object
 * @return <code>true</code> if entry was removed
 */
public boolean remove(String string, Object object) {

	// Find the index of the object
	//-----------------------------
	int iObject = _vectorObjects.indexOf(object);

	// Find the index of the string
	//-----------------------------
	int iString = -1;
	for (int i = 0; i < _vectorStrings.size(); ++i) {
		if (string.equals((String) _vectorStrings.elementAt(i)) == true) {
			iString = i;
			break;
		}
	}

	// If the index of the object and the index of the
	// string are the same, then remove the string
	// and object from their lists.
	//------------------------------------------------
	if (iString == iObject) {
		_vectorStrings.removeElementAt(iString);
		_vectorObjects.removeElementAt(iObject);
		return true;
	}
	
	return false;
}
/**
 * This method sets the objects of the string/object pairs.
 *
 * @param objects
 */
public void setObjects(Object[] objects) {
	_vectorObjects = convertArrayToVector(objects);
}
/**
 * Sets the strings of the string/object pairs.
 *
 * @param strings
 */
public void setStrings(String[] strings) {
	_vectorStrings = convertArrayToVector(strings);
}
}
